package jsetl.test;

import java.util.ArrayList;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import jsetl.*;

import junit.framework.TestCase;

public class BoolLVarTest extends TestCase {
	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	@Test
	public void testAnd01() {
		// z = x /\ false.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", false);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.and(y)));
		solver.add(y.neq(true));
		assertTrue(solver.check());
		assertNull(x.getValue());
		assertFalse(y.getValue());
		assertEquals(z, y);
	}
	
	@Test
	public void testAnd02() {
		// z = false /\ y.
		Solver solver = new Solver();
		boolean b = false;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar();
		BoolLVar z = new BoolLVar("z", x);
		assertFalse(y.setName("y").equals(z));
		assertFalse(z.equals(z.clone()));
		solver.add(z.eq(x.and(y)));
		solver.add(y.neq(x));
		assertTrue(solver.check());
		assertTrue(y.isTrue());
		assertFalse(x.getValue());
	}
	
	@Test
	public void testAnd03() {
		// true = x /\ y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", x);
		BoolLVar z = new BoolLVar("z", true);
		solver.add(z.eq(x.and(y)));
		assertTrue(solver.check());
		assertTrue(z.isTrue());
		assertTrue(y.getValue());
		assertTrue(x.getValue()); 
	}
	
	@Test
	public void testAnd04() {
		// z = false /\ true.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", z);
		solver.add(x.neq(y));
		solver.add(z.eq(x.and(y)));
		assertTrue(solver.check());
		assertTrue(z.isFalse());
		assertFalse(y.isFalse());
		assertFalse(x.getValue());
	}
	
	@Test
	public void testAnd05() {
		// z = z /\ y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", z);
		solver.add(z.eq(x.and(y)));
		assertTrue(solver.check());
		assertEquals(x, z);
		assertTrue(y.isTrue());
	}
	
	@Test
	public void testAnd06() {
		// y = x /\ y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z", y);
		BoolLVar x = new BoolLVar("x");
		solver.add(z.eq(x.and(y)));
		assertTrue(solver.check());
		assertEquals(y, z);
		assertTrue(x.isTrue());
	}
	
	@Test
	public void testAnd07() {
		// z = y /\ y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", y);
		solver.add(z.eq(x.and(y)));
		assertTrue(solver.check());
		assertEquals(x, y);
		assertEquals(y, z);
		assertEquals(z, x);
	}
	
	@Test
	public void testOr01() {
		// z = x \/ false.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", false);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.or(y)));
		solver.add(y.neq(true));
		assertTrue(solver.check());
		assertNull(x.getValue());
		assertFalse(y.getValue());
		assertEquals(z, x);
	}
	
	@Test
	public void testOr02() {
		// z = false \/ y.
		Solver solver = new Solver();
		boolean b = false;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar();
		BoolLVar z = new BoolLVar("z", x);
		assertFalse(y.setName("y").equals(z));
		assertFalse(z.equals(z.clone()));
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertTrue(x.isFalse());
		assertEquals(y, z);
	}
	
	@Test
	public void testOr03() {
		// false = x \/ y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", x);
		BoolLVar z = new BoolLVar("z", false);
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertTrue(z.isFalse());
		assertFalse(y.getValue());
		assertFalse(x.getValue()); 
	}
	
	@Test
	public void testOr04() {
		// z = false \/ true.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z", y);
		BoolLVar x = new BoolLVar("x");
		solver.add(x.neq(y));
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertTrue(z.isTrue());
		assertFalse(y.isFalse());
		assertFalse(x.getValue());
	}
	
	@Test
	public void testOr05() {
		// z = x \/ true.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertNull(x.getValue());
		assertTrue(z.getValue());
		assertEquals(z, y);
	}
	
	@Test
	public void testOr06() {
		// z = true \/ y.
		Solver solver = new Solver();
		boolean b = true;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z", x);
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertTrue(z.isTrue());
		assertEquals(z, x);
	}
	
	@Test
	public void testOr07() {
		// z = false \/ y.
		Solver solver = new Solver();
		boolean b = false;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z", y);
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertFalse(z.isTrue());
		assertEquals(z, y);
	}
	
	@Test
	public void testOr08() {
		// z = y /\ y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", y);
		solver.add(z.eq(x.or(y)));
		assertTrue(solver.check());
		assertEquals(x, y);
		assertEquals(y, z);
		assertEquals(z, x);
	}
	
	@Test
	public void testImplies01() {
		// z = x ==> false.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", false);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertFalse(x.equals(z));
		assertNull(x.getValue());
		assertNull(z.getValue());
	}
	
	@Test
	public void testImplies02() {
		// z = false ==> y.
		Solver solver = new Solver();
		boolean b = false;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertTrue(z.getValue());
		assertNull(y.getValue());
	}
	
	@Test
	public void testImplies03() {
		// false = x ==> y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z", false);
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertTrue(z.isFalse());
		assertFalse(y.getValue());
		assertTrue(x.getValue()); 
	}
	
	@Test
	public void testImplies04() {
		// z = false ==> true.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", false);
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertTrue(z.isTrue());
		assertFalse(y.isFalse());
		assertFalse(x.getValue());
	}
	
	@Test
	public void testImplies05() {
		// z = x ==> true.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertNull(x.getValue());
		assertTrue(z.getValue());
		assertEquals(z, y);
	}
	
	@Test
	public void testImplies06() {
		// z = true ==> y.
		Solver solver = new Solver();
		boolean b = true;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar z = new BoolLVar("z");
		BoolLVar y = new BoolLVar("y", z);
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertNull(z.getValue());
		assertNull(y.getValue());
		assertEquals(z, y);
	}

	@Test
	public void testImplies07() {
		// z = z ==> y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar z = new BoolLVar("z", x);
		BoolLVar y = new BoolLVar("y");
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertTrue(z.getValue());
		assertTrue(y.getValue());
		assertTrue(x.getValue());
	}
	
	@Test
	public void testImplies08() {
		// z = y ==> y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar x = new BoolLVar("x", y);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.implies(y)));
		assertTrue(solver.check());
		assertTrue(z.getValue());
		assertNull(x.getValue());
		assertNull(y.getValue());
	}

	@Test
	public void testIff01() {
		// z = x <==> false.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", false);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertFalse(x.equals(y));
		assertNull(x.getValue());
		assertNull(z.getValue());
	}
	
	@Test
	public void testIff02() {
		// z = false <==> y.
		Solver solver = new Solver();
		boolean b = false;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertFalse(z.equals(y));
		assertNull(y.getValue());
		assertNull(z.getValue());
	}
	
	@Test
	public void testIff03() {
		// false = x <==> y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z", false);
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertFalse(y.equals(x));
		assertNull(y.getValue());
		assertNull(x.getValue());
	}
	
	@Test
	public void testIff04() {
		// z = false <==> true.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		BoolLVar x = new BoolLVar("x", false);
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertTrue(z.isFalse());
		assertFalse(y.isFalse());
		assertFalse(x.getValue());
	}
	
	@Test
	public void testIff05() {
		// z = x <==> true.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", true);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertNull(x.getValue());
		assertNull(z.getValue());
		assertEquals(z, x);
	}
	
	@Test
	public void testIff06() {
		// z = true <==> y.
		Solver solver = new Solver();
		boolean b = true;
		BoolLVar x = new BoolLVar("x");
		solver.add(x.eq(b));
		BoolLVar z = new BoolLVar("z");
		BoolLVar y = new BoolLVar("y", z);
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertNull(z.getValue());
		assertNull(y.getValue());
		assertEquals(z, y);
	}

	@Test
	public void testIff07() {
		// z = z <==> y.
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar z = new BoolLVar("z", x);
		BoolLVar y = new BoolLVar("y");
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertNull(z.getValue());
		assertTrue(y.getValue());
		assertNull(x.getValue());
	}
	
	@Test
	public void testIff08() {
		// z = y <==> y.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar x = new BoolLVar("x", y);
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertTrue(z.getValue());
		assertNull(x.getValue());
		assertNull(y.getValue());
	}
	
	@Test
	public void testIff09() {
		// z = x <==> z.
		Solver solver = new Solver();
		BoolLVar y = new BoolLVar("y");
		BoolLVar x = new BoolLVar("x");
		BoolLVar z = new BoolLVar("z", y);
		solver.add(z.eq(x.iff(y)));
		assertTrue(solver.check());
		assertTrue(x.getValue());
		assertNull(y.getValue());
		assertNull(z.getValue());
	}
	
	@Test
	public void testNot01() {
		// x = not(x).
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", x);
		solver.add(y.eq(x.not()));
		assertFalse(solver.check());
	}
	
	@Test
	public void testNot02() {
		// y = not(true).
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x", true);
		BoolLVar z = new BoolLVar(x);
		BoolLVar y = new BoolLVar("y");
		solver.add(y.eq(z.not()));
		assertTrue(solver.check());
		assertTrue(x.getValue());
		assertFalse(y.getValue());
	}
	
	@Test
	public void testNot03() {
		// false = not(x).
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y", false);
		solver.add(y.eq(x.not()));
		assertTrue(solver.check());
		assertTrue(x.getValue());
		assertFalse(y.getValue());
	}
	
	@Test
	public void testAndTruthTable() {
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.and(y)));
		solver.add(x.label().and(y.label()));
		assertTrue(solver.check());
		
		// false /\ false = false.
		assertFalse(x.getValue());
		assertFalse(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// false /\ true = false.
		assertFalse(x.getValue());
		assertTrue(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true /\ false = false.
		assertTrue(x.getValue());
		assertFalse(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true /\ true = true.
		assertTrue(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertFalse(solver.nextSolution());
	}
	
	@Test
	public void testOrTruthTable() {
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.or(y)));
		BoolLVar[] b = {x, y};
		solver.add(BoolLVar.label(b));
		assertTrue(solver.check());
		
		// false \/ false = false.
		assertFalse(x.getValue());
		assertFalse(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// false \/ true = true.
		assertFalse(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true \/ false = false.
		assertTrue(x.getValue());
		assertFalse(y.getValue());
		assertTrue(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true \/ true = true.
		assertTrue(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertFalse(solver.nextSolution());
	}
	
	@Test
	public void testImpliesTruthTable() {
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.implies(y)));
		ArrayList<BoolLVar> b = new ArrayList<BoolLVar>();
		b.add(x);
		b.add(y);
		solver.add(BoolLVar.label(b));
		assertTrue(solver.check());
		
		// false ==> false = true.
		assertFalse(x.getValue());
		assertFalse(y.getValue());
		assertTrue(z.getValue());
		assertTrue(solver.nextSolution());
		
		// false ==> true = true.
		assertFalse(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true ==> false = false.
		assertTrue(x.getValue());
		assertFalse(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true ==> true = true.
		assertTrue(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertFalse(solver.nextSolution());
	}
	
	@Test
	public void testIffTruthTable() {
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add(z.eq(x.iff(y)));
		solver.add(x.label(BoolHeuristic.FALSE).and(y.label()));
		assertTrue(solver.check());
		
		// false <==> false = true.
		assertFalse(x.getValue());
		assertFalse(y.getValue());
		assertTrue(z.getValue());
		assertTrue(solver.nextSolution());
		
		// false <==> true = false.
		assertFalse(x.getValue());
		assertTrue(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true <==> false = false.
		assertTrue(x.getValue());
		assertFalse(y.getValue());
		assertFalse(z.getValue());
		assertTrue(solver.nextSolution());
		
		// true <==> true = true.
		assertTrue(x.getValue());
		assertTrue(y.getValue());
		assertTrue(z.getValue());
		assertFalse(solver.nextSolution());
	}
	
	@Test
	public void testNotTruthTable() {
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		solver.add(y.eq(x.not()));
		solver.add(x.label(BoolHeuristic.TRUE));
		assertTrue(solver.check());
		
		// false = not(true).
		assertTrue(x.getValue());
		assertFalse(y.getValue());
		assertTrue(solver.nextSolution());
		
		// true =  not(false).
		assertFalse(x.getValue());
		assertTrue(y.getValue());
		assertFalse(solver.nextSolution());
	}
	
	@Test
	public void testDeMorgan1() {
		// (not(x /\ y) <==> not(x) \/ not(y)) = true
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add((x.and(y).not().iff(x.not().or(y.not()))).eq(z));
		LabelingOptions lop = new LabelingOptions();
		lop.valueForBoolLVar = BoolHeuristic.RANDOM;
		solver.add(x.label(lop).and(y.label(lop)));
		assertTrue(solver.check());
		int i = 0;
		do {
			++i;
			assertTrue(z.getValue());
		}
		while (solver.nextSolution());
		assertEquals(i, 4);
	}
	
	@Test
	public void testDeMorgan2() {
		// (not(x \/ y) <==> not(x) /\ not(y)) = true
		Solver solver = new Solver();
		BoolLVar x = new BoolLVar("x");
		BoolLVar y = new BoolLVar("y");
		BoolLVar z = new BoolLVar("z");
		solver.add((x.or(y).not().iff(x.not().and(y.not()))).eq(z));
		LabelingOptions lop = new LabelingOptions();
		lop.valueForBoolLVar = BoolHeuristic.TRUE;
		solver.add(x.label(lop).and(y.label(lop)));
		assertTrue(solver.check());
		int i = 0;
		do {
			++i;
			assertTrue(z.getValue());
		}
		while (solver.nextSolution());
		assertEquals(i, 4);
	}
}