package jsetl.test;

import jsetl.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import junit.framework.TestCase;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//     Testing the Constraint class                       %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

public class ConstraintClassTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	@Test
	public void
	testConstructors_toStringInternals() {
		LSet s1 = new LSet("s1");
		LSet s2 = new LSet("s2");			
		Constraint c1 = new Constraint("_subset",s1,s2);
//		System.out.println(c1.toStringInternals());
		assertEquals(c1.toStringInternals(), 
		  "constraint(subset, _s1, _s2, null, null) - alternative: 0, solved?: false");

		LSet s3 = new LSet("s3");			
		Constraint c2 = new Constraint("_union",s1,s2,s3);
//		System.out.println(c2.toStringInternals());
		assertEquals(c2.toStringInternals(), 
		  "constraint(union, _s1, _s2, _s3, null) - alternative: 0, solved?: false");

		Constraint c3 = new Constraint("_and",c1,c2);
//		System.out.println(c3.toStringInternals());
		assertEquals(c3.toStringInternals(), 
		  "constraint(subset, _s1, _s2, null, null) - alternative: 0, solved?: false AND constraint(union, _s1, _s2, _s3, null) - alternative: 0, solved?: false");

		Constraint c4 = new Constraint("_or",c1,c2);
//		System.out.println(c4.toStringInternals());
		assertEquals("constraint(or, subset(_s1,_s2), union(_s1,_s2,_s3), null, null) - alternative: 0, solved?: false",
				c4.toStringInternals());

		Constraint c5 = new Constraint("myConstraint",s1);		
//		System.out.println(c5.toStringInternals());
		assertEquals(c5.toStringInternals(), 
		  "constraint(myConstraint, _s1, null, null, null) - alternative: 0, solved?: false");
	}

	@Test
	public void
	testEquals() {
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2",5);
		Constraint c1 = new Constraint("_neq",x1,x2);
		Constraint c2 = new Constraint("_neq",x1,5);
		assertTrue(c1.equals(c2));
		
		Constraint c3 = new Constraint("_neq",5,x1);
		assertFalse(c1.equals(c3));

		Constraint c4 = new Constraint("_and",c1,c3);
		assertFalse(c1.equals(c4));

		assertFalse(c4.equals(c1));

		Constraint c5 = new Constraint("_and",c2,c3);
		assertTrue(c4.equals(c5));

		assertTrue(c5.equals(c4));
	}

	@Test
	public void
	testGetName_toString() {		
        IntLVar x = new IntLVar("x");
        Constraint c0 = new Constraint();
		assertEquals(c0.getName() + ": " + c0, "no name: []");
      
        Constraint c1 = new Constraint("_label",x);
		assertEquals(c1.getName() + ": " + c1, "label: _x.label()");

        Constraint c2 = new Constraint("_neq",x,1);
		assertEquals(c2.getName() + ": " + c2, "neq: _x neq 1");
   
        Constraint c3 = new Constraint("_sum",x,1,2);
		assertEquals(c3.getName() + ": " + c3, "sum: _x = 1 + 2");
       
//constraint conjunction        
        Constraint cc1 = new Constraint("_and",c2,c3);
		assertEquals(cc1.getName() + ": " + cc1, "and: _x neq 1 AND _x = 1 + 2");
		         
        Constraint cc2 = new Constraint("_and",cc1,c1);
		assertEquals(cc2.getName() + ": " + cc2, "and: _x neq 1 AND _x = 1 + 2 AND _x.label()");
        
        Constraint cc2bis = new Constraint("_and",c1,cc1);
		assertEquals(cc2bis.getName() + ": " + cc2bis, "and: _x.label() AND _x neq 1 AND _x = 1 + 2");
        
//constraint disjunction  
        Constraint cd1 = new Constraint("_or",c2,c3);
		assertEquals(cd1.getName() + ": " + cd1, "or: _x neq 1 OR _x = 1 + 2");
        
        Constraint cd2 = new Constraint("_or",c2,cc2);
//        System.out.print(cd2.getName() + ": "); 
//        System.out.println(cd2);
		assertEquals(cd2.getName() + ": " + cd2, "or: _x neq 1 OR _x neq 1 AND _x = 1 + 2 AND _x.label()");
	}

	@Test
	public void
	testGround() {	
	        IntLVar x = new IntLVar("x");
	        IntLVar y = new IntLVar("y");
	    	Constraint c0 = new Constraint();
	        assertTrue(c0.isGround());
	        Constraint c1 = new Constraint("_label",y);
	        assertFalse(c1.isGround());
	        Constraint c2 = new Constraint("_neq",x,1);
	        assertFalse(c2.isGround());
	        Constraint c3 = new Constraint("_sum",x,1,2);
	        assertFalse(c3.isGround());
	//constraint conjunction        
	        Constraint cc1 = new Constraint("_and",c2,c3);
	        assertFalse(cc1.isGround());
	        Constraint cc2 = new Constraint("_and",cc1,c1);
	        assertFalse(cc2.isGround());
	        Constraint cc2bis = new Constraint("_and",c1,cc1);
	        assertFalse(cc2bis.isGround());
	//constraint disjunction  
	        Constraint cd1 = new Constraint("_or",c2,c3);
	        assertFalse(cd1.isGround());
	        Constraint cd2 = new Constraint("_or",c2,cc2);
	        assertFalse(cd2.isGround());
	        
	        Solver auxSolver = new Solver();
	        auxSolver.check(x.eq(1));
	        
	        assertTrue(c0.isGround());
	        assertFalse(c1.isGround());
	        assertTrue(c2.isGround());
	        assertTrue(c3.isGround());
	//constraint conjunction        
	        assertTrue(cc1.isGround());
	        assertFalse(cc2.isGround());
	        assertFalse(cc2bis.isGround());
	//constraint disjunction  
	        assertTrue(cd1.isGround());
	        assertFalse(cd2.isGround());
	        
            auxSolver.check(y.eq(2));
	        
	        assertTrue(c0.isGround());
	        assertTrue(c1.isGround());
	        assertTrue(c2.isGround());
	        assertTrue(c3.isGround());
	//constraint conjunction        
	        assertTrue(cc1.isGround());
	        assertTrue(cc2.isGround());
	        assertTrue(cc2bis.isGround());
	//constraint disjunction  
	        assertTrue(cd1.isGround());
	        assertTrue(cd2.isGround());
	}

	@Test
	public void
	testGetArg() {		
        IntLVar x = new IntLVar("x");
         Constraint c0 = new Constraint();
        assertEquals(c0.getArg(0),null);
        assertEquals(c0.getArg(1),null);
      
        Constraint c1 = new Constraint("_label",x);
        assertEquals(c1.getArg(0),null);
        assertEquals(c1.getArg(1),x);
        assertEquals(c1.getArg(2),null);
        
        Constraint c2 = new Constraint("_neq",x,1);
        assertEquals(c2.getArg(1),x);
        assertEquals(c2.getArg(2),1);
        assertEquals(c2.getArg(3),null);
		
        Constraint c3 = new Constraint("_sum",x,1,2);
        assertEquals(c3.getArg(1),x);
        assertEquals(c3.getArg(2),1);
        assertEquals(c3.getArg(3),2);
        assertEquals(c3.getArg(4),null);
       
//constraint conjunction        
        Constraint cc1 = new Constraint("_and",c2,c3);
        assertEquals(cc1.getArg(1).toString(),"_x neq 1");
        assertEquals(cc1.getArg(2).toString(),"_x = 1 + 2");
        assertEquals(cc1.getArg(3),null);

        Constraint cc2 = new Constraint("_and",cc1,c1);
        assertEquals(cc2.getArg(1).toString(),"_x neq 1");
        assertEquals(cc2.getArg(2).toString(),"_x = 1 + 2 AND _x.label()");
        assertEquals(cc2.getArg(3),null);
       
        Constraint cc2bis = new Constraint("_and",c1,cc1);
        assertEquals(cc2bis.getArg(1).toString(),"_x.label()");
        assertEquals(cc2bis.getArg(2).toString(),"_x neq 1 AND _x = 1 + 2");
        assertEquals(cc2bis.getArg(3),null);
       
//constraint disjunction  
        Constraint cd1 = new Constraint("_or",c2,c3);
        assertEquals(cd1.getArg(1).toString(),"_x neq 1");
        assertEquals(cd1.getArg(2).toString(),"_x = 1 + 2");
        assertEquals(cd1.getArg(3),null);
        
        Constraint cd2 = new Constraint("_or",c2,cc2);
        assertEquals(cd2.getArg(1).toString(),"_x neq 1");
        assertEquals(cd2.getArg(2).toString(),"_x neq 1 AND _x = 1 + 2 AND _x.label()");
        assertEquals(cd2.getArg(3),null);
      }

	@Test
	public void
	testAnd_Or_Not_Implies() {
		LSet s1 = new LSet("s1");
		LSet s2 = new LSet("s2");			
		Constraint c1 = new Constraint("_subset",s1,s2);

		LSet s3 = new LSet("s3");			
		Constraint c2 = new Constraint("_union",s1,s2,s3);

		Constraint c3 = new Constraint("_and",c1,c2);
		assertEquals("subset(_s1,_s2) AND union(_s1,_s2,_s3)", c3.toString());

		Constraint c4 = new Constraint("_or",c1,c2);
		assertEquals("subset(_s1,_s2) OR union(_s1,_s2,_s3)", c4.toString());

		Constraint c5 = new Constraint("_orTest",c1,c2);
		assertEquals("subset(_s1,_s2) OR union(_s1,_s2,_s3)", c5.toString());

		Constraint c6 = new Constraint("_impliesTest",c1,c2);
		assertEquals("subset(_s1,_s2) => union(_s1,_s2,_s3)", c6.toString());

		Constraint c7 = new Constraint("_notTest",c1);
		assertEquals("NOT(subset(_s1,_s2))", c7.toString());

		Constraint c8 = new Constraint("myConstraint",s1,s2);		
		assertEquals("myConstraint(_s1,_s2)", c8.toString());

		}

}
