package jsetl.test;

//import static org.junit.Assert.*;

//import java.io.IOException;

import java.util.ArrayList;
//import java.util.EmptyStackException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import jsetl.exception.Failure;
import jsetl.exception.NotDefConstraintException;
import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.*;


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//     Testing Inequality, Membership, and not Memwebrship Constraints    %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public class ConstraintNeqInNinTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//                      Using LVar                        %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
//%%%%%%%%%%%%%%%%%%%%%%% neq %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// LVAR NON INIZIALIZZATA ---> INTERO
// x neq 1
@Test
	public void testNEQLVarUninitializedx_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x1");
	     solver.solve(x.neq(1));
	     c = solver.getConstraint();
	     assertEquals("_x1 neq 1",c.toString());
	}

// LVAR NON INIZIALIZZATA x ---> LVAR NON INIZIALIZZATA x
// x neq x
@Test
	public void testNEQLVarUninitializedx_LVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
		 LVar x = new LVar("x1");
	     try{
		   	solver.solve(x.neq(x));
		    fail("FALLIMENTO!!!!");
		 }
		  catch(Failure e) {}
	
	}
	
// LVAR NON INIZIALIZZATA x ---> LVAR NON INIZIALIZZATA y
// x neq y - solved form
@Test
	public void testNEQLVarUninitializedx_LVarUninitializedy() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     LVar y = new LVar("y");
	     solver.solve(x.neq(y));
	     c = solver.getConstraint();
	     assertEquals("_x neq _y",c.toString());	     	
	}
	
// LVAR NON INIZIALIZZATA x ---> LVAR INIZIALIZZATA CON INTERO
// y = 2, x neq y
@Test
	public void testNEQLVarUninitializedx_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar("x1");
	    LVar y = new LVar(2);
	    solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("_x1 neq 2",c.toString());
	}
	
	// LVAR NON INIZIALIZZATA  ---> LVAR INIZIALIZZATA CON LLIST INIZIALIZZATA
	@Test
	public void testNEQLVarUninitialized_LVarInitializedLListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar("x1");
	    LList y1 = new LList("y1");
		y1.ins(1).ins(2);
		LVar y= new LVar(y1);
		solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    assertEquals("_x1 neq _y1",c.toString());
	}
	
	// LVAR INIZIALIZZATA CON INTERO ---> INTERO
	@Test
	public void testNEQLVarinitializedWithInteger_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar(8);
	     solver.solve(x.neq(1));
		 c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
	 
	// LVAR INIZIALIZZATA CON INTERO ---> LVAR NON INIZIALIZZATA 
	@Test
	public void testNEQLVarinitializedWithInteger_LVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar(8);
	     LVar y = new LVar("y");
	     solver.solve(x.neq(y));
		 c = solver.getConstraint();
		 assertEquals("8 neq _y",c.toString());
	}
	
// LVAR INIZIALIZZATA CON INTERO ---> LVAR INIZIALIZZATA CON INTERO
// x = 8, y = 2, x neq y
@Test
	public void testNEQLVarinitializedWithInteger_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar(8);
	    LVar y = new LVar(2);
	   	solver.solve(x.neq(y));
		c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}

//LVAR INIZIALIZZATA CON INTERO ---> LVAR INIZIALIZZATA CON INTERO
//x = 2, y = 2, x neq y
@Test
	public void testNEQLVarInitWithInteger_LVarInitWithInteger_False()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LVar x = new LVar(2);
	    LVar y = new LVar(2);
	    try{
	       	solver.solve(x.neq(y));
			fail("FAILURE!!!!");
		 }
	    catch(Failure e) {}
	}

	// LVAR INIZIALIZZATA CON INTERO ---> LVAR INIZIALIZZATA CON LLIST INIZIALIZZATA 
	@Test
	public void testNEQLVarinitializedWithInteger_LVarInitializedLListInitialized()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar(8);
	    LList y1 = new LList();
		y1.ins(1).ins(2);
		LVar y= new LVar(y1);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
			
	// LVAR INIZIALIZZATA LLIST INIZ ---> INTERO
	@Test
	public void testNEQLVarInitializedWithLListInit_Integer() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LList x1 = new LList();
		 x1.ins(1).ins(2);
		 LVar x= new LVar(x1);
	     solver.solve(x.neq(1));
		 c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
	// LVAR INIZIALIZZATA LLIST INIZ  ---> LVAR NON INIZIALIZZATA x
	@Test
	public void testNEQLVarInitializedWithLListInit_LVarUninitializedx() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LList x1 = new LList("x1");
		 x1.ins(1).ins(2);
		 LVar x= new LVar("x",x1);
		 LVar y = new LVar("y");
	     solver.solve(x.neq(y));
		 c = solver.getConstraint();
		 assertEquals("_x1 neq _y",c.toString());	
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ CON INTERO ---> LVAR INIZIALIZZATA CON INTERO
	@Test
	public void testNEQLVarInitializedWithLListInitWithInteger_LVarInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LList x1 = new LList();
		x1.ins(1).ins(2);
		LVar x= new LVar(x1);
	    LVar y = new LVar(2);
	    solver.solve(x.neq(y));
		c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ CON INTERO ---> LVAR INIZIALIZZATA CON LLIST INIZIALIZZATA INTERO
	@Test
	public void testNEQLVarInitializedWithLListInitWithLInteger_LVarInitializedLListInitializedWithInteger()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LList x1 = new LList();
		x1.ins(1).ins(2);
		LVar x= new LVar(x1);
	    LList y1 = new LList();
		y1.ins(1).ins(2);
		LVar y= new LVar(y1);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_? neq _?",c.toString());
	}
	
//%%%%%%%%%%%%%%%%%%%%%%% in %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	// LVAR NON INIZIALIZZATA ---> LSET NON INIZ
	@Test
	public void testINLVarUninitializedx_LSetUninizialized() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x1");
	     LSet y = new LSet();
	     solver.solve(x.in(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	// LVAR NON INIZIALIZZATA  ---> LSET INIZ
	@Test
	public void testINLVarUninitializedx_LSetIninitialized() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x1");
	     LSet y = LSet.empty().ins(2).ins(3);
	     solver.solve(x.in(y));
		 c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
	
	// LVAR NON INIZIALIZZATA ---> SET DI JAVA
	@Test
	public void testINLVarUninitializedx_SetJava()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LVar x = new LVar("x1");
	    Set<Integer> y = new HashSet<Integer>();
	    try{
	       	solver.solve(x.in(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
		
	// LVAR INIZIALIZZATA CON INTERO ---> LSET NON INIZ
	@Test
	public void testINLVarinitializedWithInteger_LSetUninitialized() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar(8);
	     LSet y = new LSet();
	     solver.solve(x.in(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}
	// LVAR INIZIALIZZATA CON INTERO  ---> LSET INIZ
	@Test
	public void testINLVarinitializedWithInteger_LSetInizialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LVar x = new LVar(8);
	     LSet y = LSet.empty().ins(3).ins(4);
	     try{
		   	solver.solve(x.in(y));
			fail("FALLIMENTO!!!!");
		 }
		 catch(Failure e) {}
	}
	
	// LVAR INIZIALIZZATA CON INTERO ---> SET DI JAVA
	@Test
	public void testINLVarinitializedWithInteger_SetJava()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LVar x = new LVar(8);
	    Set<Integer> y = new HashSet<Integer>();
	    try{
	       	solver.solve(x.in(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ  ---> LSET NON INIZ
	@Test
	public void testINLVarInitializedWithLListInitialized_LSetUninitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LList x1 = new LList();
		 x1.ins(1).ins(2);
		 LVar x= new LVar(x1);
	     LSet y = new LSet();
	     solver.solve(x.in(y));
	     c = solver.getConstraint();
	     assertEquals("[]",c.toString());
	}

	// LVAR INIZIALIZZATA LLIST INIZ  ---> LSET INIZ
	@Test
	public void testINLVarInitializedWithLListInitialized_LSetInitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LList x1 = new LList();
		 x1.ins(1).ins(2);
		 LVar x= new LVar(x1);
	     LSet y = LSet.empty().ins(3).ins(4);
	     try{
		   	solver.solve(x.in(y));
			fail("FALLIMENTO!!!!");
			 }
	     catch(Failure e) {}
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ  ---> SET DI JAVA
	@Test
	public void testINLVarInitializedWithLListInitialized_SetJava()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LList x1 = new LList();
		x1.ins(1).ins(2);
		LVar x= new LVar(x1);
	    Set<Integer> y = new HashSet<Integer>();
	    try{
	       	solver.solve(x.in(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	
	
//%%%%%%%%%%%%%%%%%%%%%%% nin %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
	// LVAR NON INIZIALIZZATA ---> LSET NON INIZ
	@Test
	public void testNINLVarUninitialized_LSetUninitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     LSet y = new LSet("y");
	     solver.solve(x.nin(y));
	     c = solver.getConstraint();
	     assertEquals("_x nin _y",c.toString());
	}
	// LVAR NON INIZIALIZZATA  ---> LSET INIZ
	@Test
	public void testNINLVarUninitialized_LSetInitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar("x");
	     LSet y =LSet.empty().ins(3).ins(8);
	     solver.solve(x.nin(y));
		 c = solver.getConstraint();
		 assertEquals("_x neq 8 AND _x neq 3",c.toString());
	}
	
	// LVAR NON INIZIALIZZATA ---> SET DI JAVA
	@Test
	public void testNINLVarUninitializedx_SetJava()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar("x1");
	    Set<Integer> y = new HashSet<Integer>();
	    solver.solve(x.nin(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
		
	// LVAR INIZIALIZZATA CON INTERO ---> LSET NON INIZ
	@Test
	public void testNINLVarinitializedWithInteger_LSetUninitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LVar x = new LVar(8);
	     LSet y = new LSet("y");
	     solver.solve(x.nin(y));
	     c = solver.getConstraint();
	     assertEquals("8 nin _y",c.toString());
	}
	// LVAR INIZIALIZZATA CON INTERO  ---> LSET INIZ
	@Test
	public void testNINLVarinitializedWithInteger_LSetInitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LVar x = new LVar(8);
	     LSet y =LSet.empty().ins(3).ins(8);
	     try{
		       	solver.solve(x.nin(y));
			    fail("FALLIMENTO!!!!");
			 }
		    catch(Failure e) {}
	}
	
	// LVAR INIZIALIZZATA CON INTERO ---> SET DI JAVA
	@Test
	public void testNINLVarinitializedWithInteger_SetJava()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LVar x = new LVar(8);
	    Set<Integer> y = new HashSet<Integer>();
	    solver.solve(x.nin(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ  ---> LSET NON INIZ
	@Test
	public void testNINLVarInitializedWithLListInitialized_LSetUninitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LList x1 = new LList("x1");
		 x1.ins(1).ins(2);
		 LVar x= new LVar("x",x1);
	     LSet y = new LSet("y");
	     solver.solve(x.nin(y));
	     c = solver.getConstraint();
	     assertEquals("_x1 nin _y",c.toString());
	}
	// LVAR INIZIALIZZATA LLIST INIZ ---> LSET INIZ
	@Test
	public void testNINLVarInitializedWithLListInitialized_LSetInitialized() throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Constraint c;
	     LList x1 = new LList();
		 x1.ins(1).ins(2);
		 LVar x= new LVar(x1);
	     LSet y =LSet.empty().ins(3).ins(8);
	     solver.solve(x.nin(y));
		 c = solver.getConstraint();
		 assertEquals("[]",c.toString());
	}
	
	// LVAR INIZIALIZZATA LLIST INIZ ---> SET DI JAVA
	@Test
	public void testNINLVarInitializedWithLListInitialized_SetJava()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    Constraint c;
	    LList x1 = new LList();
		x1.ins(1).ins(2);
		LVar x= new LVar(x1);
	    Set<Integer> y = new HashSet<Integer>();
	    solver.solve(x.nin(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//	                     Using LSet                       %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//%%%%%%%%%%%%%%%%%%%%%%% neq %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	// LSet NON INIZIALIZZATA x ---> LSet NON INIZIALIZZATA x
	@Test
	public void testNEQLSetUninitializedx_LSetUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
 		LSet x = new LSet("x1");
 		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	// LSet NON INIZIALIZZATA x ---> LSet NON INIZIALIZZATA y
	@Test
	public void testNEQLSetUninitializedx_LSetUninitializedy() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x neq _y",c.toString());
	}
	
	// LSet NON INIZIALIZZATA x ---> LSET INIZIALIZZATO
	@Test
	public void testNEQLSetUninitializedx_LSetInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x neq {2,3}",c.toString());
	}

	// LSet NON INIZIALIZZATA x ---> SET INIZIALIZZATO
	@Test
	public void testNEQLSetUninitializedx_Set()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x neq [2, 3]",c.toString());
	}

	// LSet NON INIZIALIZZATA x ---> LSET CON RESTO
	@Test
	public void testNEQLSetUninitializedx_LSetWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		
		LSet y = x.ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("2 nin _x",c.toString());		
	}
		
	// LSet NON INIZIALIZZATA x ---> LSET INIZ CON LVAR NON INIZI
	@Test
	public void testNEQLSetUninitializedx_LSetInitializedWithLVarInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LSet y = LSet.empty().ins(x).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x neq {3,_x}",c.toString());
	}
		
	// LSet INIZIALIZZATO CON INTERO ---> LSet NON INIZIALIZZATA 
	@Test
	public void testNEQLSetInitializedWithInteger_LSetUninitialized() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LSet y = new LSet ("y");
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("{1} neq _y",c.toString());

	}
	
	// LSet INIZIALIZZATO CON INTERO ---> LSET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithInteger_LSetInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LSet NON INIZIALIZZATO CON INTERO ---> SET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithInteger_Set()  
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LSet INIZIALIZZATO CON INTERO ---> LSET CON RESTO
	@Test
	public void testNEQLSetInitializedWithInteger_LSetWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LSet y1 = new LSet("y1");
		LSet y = y1.ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("1 nin _y1",c.toString());
	}
	
	
	// LSet INIZIALIZZATA CON INTERO ---> LSET INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLSetInitilizadWithInteger_LSetInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(1);
		LVar y1 = new LVar("y1");
		LSet y = LSet.empty().ins(y1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("1 neq _y1",c.toString());
	}
	

	// LSet NON INIZIALIZZATA CON RESTO ---> LSet NON INIZIALIZZATA x
	@Test
	public void testNEQLSetInitializedWithRest_LSetUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet("s");
		LSet x = s.ins(1).setName("x");
		LSet y = new LSet("y");
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("{1/_s} neq _y",c.toString());
	}
	
	// LSet NON INIZIALIZZATA CON RESTO ---> LSET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithRest_LSetInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
		LSet x = s.ins(1);
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LSet NON INIZIALIZZATA CON RESTO ---> SET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithRest_Set()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet();
		LSet x = s.ins(1);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LSet NON INIZIALIZZATA CON RESTO ---> LSET CON RESTO
	@Test
	public void testNEQLSetInitializedWithRest_LSetWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet("s");
		LSet x = s.ins(1).setName("x");    // x = {1 | s}
		LSet y = s.ins(2).setName("y");    // y = {1 | s}
		solver.solve(x.neq(y));            // x neq y
		c = solver.getConstraint();
		assertEquals("1 nin _s",c.toString());
		solver.nextSolution();
		assertEquals("_? nin _s",c.toString());  // no!! BUG!
		assertFalse(solver.nextSolution()); // --> false (no other solutions)
//		solver.nextSolution();
//		assertEquals("_? nin {_?|_?}",c.toString());
	}
	
	
	// LSet NON INIZIALIZZATA CON RESTO ---> LSET INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLSetInitializedWithRest_LSetiNIZ()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet s = new LSet("s");
		LSet x = s.ins(1).setName("x");
		LVar y1 = new LVar("y1");
		LSet y = LSet.empty().ins(y1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("1 neq _y1",c.toString());
		solver.nextSolution();
		assertEquals("_? neq _y1",c.toString());
	}
		
	//LSet INIZIALIZZATA CON LVAR NON INIZ ---> LSet NON INIZIALIZZATA 
	@Test
	public void testNEQLSetInitializedWithLVarUninitialized_LSetUninitialized() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2");
		LSet x = LSet.empty().ins(x1).ins(x2);
		LSet y= new LSet("y");
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("{_x2,_x1} neq _y",c.toString());
	}
	
	//LSet INIZIALIZZATA CON LVAR NON INIZ ---> LSET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithLVarUninitialized_LSetInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2");
		LSet x = LSet.empty().ins(x1).ins(x2);
		LSet y = LSet.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x2 neq 2 AND _x2 neq 3",c.toString());		
	}

	//LSet INIZIALIZZATA CON LVAR NON INIZ ---> SET INIZIALIZZATO
	@Test
	public void testNEQLSetInitializedWithLVarUninitialized_Set()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2");
		LSet x = LSet.empty().ins(x1).ins(x2);
		Set<Integer> y = new HashSet<Integer>();
		y.add(3); y.add(2); 
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x2 neq 2 AND _x2 neq 3",c.toString());		
	}

	//LSet INIZIALIZZATA CON LVAR NON INIZ ---> LSET CON RESTO
	@Test
	public void testNEQLSetInitializedWithLVarUninitialized_LSetWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2");
		LSet x = LSet.empty().ins(x1).ins(x2);  // x = {x1,x2}
		LSet y = x.ins(2);                      // y = {2,x1,x2}
		solver.solve(x.neq(y));                 // x neq y
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("2 neq _x2 AND 2 neq _x1",c.toString());
		assertFalse(solver.nextSolution());     // --> false (no other solutions)	    
//	    solver.nextSolution();                   
//	    assertEquals("_x1 neq _x2 AND _x1 neq _x1",c.toString());
	}
		
	//LSet INIZIALIZZATA CON LVAR NON INIZ ---> LSET INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLSetInitializedWithLVarUninitialized_LSetInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar("x1");
		LVar x2 = new LVar("x2");
		LSet x = LSet.empty().ins(x1).ins(x2).setName("x");
		LSet y = LSet.empty().ins(x1).ins(3).setName("y");
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x2 neq 3 AND _x2 neq _x1",c.toString());
	}
					
//%%%%%%%%%%%%%%%%%%%%%%% contains %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	// LSET NON INIZIALIZZATO ---> LVAR  INIZIALIZZATA
	@Test
	public void testCONTAINSLSetUninitializedx_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet();
		LVar y = new LVar(3);
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET NON INIZIALIZZATO ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testCONTAINSLSetUninitializedx_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x1");
		LVar y = new LVar();
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	// LSet NON INIZIALIZZATA x ---> INTERO
	@Test
	public void testCONTAINSLSetUninitializedx_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		solver.solve(x.contains(3));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET INIZIALIZZATO ---> LVAR  INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitialized_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(2);
		LVar y = new LVar(3);
		try{
	       	solver.solve(x.contains(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}
	// LSET INIZIALIZZATO ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitialized_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(2);
		LVar y = new LVar();
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	// LSet INIZIALIZZATA x ---> INTERO
	@Test
	public void testCONTAINSLSetInitialized_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(2);
		try{
	       	solver.solve(x.contains(3));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

	// LSET INIZIALIZZATO CON RESTO---> LVAR  INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitializedWithRest_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet();
		LSet x = r.ins(2);
		LVar y = new LVar(3);
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET INIZIALIZZATO CON RESTO---> LVAR  NON INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitializedWithRest_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet();
		LSet x = r.ins(2);
		LVar y = new LVar();
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	// LSet INIZIALIZZATA CON RESTO ---> INTERO
	@Test
	public void testCONTAINSLSetInitializedWithRest_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet();
		LSet x = r.ins(2);
		solver.solve(x.contains(3));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET INIZIALIZZATO CON LVAR NON INIZ---> LVAR  INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitializedWithLvarUninitialized_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar();
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x0).ins(x1);
		LVar y = new LVar(3);
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET INIZIALIZZATO LVAR NON INIZ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testCONTAINSLSetInitializedWithLvarUninitialized_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar();
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x0).ins(x1);
		LVar y = new LVar();
		solver.solve(x.contains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSet INIZIALIZZATA LVAR NON INIZ ---> INTERO
	@Test
	public void testCONTAINSLSetInitializedWithLvarUninitialized_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar();
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x0).ins(x1);
		solver.solve(x.contains(3));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
//%%%%%%%%%%%%%%%%%%%%%%% ncontains %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	// LSET NON INIZIALIZZATO ---> LVAR  INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetUninitializedx_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		LVar y = new LVar("y",3);
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("3 nin _x",c.toString());
	}
	
	// LSET NON INIZIALIZZATO ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetUninitializedx_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x1");
		LVar y = new LVar("y");
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("_y nin _x1",c.toString());
	}
	
	// LSet NON INIZIALIZZATA x ---> INTERO
	@Test
	public void testNCONTAINSLSetUninitializedx_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = new LSet("x");
		solver.solve(x.ncontains(3));
		c = solver.getConstraint();
		assertEquals("3 nin _x",c.toString());
	}
	
	// LSET INIZIALIZZATO ---> LVAR  INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitialized_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(2);
		LVar y = new LVar(3);
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LSET INIZIALIZZATO ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitialized_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(2);
		LVar y = new LVar("x1");
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("_x1 neq 2",c.toString());
	}
	
	// LSet INIZIALIZZATA x ---> INTERO
	@Test
	public void testNCONTAINSLSetInitialized_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet x = LSet.empty().ins(2);
		solver.solve(x.ncontains(3));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LSET INIZIALIZZATO CON RESTO---> LVAR  INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitializedWithRest_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet("r");
		LSet x = r.ins(2);
		LVar y = new LVar(3);
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("3 nin _r",c.toString());
	}
	
	// LSET INIZIALIZZATO CON RESTO---> LVAR  NON INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitializedWithRest_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet("r");
		LSet x = r.ins(2);
		LVar y = new LVar("y");
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("_y nin _r AND _y neq 2",c.toString());
	}
	
	// LSet INIZIALIZZATA CON RESTO ---> INTERO
	@Test
	public void testNCONTAINSLSetInitializedWithRest_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LSet r = new LSet("r");
		LSet x = r.ins(2);
		solver.solve(x.ncontains(3));
		c = solver.getConstraint();
		assertEquals("3 nin _r",c.toString());
	}
	
	// LSET INIZIALIZZATO LVAR NON INIZ---> LVAR  INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitializedWithLvarUninitialized_LVarInitializedx()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1");
		LSet x = LSet.empty().ins(x0).ins(x1);
		LVar y = new LVar("y",3);
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("3 neq _x1 AND 3 neq _x0",c.toString());
	}
	
	// LSET INIZIALIZZATO LVAR NON INIZ---> LVAR  NON INIZIALIZZATA
	@Test
	public void testNCONTAINSLSetInitializedWithLvarUninitialized_LVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1");
		LSet x = LSet.empty().ins(x0).ins(x1);
		LVar y = new LVar("y");
		solver.solve(x.ncontains(y));
		c = solver.getConstraint();
		assertEquals("_y neq _x1 AND _y neq _x0",c.toString());
	}
	
	// LSet INIZIALIZZATA LVAR NON INIZ ---> INTERO
	@Test
	public void testNCONTAINSLSetInitializedWithLvarUninitialized_LSetUninitializedy()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1");
		LSet x = LSet.empty().ins(x0).ins(x1);
		solver.solve(x.ncontains(3));
		c = solver.getConstraint();
		assertEquals("3 neq _x1 AND 3 neq _x0",c.toString());
	}
	
	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//	                     Using LList                      %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
//%%%%%%%%%%%%%%%%%%%%%%% neq %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	// LList NON INIZIALIZZATA x ---> LList NON INIZIALIZZATA x
	@Test
	public void testNEQLListUninitializedx_LListUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
 		LList x = new LList("x1");
 		solver.solve(x.eq(x));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	// LList NON INIZIALIZZATA x ---> LList NON INIZIALIZZATA y
	@Test
	public void testNEQLListUninitializedx_LListUninitializedy() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		LList y = new LList();
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		
		assertEquals("_x1 neq _?", c.toString());
	}
	
	// LList NON INIZIALIZZATA x ---> LList INIZIALIZZATO
	@Test
	public void testNEQLListUninitializedx_LListInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x1 neq [2,3]", c.toString());
	}

	// LList NON INIZIALIZZATA x ---> LIST INIZIALIZZATO
	@Test
	public void testNEQLListUninitializedx_List()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x1 neq [1]", c.toString());
	}

	// LList NON INIZIALIZZATA x ---> LList CON RESTO
	@Test
	public void testNEQLListUninitializedx_LListWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");		
		LList y = x.ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
		
	// LList NON INIZIALIZZATA x ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLListUninitializedx_LListInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = new LList("x1");
		LVar y1 = new LVar();
		LList y = LList.empty().ins(y1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_x1 neq [3,_?]", c.toString());
	}
		
	// LList INIZIALIZZATO CON INTERO ---> LList NON INIZIALIZZATA x
	@Test
	public void testNEQLListInitializedWithInteger_LListUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		LList y = new LList ();
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[1] neq _?",c.toString());
	}
	
	// LList INIZIALIZZATO CON INTERO ---> LList INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithInteger_LListInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LList INIZIALIZZATO CON INTERO ---> LIST INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithInteger_List()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList x = LList.empty().ins(1);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1); 
	    try{
	       	solver.solve(x.neq(y));
		    fail("FALLIMENTO!!!!");
		 }
	    catch(Failure e) {}
	}

	// LList INIZIALIZZATO CON INTERO ---> LList CON RESTO
	@Test
	public void testNEQLListInitializedWithInteger_LListWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		LList y1= new LList();
		LList y = y1.ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
		
	// LList  INIZIALIZZATA CON INTERO ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLListInitilizadWithInteger_LListInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList x = LList.empty().ins(1);
		LVar y1 = new LVar();
		LList y = LList.empty().ins(y1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
	
	// LList INIZIALIZZATA CON RESTO ---> LList NON INIZIALIZZATA x
	@Test
	public void testNEQLListInitializedWithRest_LListUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		LList y = new LList();
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[1|_?] neq _?", c.toString());
	}
	
	// LList NON INIZIALIZZATA CON RESTO ---> LList INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithRest_LListInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

	// LList NON INIZIALIZZATA CON RESTO ---> LIST INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithRest_List()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LList s = new LList();
		LList x = s.ins(1);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
	    assertTrue(solver.check(x.eq(y)));
	    assertEquals("[]", s.toString());
	}

	// LList NON INIZIALIZZATA CON RESTO ---> LList CON RESTO
	@Test
	public void testNEQLListInitializedWithRest_LListWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		LList y = s.ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());		
	}
		
	// LList NON INIZIALIZZATA CON RESTO ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLListInitializedWithRest_LListInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LList s = new LList();
		LList x = s.ins(1);
		LVar y1= new LVar();
		LList y = LList.empty().ins(y1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}
		
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList NON INIZIALIZZATA x
	@Test
	public void testNEQLListInitializedWithLVarUninitialized_LListUninitializedx() throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1).ins(3);
		LList y= new LList();
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[3,_?] neq _?",c.toString());
	}
	
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithLVarUninitialized_LListInitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1).ins(2);
		LList y = LList.empty().ins(3).ins(2);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("_? neq 3",c.toString());
	}

	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LIST INIZIALIZZATO
	@Test
	public void testNEQLListInitializedWithLVarUninitialized_List()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1).ins(2);
		List<Integer> y = new ArrayList<Integer>();
	    y.add(1);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());		
	}

	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList CON RESTO
	@Test
	public void testNEQLListInitializedWithLVarUninitialized_LListWithRest()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1).ins(2);
		LList y1 = new LList();
		LList y = y1.ins(2);
		solver.solve(x.neq(y));
	    c = solver.getConstraint();
	    solver.solve(c);
	    assertEquals("[_?] neq _?",c.toString());
	}
		
	//LList INIZIALIZZATA CON LVAR NON INIZ ---> LList INIZ CON LVAR NON INIZ
	@Test
	public void testNEQLListInitializedWithLVarUninitialized_LListInitializedWithLVarUninitialized()  throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Constraint c;
		LVar x1 = new LVar();
		LList x = LList.empty().ins(x1).ins(2);
		LList y = LList.empty().ins(x1).ins(3);
		solver.solve(x.neq(y));
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

}