package jsetl.test;

//import static org.junit.Assert.*;

//import java.io.IOException;

//import java.util.ArrayList;
import java.util.HashSet;
//import java.util.List;
import java.util.Set;

import jsetl.exception.Failure;
import jsetl.exception.NotDefConstraintException;
import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.*;

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//        Testing Partially Specified Logical Set Constraints         %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


public class ConstraintSetTest extends TestCase {

	class Solver extends jsetl.Solver {
		@Override
		public void solve(int lev) throws Failure{
			super.solve(lev);
		}
		@Override
		public void solve(Constraint cc, int lev) throws Failure{
			super.solve(cc, lev);
		}
	}

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//               Constraint disj     					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// LSET NON INIZIALIZZATO disj LSET NON INIZIALIZZATO
// disj(x,y) --- solved form
	@Test
	public void testDISJ_LSetUninit_LSetUninit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LSet y = new LSet("y");
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("disj(_x,_y)",c.toString());
//	     System.out.println(c.toString());
	}

// LSET NON INIZIALIZZATO disj STESSO LSET NON INIZIALIZZATO
// disj(x,x)
	@Test
	public void testDISJ_LSetUninit_SameLSetrUninit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     solver.solve(x.disj(x));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());   
	     assertEquals("{}",x.toString());   
	}

//LSET INIZIALIZZATO disj STESSO LSET  INIZIALIZZATO
//x = {1}, disj(x,x)
	@Test
	public void testDISJ_LSetInit_SameLSetInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     try{		    	
	    	solver.solve(x.disj(x));
			fail("FAILURE!!!!");
		 }
	     catch(Failure e) {}	
	}

//LSET INIZIALIZZATO disj LSET INIZIALIZZATO IDENTICO
//x = {1}, y = {1}, disj(x,y)
	@Test
	public void testDISJ_LSetInit_LSetInitIdentical() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     LSet y = LSet.empty().ins(1);
	     try{		    	
	    	solver.solve(x.disj(y));
			fail("FAILURE!!!!");
		 }
	     catch(Failure e) {}	
	}
	
//LSET NON INIZIALIZZATO disj LSET VUOTO
//disj(x,{})
	@Test
	public void testDISJ_LSetUninit_EmptyLSet() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LSet y = LSet.empty();
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//LSET VUOTO disj LSET VUOTO
//disj({},{})
	@Test
	public void testDISJ_EmptyLSet_EmptyLSet() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     solver.solve(x.disj(LSet.empty()));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//LSET VUOTO disj LSET INIZIALIZZATO
//x = {1}, disj({},x)
	@Test
	public void testDISJ_EmptyLSet_LSetInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     solver.solve(LSet.empty().disj(x));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//LSET INIZIALIZZATO disj LSET VUOTO  
//x = {1}, disj(x,{})
	@Test
	public void testDISJ_LSetInit_EmptyLSet() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     solver.solve(x.disj(LSet.empty()));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//LSET NON INIZIALIZZATO disj LSET INIZIALIZZATO COMPL. SPEC. CHIUSO
//disj(x,{1})
	@Test
	public void testDISJ_LSetUninit_LSetInitClosedCS() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LSet y = LSet.empty().ins(1);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("1 nin _x",c.toString());    
	}

//LSET NON INIZIALIZZATO disj LSET INIZIALIZZATO PARZ. SPEC. CHIUSO
//disj(x,{z})
	@Test
	public void testDISJ_LSetUninit_LSetInitClosedPS() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LVar z = new LVar("z");
	     LSet y = LSet.empty().ins(z);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("_z nin _x",c.toString());    
//	     System.out.println(c.toString());
	}

//LSET NON INIZIALIZZATO disj LSET INIZIALIZZATO APERTO
//disj(x,{z|r})
	@Test
	public void testDISJ_LSetUninit_LSetInitOpen() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LVar z = new LVar("z");
	     LSet r = new LSet("r");
	     LSet y = r.ins(z);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("disj(_x,_r) AND _z nin _x",c.toString());
	}

//LSET INIZIALIZZATO APERTO  disj LSET NON INIZIALIZZATO
//disj({z|r},x)
	@Test
	public void testDISJ_LSetInitOpen_LSetUninit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LVar z = new LVar("z");
	     LSet r = new LSet("r");
	     LSet y = r.ins(z);
	     solver.solve(y.disj(x));               
	     Constraint c = solver.getConstraint();
	     assertEquals("disj(_r,_x) AND _z nin _x",c.toString());
	}
	
//LSET NON INIZIALIZZATO disj LSET INIZIALIZZATO PARZ. SPEC. CHIUSO
//disj(x,{x})
	@Test
	public void testDISJ_LSetUninit_LSetInitClosedPS_SameVar() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LSet y = LSet.empty().ins(x);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("_x nin _x",c.toString());    // BUG! Should be --> TRUE
//	     System.out.println(c.toString());
	}

//LSET INIZIALIZZATO disj LSET INIZIALIZZATO CON INTERSEZIONE
//x = {1,2}, y = {3,1}, disj(x,y)
	@Test
	public void testDISJ_LSetInit_LSetInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1).ins(2);
	     LSet y = LSet.empty().ins(3).ins(1);
	     try{		    	
	    	solver.solve(x.disj(y));
			fail("FAILURE!!!!");
		 }
	     catch(Failure e) {}	
	}

//LSET INIZIALIZZATO disj LSET INIZIALIZZATO SENZA INTERSEZIONE
//x = {1,2}, y = {3,1}, disj(x,y)
	@Test
	public void testDISJ_LSetInit_LSetInitOK() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1).ins(2);
	     LSet y = LSet.empty().ins(3).ins(4);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());    
	}

//LSET INIZIALIZZATO disj LSET INIZIALIZZATO P.S.
//x = {1,2}, y = {z,3}, disj(x,y)
	@Test
	public void testDISJ_LSetInit_LSetInitPS() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1).ins(2);
	     LVar z = new LVar("z");
	     LSet y = LSet.empty().ins(3).ins(z);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("2 neq _z AND _z neq 1",c.toString());    
	}

//LSET INIZIALIZZATO P.S. disj LSET INIZIALIZZATO P.S.
//x = {z1,2}, y = {z2,3}, disj(x,y)
	@Test
	public void testDISJ_LSetInitPS_LSetInitPS() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LVar z1 = new LVar("z1");
	     LSet x = LSet.empty().ins(z1).ins(2);
	     LVar z2 = new LVar("z2");
	     LSet y = LSet.empty().ins(3).ins(z2);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("2 neq _z2 AND _z2 neq _z1 AND _z1 neq 3",c.toString());    
//	     System.out.println(c.toString());
	}

//LSET INIZIALIZZATO P.S. disj LSET INIZIALIZZATO P.S. NO
//x = {1,2}, y = {z,1}, disj(x,y)
	@Test
	public void testDISJ_LSetInitPS_LSetInitPS_NO() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1).ins(2);
	     LVar z = new LVar("z");
	     LSet y = LSet.empty().ins(1).ins(z);
	     try{		    	
		    	solver.solve(x.disj(y));
				fail("FAILURE!!!!");
			 }
		     catch(Failure e) {}	
	}

//LSET INIZIALIZZATO P.S. disj LSET INIZIALIZZATO P.S. STESSA VAR. NO
//x = {1,2}, y = {z,1}, disj(x,y)
	@Test
	public void testDISJ_LSetInitPS_LSetInitPS_SameVar_NO() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LVar z = new LVar("z");
	     LSet x = LSet.empty().ins(z).ins(2);
	     LSet y = LSet.empty().ins(1).ins(z);
	     try{		    	
		    	solver.solve(x.disj(y));
				fail("FAILURE!!!!");
			 }
		     catch(Failure e) {}	
	}

//LSET INIZIALIZZATO disj LSET INIZIALIZZATO APERTO
//x = {1,2}, y = {z|r}, disj(x,y)
	@Test
	public void testDISJ_LSetInit_LSetInitOpen() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1).ins(2);
	     LVar z = new LVar("z");
	     LSet r = new LSet("r");
	     LSet y = r.ins(z);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("2 neq _z AND 2 nin _r AND _z neq 1 AND 1 nin _r",c.toString());    
//	     System.out.println(c.toString());
	}

// USING java.util.Set

//LSET NON INIZIALIZZATO disj SET INIZIALIZZATO
//disj(x,{1})
	@Test
	public void testDISJ_LSetUninit_SetInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     Set<Integer> y = new HashSet<Integer>();
	     y.add(1);
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("1 nin _x",c.toString());    
	}

//LSET NON INIZIALIZZATO disj SET VUOTO
//disj(x,{})
	@Test
	public void testDISJ_LSetUninit_EmptySet() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     Set<Integer> y = new HashSet<Integer>();
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//LSET VUOTO disj SET VUOTO
//disj({},{})
	@Test
	public void testDISJ_EmptyLSet_EmptySet() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     Set<Integer> y = new HashSet<Integer>();
	     solver.solve(x.disj(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}

//SET INIZIALIZZATO disj LSET INIZIALIZZATO P.S.
//x = {1,2}, y = {z,3}, disj(x,y)
	@Test
	public void testDISJ_SetInit_LSetInitPS() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Set<Integer> x = new HashSet<Integer>();
	     x.add(1); x.add(2);
	     LVar z = new LVar("z");
	     LSet y = LSet.empty().ins(3).ins(z);
	     solver.solve(y.disj(x));               
	     Constraint c = solver.getConstraint();
	     assertEquals("_z neq 1 AND _z neq 2",c.toString());    
	}

	
//SET INIZIALIZZATO disj LSET INIZIALIZZATO APERTO
//x = {1,2}, y = {z|r}, disj(x,y)
	@Test
	public void testDISJ_SetInit_LSetInitOpen() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     Set<Integer> x = new HashSet<Integer>();
	     x.add(1); x.add(2);
	     LVar z = new LVar("z");
	     LSet r = new LSet("r");
	     LSet y = r.ins(z);
	     solver.solve(y.disj(x));               
	     Constraint c = solver.getConstraint();
	     assertEquals("_z neq 1 AND 1 nin _r AND _z neq 2 AND 2 nin _r",c.toString());    
//	     System.out.println(c.toString());
	}
	
	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//              Constraint union     					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO
//z = union(x,y)
	@Test
	public void testUNION_LSetUninit_LSetUninit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("union(_x,_y,_z)",c.toString());
	}

//LSET NON INIZIALIZZATO union stesso LSET NON INIZIALIZZATO
//z = union(x,x)
	@Test
	public void testUNION_LSetUninit_SameLSetUninit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet z = new LSet("z");
		solver.solve(x.union(x,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("_x",z.toString());
	}

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET VUOTO
//{} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUninit_LSetEmpty()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = LSet.empty().setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{}",x.toString());
		assertEquals("{}",y.toString());
	}

//LSET vuoto union LSET NON INIZIALIZZATO = LSET VUOTO
//{} = union({},y)
	@Test
	public void testUNION_LSetEmpty_LSetUninit_LSetEmpty()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().setName("x");
		LSet y = new LSet("y");
		LSet z = LSet.empty().setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{}",y.toString());
	}

//LSET vuoto union LSET INIZIALIZZATO = LSET NON INIZIAlizzato
//z = union({},{1})
	@Test
	public void testUNION_LSetEmpty_LSetInit_LSetUninit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().setName("x");
		LSet y = LSet.empty().ins(1).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1}",z.toString());
	}

//LSET INIZIALIZZATO union LSET vuoto = LSET NON INIZIAlizzato
//z = union({1},{})
	@Test
	public void testUNION_LSetInit_LSetEmpty_LSetUnint()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().setName("x");
		LSet y = LSet.empty().ins(1).setName("y");
		LSet z = new LSet("z");
		solver.solve(y.union(x,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1}",z.toString());
	}

//LSET INIZIALIZZATO GROUND union LSET INIZIALIZZATO GROUND
//z = union({1},{2})
	@Test
	public void testUNION_LSetInitGround_LSetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).setName("x");
		LSet y = LSet.empty().ins(2).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2}",z.toString());
	}

//LSET INIZIALIZZATO GROUND union LSET INIZIALIZZATO GROUND CON RIPETIZIONI
//z = union({1,2},{2,3})
	@Test
	public void testUNION_LSetInitGround_LSetInitGround_Repetitions()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet y = LSet.empty().ins(2).ins(3).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{2,1,3}",z.toString());
	}

//LSET INIZIALIZZATO GROUND union LSET INIZIALIZZATO GROUND = LSET INIZIALIZZATO GROUND
//{1,2} = union({1},{2})
	@Test
	public void testUNION_LSetInitGround_LSetinitGround_LSetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).setName("x");
		LSet y = LSet.empty().ins(2).setName("y");
		LSet z = LSet.empty().ins(2).ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
	}

//LSET INIZIALIZZATO GROUND union LSET NON INIZIALIZZATO
//z = union({1,2},y)
	@Test
	public void testUNION_LSetInitGround_LSetUninit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(2).ins(1).setName("x");
		LSet y = new LSet("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2/_y}",z.toString());
	}

//LSET NON INIZIALIZZATO union LSET INIZIALIZZATO GROUND
//z = union(y,{1,2})
	@Test
	public void testUNION_LSetUninit_LSetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(2).ins(1).setName("x");
		LSet y = new LSet("y");
		LSet z = new LSet("z");
		solver.solve(y.union(x,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2/_y}",z.toString());
	}

//------------- Using Set objects --------------

//EMPTY SET union SET INIZIALIZZATO GROUND
//z = union({},{2})
	@Test
	public void testUNION_SetEmpty_SetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Set<Integer> x = new HashSet<Integer>();
		Set<Integer> y = new HashSet<Integer>(); y.add(2);
		LSet z = new LSet("z");
		solver.solve(new LSet(x).union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{2}",z.toString());
	}

//SET INIZIALIZZATO GROUND union SET INIZIALIZZATO GROUND
//z = union({1},{2})
	@Test
	public void testUNION_SetInitGround_SetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		Set<Integer> x = new HashSet<Integer>(); x.add(1);
		Set<Integer> y = new HashSet<Integer>(); y.add(2);
		LSet z = new LSet("z");
		solver.solve(new LSet(x).union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2}",z.toString());
	}

//LSET INIZIALIZZATO GROUND union SET INIZIALIZZATO GROUND
//z = union({1},{2})
	@Test
	public void testUNION_LSetInitGround_SetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).setName("x");
		Set<Integer> y = new HashSet<Integer>(); y.add(2);
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{1,2}",z.toString());
	}

//LSET NON INIZIALIZZATO union SET INIZIALIZZATO GROUND
//z = union(x,{2})
	@Test
	public void testUNION_LSetUnInit_SetInitGround()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		Set<Integer> y = new HashSet<Integer>(); y.add(2);
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{2/_x}",z.toString());
	}

//------------- Using "equ" chain --------------

//LSET NON INIZIALIZZATO union LSET INIZIALIZZATO GROUND
//z = union(y,{1,2})
	@Test
	public void testUNION_LSetUninit_LSetInitGround_equ()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x1 = LSet.empty().ins(2).ins(1);
		LSet x = new LSet("x");
		solver.add(x.eq(x1));
		LSet y1 = new LSet();
		LSet y = new LSet("y");
		solver.add(y.eq(y1));
		LSet z = new LSet("z");
		solver.add(y.union(x,z));
		solver.solve();
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
//		System.out.println(c.toString());
		assertEquals("{1,2/_y}",z.toString());
//		System.out.println(z.toString());
	}

//------------- Nondeterministic rewriting --------------

// s3 = union(s1,s2)
// with known (possibly p.s.) s s3;
// with either unknown or p.s. sets (not empty) s1 and s2

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO GROUND
//{1} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = LSet.empty().ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
//		System.out.println(c.toString());
		assertEquals("[]",c.toString());
//		System.out.println(x.toString());
		assertEquals("{1}",x.toString());
//		System.out.println(y.toString());
		assertEquals("{}",y.toString());

		solver.nextSolution();
		assertEquals("{}",x.toString());
		assertEquals("{1}",y.toString());

		solver.nextSolution();
		assertEquals("{1}",x.toString());
		assertEquals("{1}",y.toString());

		assertFalse(solver.nextSolution());
	}

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO GROUND
//satisfiability level = 1
//{1} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInit_NC()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = LSet.empty().ins(1).setName("z");
		solver.solve(x.union(y,z),1);  // <==== level 1 solve
		Constraint c = solver.getConstraint();
		assertEquals("union(_x,_y,{1})",c.toString());
		assertEquals("_x",x.toString());
		assertEquals("_y",y.toString());

		assertFalse(solver.nextSolution());  // n.b. one solution only
	}
//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO GROUND
//{1,2} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInit2()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = LSet.empty().ins(2).ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
//		System.out.println(c.toString());
		assertEquals("[]",c.toString());
//		System.out.println(x.toString());
		assertEquals("{1,2}",x.toString());
//		System.out.println(y.toString());
		assertEquals("{}",y.toString());

		solver.nextSolution();
		assertEquals("{1}",x.toString());
		assertEquals("{2}",y.toString());

		solver.nextSolution();
		assertEquals("{1,2}",x.toString());
		assertEquals("{2}",y.toString());

		solver.nextSolution();
		assertEquals("{2}",x.toString());
		assertEquals("{1}",y.toString());

		solver.nextSolution();
		assertEquals("{}",x.toString());
		assertEquals("{1,2}",y.toString());

		solver.nextSolution();
		assertEquals("{2}",x.toString());
		assertEquals("{1,2}",y.toString());

		solver.nextSolution();
		assertEquals("{1,2}",x.toString());
		assertEquals("{1}",y.toString());

		solver.nextSolution();
		assertEquals("{1}",x.toString());
		assertEquals("{1,2}",y.toString());

		solver.nextSolution();
		assertEquals("{1,2}",x.toString());
		assertEquals("{1,2}",y.toString());

		assertFalse(solver.nextSolution());
	}

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO P.S
//{w} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInitPS()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LVar w = new LVar("w");
		LSet z = LSet.empty().ins(w).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{_w}",x.toString());
		assertEquals("{}",y.toString());

		solver.nextSolution();
		assertEquals("{}",x.toString());
		assertEquals("{_w}",y.toString());

		solver.nextSolution();
		assertEquals("{_w}",x.toString());
		assertEquals("{_w}",y.toString());

		assertFalse(solver.nextSolution());
	}

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO P.S
//{w,v} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInitPS2()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LVar w = new LVar("w");
		LVar v = new LVar("v");
		LSet z = LSet.empty().ins(v).ins(w).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("_w neq _v AND _w neq _v",c.toString());     // TO BE IMPROVED
		assertEquals("{_w,_v}",x.toString());
		assertEquals("{}",y.toString());
//		System.out.println(c.toString());
//		System.out.println(x.toString());
//		System.out.println(y.toString());

		assertTrue(solver.nextSolution());
	}

//LSET NON INIZIALIZZATO union LSET NON INIZIALIZZATO = LSET INIZIALIZZATO open
//{1|r} = union(x,y)
	@Test
	public void testUNION_LSetUnInit_LSetUnInit_LSetInitOpen()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet r = new LSet("r");
		LSet z = r.ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("1 nin _r AND 1 nin _y AND 1 nin _? AND union(_?,_y,_r)", c.toString());
		assertEquals("{1/_?}",x.toString());
		assertEquals("_y",y.toString());
		assertEquals("_r",r.toString());

		solver.nextSolution();
		assertEquals("1 nin _r AND 1 nin {1/_?} AND 1 nin _? AND union(_?,{1/_?},_r)",c.toString());
		assertEquals("_x",x.toString());
		assertEquals("{1/_?}",y.toString());
		assertEquals("_r",r.toString());

		solver.nextSolution();
		assertEquals("1 nin _r AND 1 nin {1/_?} AND 1 nin _? AND union(_?,{1/_?},_r)",c.toString());
		assertEquals("{1/_?}",x.toString());
		assertEquals("{1/_?}",y.toString());
		assertEquals("_r",r.toString());

		solver.nextSolution();
//      System.out.println(c.toString());
//		System.out.println(x.toString());
//		System.out.println(y.toString());
//		System.out.println(r.toString());

		//to be completed:  controllare anche altre soluzioni
	}

//LSET INIZIALIZZATO non ground union LSET INIZIALIZZATO n.g. = LSET INIZIALIZZATO GROUND
//{1,2} = union({w},{v})
	@Test
	public void testUNION_LSetInitPS_LSetInit_LSetInit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LVar v = new LVar("v");
		LSet y = LSet.empty().ins(v).setName("y");
		LSet z = LSet.empty().ins(2).ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("1",w.toString());
		assertEquals("2",v.toString());

		solver.nextSolution();
//		System.out.println(w.toString());
		assertEquals("2",w.toString());
//		System.out.println(v.toString());
		assertEquals("1",v.toString());

		assertFalse(solver.nextSolution());
	}

//LSET INIZIALIZZATO non ground union LSET INIZIALIZZATO n.g. = LSET INIZIALIZZATO n.g.
//{u} = union({w},{v})
	@Test
	public void testUNION_LSetInitPS_LSetInit_LSetInitPS()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LVar v = new LVar("v");
		LSet y = LSet.empty().ins(v).setName("y");
		LVar u = new LVar("u");
		LSet z = LSet.empty().ins(u).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("_u",v.toString());
		assertEquals("_u",v.toString());
		assertEquals("_u",v.toString());

		assertFalse(solver.nextSolution());
	}

//LSET INIZIALIZZATO non ground union LSET NON INIZIALIZZATO  = LSET INIZIALIZZATO ground
//{1} = union({w},y)
	@Test
	public void testUNION_LSetInitPS_LSetUnInit_LSetInit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LSet y = new LSet().setName("y");
		LSet z = LSet.empty().ins(1).setName("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("1",w.toString());
		assertEquals("{}",y.toString());

		solver.nextSolution();
		assertEquals("1",w.toString());
		assertEquals("{1}",y.toString());

		assertFalse(solver.nextSolution());
	}

//LSET NON INIZIALIZZATO union LSET INIZIALIZZATO non ground = LSET INIZIALIZZATO ground
//{1} = union(y,{w})
	@Test
	public void testUNION_LSetUnInit_LSetInitPS_LSetInit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LSet y = new LSet().setName("y");
		LSet z = LSet.empty().ins(1).setName("z");
		solver.solve(y.union(x,z));
		Constraint c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("1",w.toString());
		assertEquals("{}",y.toString());

		solver.nextSolution();
		assertEquals("1",w.toString());
		assertEquals("{1}",y.toString());

		assertFalse(solver.nextSolution());
	}

// s3 = union(s1,s2)
// with unknown s s3;
// with either unknown or p.s. (not empty) sets s1 and s2

//LSET INIZIALIZZATO non ground union LSET INIZIALIZZATO n.g. = LSET NON INIZIALIZZATO
//z = union({w},{1})
	@Test
	public void testUNION_LSetInitPS_LSetInitPS_LSetUnInit()
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LVar v = new LVar("v");
		LSet y = LSet.empty().ins(v).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.union(y,z));
		Constraint c = solver.getConstraint();
		assertEquals("_w neq _v AND _w neq _v",c.toString());     
		assertEquals("{_w,_v}",z.toString());     // to be improved

		solver.nextSolution();               
		c = solver.getConstraint();
		assertEquals("[]",c.toString());
		assertEquals("{_w}",z.toString());     

		assertFalse(solver.nextSolution());               
	} 

// to be tested:
//z = union(x,{1})
//z = union({1},y)
//z = union({2|x},{1})
//z = union({1|x},{1|y})
//z = union({2|x},{1|y})
//z = union({2|x},{1|x})
//z = union({1},z)
//z = union({w},z)


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//            Constraint subset      					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET NON INIZIALIZZATO subset LSET NON INIZIALIZZATO
//subset(x,y) - solved form
	@Test
	public void testSUBSET_LSetUninit_LSetUninit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = new LSet("x");
	     LSet y = new LSet("y");
	     solver.solve(x.subset(y));               
	     Constraint c = solver.getConstraint();
		 assertEquals("union(_x,_y,_y)",c.toString());
	}

//LSET INIZIALIZZATO subset LSET INIZIALIZZATO
//x = {1}, y = {1,2}, subset(x,y)  ===> subset({1},{1,2})
	@Test
	public void testSUBSET_LSetInit_LSetInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     LSet y = LSet.empty().ins(2).ins(1);
	     solver.solve(x.subset(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	}
	
//LSET INIZIALIZZATO subset LSET INIZIALIZZATO caso false 
//x = {3}, y = {1,2}, subset(x,y)  ===> subset({3},{1,2})
	@Test
	public void testDISJ_LSetInit_LSetInitFalse() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(3);
	     LSet y = LSet.empty().ins(2).ins(1);
	     try{		    	
	    	solver.solve(x.subset(y));
			fail("FAILURE!!!!");
		 }
	     catch(Failure e) {}	
	}

//LSET INIZIALIZZATO subset LSET NON INIZIALIZZATO
//x = {1}, subset(x,y)  ===> subset({1},y)
	@Test
	public void testSUBSET_LSetInit_LSetUnInit() 
	throws NotDefConstraintException, Failure {
		 Solver solver = new Solver();
	     LSet x = LSet.empty().ins(1);
	     LSet y = new LSet("y");
	     solver.solve(x.subset(y));               
	     Constraint c = solver.getConstraint();
	     assertEquals("[]",c.toString());     
	     assertEquals("{1/_?}",y.toString());     
	}

//LSET NON INIZIALIZZATO subset LSET INIZIALIZZATO
//y = {1,2}, subset(x,y)  ===> subset(x,{1,2})
	@Test
	public void testSUBSET_LSetUnInit_LSetInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
	    LSet x = new LSet("y");
	    LSet y = LSet.empty().ins(2).ins(1);
	    solver.solve(x.subset(y));               
	    Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	     assertEquals("[]",c.toString());     
//	    System.out.println(x.toString());
	     assertEquals("{}",x.toString());   
	     
		solver.nextSolution();               
		assertEquals("{2}",x.toString());     

		solver.nextSolution();               
		assertEquals("{1}",x.toString());     

		solver.nextSolution();               
		assertEquals("{1,2}",x.toString());     

		assertFalse(solver.nextSolution());               
	}

	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//          Constraint inters        					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET INIZIALIZZATO GROUND inters LSET INIZIALIZZATO GROUND 
//z = inters({1,2},{2,3})
	@Test
	public void testINTERS_LSetInitGround_LSetInitGround() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet y = LSet.empty().ins(2).ins(3).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.inters(y,z));
		Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("[]",c.toString());     
//	    System.out.println(z.toString());
	    assertEquals("{2}",z.toString());     
	}	
	
//LSET NON INIZIALIZZATO inters LSET NON INIZIALIZZATO
//z = inters(x,y) - solved form
	@Test
	public void testINTERS_LSetUnInit_LSetUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = new LSet("z");
		solver.solve(x.inters(y,z));
		Constraint c = solver.getConstraint();
	    assertEquals("union(_?,_z,_x) AND union(_?,_z,_y) AND disj(_?,_?)",c.toString());

		assertFalse(solver.nextSolution());               	    
    }

//LSET INIZIALIZZATO GROUND inters LSET INIZIALIZZATO GROUND VUOTA
//z = inters({1,2},{4,3})
	@Test
	public void testINTERS_LSetInitGround_LSetInitGround_Empty() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet y = LSet.empty().ins(4).ins(3).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.inters(y,z));
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
//	    System.out.println(z.toString());
	    assertEquals("{}",z.toString());     
	}	

//LSET INIZIALIZZATO NON GROUND inters LSET INIZIALIZZATO GROUND
//z = inters({W},{1})
	@Test
	public void testINTERS_LSetInitPS_LSetInitGround_Empty() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).setName("x");
		LSet y = LSet.empty().ins(1).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.inters(y,z));
		Constraint c = solver.getConstraint();
	    assertEquals("_w neq 1",c.toString());     
	    assertEquals("{}",z.toString());     
	    
	    solver.nextSolution();	    
	    c = solver.getConstraint();
        assertEquals("[]",c.toString());   
	    assertEquals("{1}",z.toString());     

	    solver.nextSolution();
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());   // to be improved: repeated solutions
	    assertEquals("{1}",z.toString());     

	    solver.nextSolution();
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());   // to be improved: repeated solutions
	    assertEquals("{1}",z.toString());     

		assertFalse(solver.nextSolution());               
	}	


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//        Constraint diff        					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET INIZIALIZZATO GROUND inters LSET INIZIALIZZATO GROUND 
//z = diff({1,2},{2,3})
	@Test
	public void testDIFF_LSetInitGround_LSetInitGround() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet y = LSet.empty().ins(2).ins(3).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.diff(y,z));
				
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
	    assertEquals("{1}",z.toString());     

	   /* solver.nextSolution();		
	    assertEquals("{1}",z.toString());        // to be improved: repeated solutions

	    solver.nextSolution();
		assertEquals("{1}",z.toString());     

	    solver.nextSolution();
		assertEquals("{1}",z.toString());     

		assertFalse(solver.nextSolution());    */           	    
    }	

//LSET NON INIZIALIZZATO diff LSET NON INIZIALIZZATO
//z = diff(x,y) - solved form
	@Test
	public void testDIFF_LSetUnInit_LSetUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet y = new LSet("y");
		LSet z = new LSet("z");
		solver.solve(x.diff(y,z));
		Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("union(_z,_x,_x) AND union(_x,_?,_?) AND union(_y,_z,_?) AND disj(_y,_z)",c.toString());

		assertFalse(solver.nextSolution());               	    
    }
	
//LSET INIZIALIZZATO GROUND inters LSET INIZIALIZZATO GROUND disgiunti
//z = diff({1,2},{4,3})
	@Test
	public void testDIFF_LSetInitGround_LSetInitGround_disj() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet y = LSet.empty().ins(4).ins(3).setName("y");
		LSet z = new LSet("z");
		solver.solve(x.diff(y,z));
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());  
	    assertEquals("{2,1}",z.toString());     
    }	


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//      Constraint less             					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET INIZIALIZZATO GROUND less LVAR INIZIALIZZATO  
//z = less({1,2},2)
	@Test
	public void testLESS_LSetInitGround_LVarInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet z = new LSet("z");
		LVar w = new LVar(2);
		solver.solve(x.less(w, z));
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
	    assertEquals("{1}",z.toString());     

	    assertFalse(solver.nextSolution());               	    
  }	

//LSET NON INIZIALIZZATO less LVAR NON INIZIALIZZATO
//z = less(x,w) - solved form
	@Test
	public void testLESS_LSetUnInit_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		LSet z = new LSet("z");
		LVar w = new LVar("w");
		solver.solve(x.less(w, z));
		Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("_w nin _z",c.toString());     
//	    System.out.println(x.toString());
	    assertEquals("{_w/_z}",x.toString());     
	    
	    assertFalse(solver.nextSolution());               	    
  }
	
//LSET INIZIALIZZATO GROUND less LVAR INIZIALIZZATA non presente
//z = less({1,2},3)
	@Test
	public void testLESS_LSetInitGround_LVarInit_disj() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet z = new LSet("z");
		LVar w = new LVar(3);
	    try{		    	
	    	solver.solve(x.less(w, z));
			fail("FAILURE!!!!");
		}
		catch(Failure e) {}	
    }	

//LSET INIZIALIZZATO NON GROUND less INTERO  
//z = less({w,2},2)
	@Test
	public void testLESS_LSetInitPS_Int() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(w).ins(2).setName("x");
		LSet z = new LSet("z");
		solver.solve(x.less(2, z));
		Constraint c = solver.getConstraint();
	    assertEquals("2 neq _w",c.toString());   
	    assertEquals("_w",w.toString());     
	    assertEquals("{_w}",z.toString());     

	    solver.nextSolution();
	    c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("[]",c.toString());     
	    assertEquals("2",w.toString());     
	    assertEquals("{}",z.toString());     

		assertFalse(solver.nextSolution());        
	}	

//LSET INIZIALIZZATO  GROUND less INTERO  
//z = less({1,2},w)
	@Test
	public void testLESS_LSetInit_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		LSet z = new LSet("z");
		LVar w = new LVar("w");
		solver.solve(x.less(w, z));
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
	    assertEquals("2",w.toString());     
	    assertEquals("{1}",z.toString());     

	    solver.nextSolution();
	    c = solver.getConstraint();
	    assertEquals("[]",c.toString());   
	    assertEquals("1",w.toString());     
	    assertEquals("{2}",z.toString());     

		assertFalse(solver.nextSolution());        
	}
	

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//    Constraint size                					  %	
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//LSET INIZIALIZZATO GROUND   
//n = size({1,2})
	@Test
	public void testSIZE_LSetInitGround_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = LSet.empty().ins(1).ins(2).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
//	    System.out.println(n.toString());
	    assertEquals("2",n.toString());     
    }	

//LSET INIZIALIZZATO GROUND con LVar  
//w = 2, n = size({1,w})
	@Test
	public void testSIZE_LSetInit_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w",2);
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
	    assertEquals("2",n.toString());     
    }	
	
//LSET INIZIALIZZATO GROUND con LVar ripetuta  
//w = 1, n = size({1,w})
	@Test
	public void testSIZE_LSetInitRepetitions_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w",1);
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
	    assertEquals("[]",c.toString());     
	    assertEquals("1",n.toString());     
    }	

//LSET NON INIZIALIZZATO   
//n = size(x) - solved form
	@Test
	public void testSIZE_LSetUnInit_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("size(_x,_n)",c.toString());
    }
	
//LSET NON INIZIALIZZATO, LVAR INIZIALIZZATA 
//n = 3, n = size(x)
	@Test
	public void testSIZE_LSetUnInit_LVarInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		IntLVar n = new IntLVar("n",3);
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
//	    System.out.println(c.toString());
	    assertEquals("_? neq _? AND _? neq _? AND _? neq _?",c.toString());     
	    assertEquals("{_?,_?,_?}",x.toString());     
    }	

//LSET NON INIZIALIZZATO, INTEGER 
//3 = size(x)
	@Test
	public void testSIZE_LSetUnInit_Integer() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet x = new LSet("x");
		solver.solve(x.size(3));               
		Constraint c = solver.getConstraint();
	    assertEquals("_? neq _? AND _? neq _? AND _? neq _?",c.toString());     
	    assertEquals("{_?,_?,_?}",x.toString());     
    }	

// non-deterministic size
	
//LSET INIZIALIZZATO NON GROUND, LVar NON INIZIALIZZATA  
//n = size({1,w})
	@Test
	public void testSIZE_LSetInitPS_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));
		Constraint c = solver.getConstraint();
		assertEquals("_w neq 1",c.toString());     
		assertEquals("2",n.toString());     
		assertEquals("_w",w.toString());    
		
	    solver.nextSolution();
	    c = solver.getConstraint();
		assertEquals("[]",c.toString());     
		assertEquals("1",n.toString());     
		assertEquals("1",w.toString()); 
		
		assertFalse(solver.nextSolution());        
    }	

//LSET INIZIALIZZATO NON GROUND, LVar NON INIZIALIZZATA 
//satisfiability level = 1
//n = size({1,w})
	@Test
	public void testSIZE_LSetInitPS_LVarUnInit_NC() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n),1);  // <===== level 1              
		Constraint c = solver.getConstraint();
		assertEquals("size({_w,1},_n)",c.toString());
		assertEquals("_n",n.toString());     
		assertEquals("_w",w.toString());    
		
		assertFalse(solver.nextSolution()); // n.b. one solution only  
  }	

//LSET INIZIALIZZATO NON GROUND, LVar NON INIZIALIZZATA 
//incremental solving
//n = size({1,w})
	@Test
	public void testSIZE_LSetInitPS_LVarUnInit_INCR() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n),1);  // == partialSolve              
					                // OR
		                            // solver.setLevel(1);
		                            // solver.solve(x.size(n));   
		Constraint c = solver.getConstraint();
		assertEquals("size({_w,1},_n)",c.toString());

		solver.solve(3);            // == finalSolve              
                                    // OR
                                    // solver.setLevel(3);
                                    // solver.solve();   
		c = solver.getConstraint();
		assertEquals("_w neq 1",c.toString());     
		assertEquals("2",n.toString());     
		assertEquals("_w",w.toString());    
		
	    solver.nextSolution();
	    c = solver.getConstraint();
		assertEquals("[]",c.toString());     
		assertEquals("1",n.toString());     
		assertEquals("1",w.toString()); 
		
		assertFalse(solver.nextSolution());  		
	}	

//LSET INIZIALIZZATO NON GROUND, Integer  
//2 = size({1,w})
	@Test
	public void testSIZE_LSetInitPS_Integer() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LVar w = new LVar("w");
		LSet x = LSet.empty().ins(1).ins(w).setName("x");
		solver.solve(x.size(2));               
		Constraint c = solver.getConstraint();
		assertEquals("_w neq 1",c.toString());     
		assertEquals("_w",w.toString());    
		
		assertFalse(solver.nextSolution());        
    }
	
//LSET INIZIALIZZATO open, LVar INIZIALIZZATA  
//w = 2, w = size({1|r})
	@Test
	public void testSIZE_LSetInitOpen_LVarInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet r = new LSet("r");
		LSet x = r.ins(1).setName("x");
		IntLVar n = new IntLVar("n",2);
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
		assertEquals("1 neq _?",c.toString());     
		assertEquals("{_?}",r.toString());     
		
	    solver.nextSolution();
	    c = solver.getConstraint();
	    	    
		assertEquals("1 neq _?",c.toString());     
		assertEquals("{1,_?}",r.toString());     
		
		assertFalse(solver.nextSolution());        
    }	

//LSET INIZIALIZZATO open, LVar NON INIZIALIZZATA  
//n = size({1|r})
	@Test
	public void testSIZE_LSetInitOpen_LVarUnInit() 
	throws NotDefConstraintException, Failure {
		Solver solver = new Solver();
		LSet r = new LSet("r");
		LSet x = r.ins(1).setName("x");
		IntLVar n = new IntLVar("n");
		solver.solve(x.size(n));               
		Constraint c = solver.getConstraint();
//		System.out.println(c.toString());
//		assertEquals("1 nin _r AND _? = _n - 1 AND size(_r) = _?",c.toString());     
//		System.out.println(r.toString());
		assertEquals("_r",r.toString());     
		
	    solver.nextSolution();
	    c = solver.getConstraint();
//		System.out.println(c.toString());
//		assertEquals("1 nin _? AND _? = _n - 1 AND size(_?) = _?",c.toString());     
//		System.out.println(r.toString());
		assertEquals("{1/_?}",r.toString());     
		
		assertFalse(solver.nextSolution());        
  }	

}
