package jsetl.test;

import jsetl.*;
import jsetl.test.utilities.comparing.ConstraintStringComparator;
import jsetl.test.utilities.comparing.ConstraintStringNormalizer;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class ConstraintStringComparatorTest extends TestCase {
    @Before
    public void setUp() {
        LObject.USEJOLLYTOKEN = true;
    }

    @After
    public void tearDown(){LObject.USEJOLLYTOKEN = false;}

    @Test
    public void testConstraintNormalization1(){
        IntLVar i = new IntLVar("_I123");
        IntLVar j = new IntLVar("_I1234");

        Constraint constraint = i.eq(j);
        assertEquals("__I123 = __I1234", constraint.toString());
        ConstraintStringNormalizer constraintStringNormalizer = new ConstraintStringNormalizer(constraint);

        assertEquals("__I1 = __I2", constraintStringNormalizer.normalize());
    }

    @Test
    public void testConstraintNormalization2(){
        IntLVar i = new IntLVar("_I123");
        IntLVar j = new IntLVar("_I1234");
        IntLVar k = new IntLVar("ASD3739");

        Constraint constraint = i.eq(j).or(i.neq(k).and(j.neq(i))).or(i.in(new IntLSet()));
        assertEquals("__I123 = __I1234 OR __I123 neq _ASD3739 AND __I1234 neq __I123 OR __I123 in _?", constraint.toString());
        ConstraintStringNormalizer constraintStringNormalizer = new ConstraintStringNormalizer(constraint);

        assertEquals("__I1 = __I2 OR __I1 neq _ASD3 AND __I2 neq __I1 OR __I1 in _?", constraintStringNormalizer.normalize());
    }

    @Test
    public void testConstraintComparing1(){
        IntLVar i1 = new IntLVar("_I1231");
        IntLVar j1 = new IntLVar("_I12341");

        Constraint constraint1 = i1.eq(j1);
        assertEquals("__I1231 = __I12341", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");

        Constraint constraint2 = i2.eq(j2);
        assertEquals("__I1232 = __I12342", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertTrue(constraintStringComparator.areEqualInSameOrder());
    }

    @Test
    public void testConstraintComparator2(){
        IntLVar i1 = new IntLVar("_I1231");
        IntLVar j1 = new IntLVar("_I12341");
        IntLVar k1 = new IntLVar("ASD37391");

        Constraint constraint1 = i1.eq(j1).or(i1.neq(k1).and(j1.neq(i1))).or(i1.in(new IntLSet()));
        assertEquals("__I1231 = __I12341 OR __I1231 neq _ASD37391 AND __I12341 neq __I1231 OR __I1231 in _?", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");
        IntLVar k2 = new IntLVar("ASD37392");

        Constraint constraint2 = i2.eq(j2).or(i2.neq(k2).and(j2.neq(i2))).or(i2.in(new IntLSet()));
        assertEquals("__I1232 = __I12342 OR __I1232 neq _ASD37392 AND __I12342 neq __I1232 OR __I1232 in _?", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertTrue(constraintStringComparator.areEqualInSameOrder());

    }

    @Test
    public void testConstraintComparing3(){
        IntLVar i1 = new IntLVar("_I1231");

        Constraint constraint1 = i1.eq(i1);
        assertEquals("__I1231 = __I1231", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");

        Constraint constraint2 = i2.eq(j2);
        assertEquals("__I1232 = __I12342", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertFalse(constraintStringComparator.areEqualInSameOrder());
    }

    @Test
    public void testConstraintComparator4(){
        IntLVar i1 = new IntLVar("_I1231");
        IntLVar j1 = new IntLVar("_I12341");
        IntLVar k1 = new IntLVar("ASD37391");

        Constraint constraint1 = i1.eq(j1).or(i1.neq(k1).and(j1.neq(i1))).or(i1.in(new IntLSet()));
        assertEquals("__I1231 = __I12341 OR __I1231 neq _ASD37391 AND __I12341 neq __I1231 OR __I1231 in _?", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");
        IntLVar k2 = new IntLVar("ASfD37392");

        Constraint constraint2 = i2.eq(j2).or(i2.neq(k2).and(j2.neq(i2))).or(i2.in(new IntLSet()));
        assertEquals("__I1232 = __I12342 OR __I1232 neq _ASfD37392 AND __I12342 neq __I1232 OR __I1232 in _?", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertFalse(constraintStringComparator.areEqualInSameOrder());

    }

    @Test
    public void testConstraintComparator5(){
        IntLVar i1 = new IntLVar("_I1231");
        IntLVar j1 = new IntLVar("_I12341");
        IntLVar k1 = new IntLVar("ASD37391");

        Constraint constraint1 = i1.eq(j1).and(new Constraint("_eq",1,2)).or(i1.neq(k1).and(j1.neq(i1))).or(i1.in(new IntLSet()));
        assertEquals("__I1231 = __I12341 AND 1 = 2 OR __I1231 neq _ASD37391 AND __I12341 neq __I1231 OR __I1231 in _?", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");
        IntLVar k2 = new IntLVar("ASD37392");

        Constraint constraint2 = i2.eq(j2).and(new Constraint("_eq", 4, 5)).or(i2.neq(k2).and(j2.neq(i2))).or(i2.in(new IntLSet()));
        assertEquals("__I1232 = __I12342 AND 4 = 5 OR __I1232 neq _ASD37392 AND __I12342 neq __I1232 OR __I1232 in _?", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertFalse(constraintStringComparator.areEqualInSameOrder());

    }

    @Test
    public void testConstraintComparator6(){
        IntLVar i1 = new IntLVar("_I1231");
        IntLVar j1 = new IntLVar("_I12341");
        IntLVar k1 = new IntLVar("ASD37391");

        Constraint constraint1 = i1.eq(j1).and(new Constraint("_eq", 1, 2)).or(i1.neq(k1).and(j1.neq(i1))).or(i1.in(new IntLSet()));
        assertEquals("__I1231 = __I12341 AND 1 = 2 OR __I1231 neq _ASD37391 AND __I12341 neq __I1231 OR __I1231 in _?", constraint1.toString());

        IntLVar i2 = new IntLVar("_I1232");
        IntLVar j2 = new IntLVar("_I12342");
        IntLVar k2 = new IntLVar("ASD37392");

        Constraint constraint2 = i2.eq(j2).and(new Constraint("_eq",1,2)).or(i2.neq(k2).and(j2.neq(i2))).or(i2.in(new IntLSet()));
        assertEquals("__I1232 = __I12342 AND 1 = 2 OR __I1232 neq _ASD37392 AND __I12342 neq __I1232 OR __I1232 in _?", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertTrue(constraintStringComparator.areEqualInSameOrder());

    }

    @Test
    public void testconstraintComparator7(){
        IntLVar i1 = new IntLVar("_?");
        IntLVar j1 = new IntLVar("_?");
        int a1 = 1;
        int b1 = 2;
        Constraint constraint1 = i1.eq(j1).or(i1.neq(j1).and(new Constraint("_neq", a1,b1)));
        assertEquals("__? = __? OR __? neq __? AND 1 neq 2", constraint1.toString());

        IntLVar i2 = new IntLVar("_?");
        IntLVar j2 = new IntLVar("_?");
        int a2 = 1;
        int b2 = 2;
        Constraint constraint2 = i2.eq(j2).or(i2.neq(j2).and(new Constraint("_neq", a2,b2)));
        assertEquals("__? = __? OR __? neq __? AND 1 neq 2", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertTrue(constraintStringComparator.areEqualInSameOrder());
    }

    @Test
    public void testconstraintComparator8(){
        IntLVar i1 = new IntLVar("_?");
        IntLVar j1 = new IntLVar("_?");
        int a1 = 1;
        int b1 = 2;
        Constraint constraint1 = i1.eq(j1).or(i1.neq(j1).and(new Constraint("_neq", a1,b1)));
        assertEquals("__? = __? OR __? neq __? AND 1 neq 2", constraint1.toString());

        IntLVar i2 = new IntLVar("_?");
        IntLVar j2 = new IntLVar("_?");
        int a2 = 2;
        int b2 = 3;
        Constraint constraint2 = i2.eq(j2).or(i2.neq(j2).and(new Constraint("_neq", a2,b2)));
        assertEquals("__? = __? OR __? neq __? AND 2 neq 3", constraint2.toString());

        ConstraintStringComparator constraintStringComparator = new ConstraintStringComparator(constraint1, constraint2);
        assertFalse(constraintStringComparator.areEqualInSameOrder());
    }
}