package jsetl.test;

import jsetl.*;
import jsetl.exception.Failure;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.LMap;
import jsetl.Solver;
import junit.framework.TestCase;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class LMapTest extends TestCase{

    @Before
    public void setUp() {
        LObject.USEJOLLYTOKEN = true;
    }

    @After
    public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						DOMAIN
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     *       r in V
     * --------------------
     * dom(r,r) -- > r = {}
     * 
     * */
    
    @Test
    public void testDomainSameVariableOpen() throws Failure{
        
    	Solver solver = new Solver();
        LMap map  = new LMap();

        solver.add(map.dom(map));
        solver.solve();

        assertTrue(map.isEmpty());
    }
    
    
    /**
     *       empty(r)
     * --------------------      
     * dom(r,a) ---> a = {}
     * 
     * */
    @Test
    public void testDomainEmptyMapOpen() throws Failure{
        Solver solver = new Solver();
        LMap map  = new LMap();
        LSet dom = new LSet();

        map = LMap.empty();

        solver.add(map.dom(dom));
        
        solver.solve();
        
        assertTrue(map.equals(LMap.empty()) && dom.equals(LSet.empty()));
    }
    
    
    /**
     *       empty(a)
     * --------------------      
     * dom(r,a) ---> r = {}
     * 
     * */
    @Test
    public void testDomainEmptyDomOpen() throws Failure{
        Solver solver = new Solver();
        LMap map  = new LMap();
        LSet dom = LSet.empty();

        solver.add(map.dom(dom));
        
        solver.solve();
        
        assertTrue(map.equals(LMap.empty()) && dom.equals(LSet.empty()));
    }

    

    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * dom = {0,1,2,3,4|_?}
     * 
     * dom(map,domControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyDomOpen(){
        Solver solver = new Solver();

        LMap map  = new LMap();
        LSet domControl = new LSet();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(map.dom(domControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * dom = {0,1,2,3,4}
     * 
     * dom(map,domControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyDomClosed(){
        Solver solver = new Solver();

        LMap map  = LMap.empty();
        LSet domControl = LSet.empty();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(map.dom(domControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * dom in V
     * 
     * dom(map,dom)  ---->  dom = {0,1,2,3,4|_?}
     *  
     * */
    @Test
    public void testNotEmptyDomUndefinedOpen() throws Failure{

        Solver solver = new Solver();

        LSet dom = new LSet();
        LMap map  = new LMap();
        LSet domControl = new LSet();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(map.dom(dom));
        solver.solve();
        // dom.output();
        solver.add(dom.eq(domControl));
        solver.solve();
        assertTrue(true);
    }
    
    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * dom in V
     * 
     * dom(map,dom)  ---->  dom = {0,1,2,3,4}
     *  
     * */
    @Test
    public void testNotEmptyDomUndefinedClosed() throws Failure{

        Solver solver = new Solver();

        LSet dom = new LSet();
        LMap map  = new LMap();
        LSet domControl = new LSet();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(map.dom(dom));
        solver.solve();
        // dom.output();
        solver.add(dom.eq(domControl));
        solver.solve();
        assertTrue(true);
    }
    
      
    /**
     *  map = {?}
     *  dom = {4,3,2,1,0|_?}
     * 
     *  dom(map,dom)  ------>  map = {(4,_?),(3,_?),(2,_?),(1,_?),(0,_?)|_?}
     *  
     * */

    @Test
    public void testNotEmptyMapUndefinedOpen() throws Failure{
        
    	Solver solver = new Solver();
        
    	LMap map  = new LMap();
        LMap mapControl = new LMap(); 
        LSet dom = new LSet();

        for(int i = 0 ; i<3; ++i){
            dom = dom.ins(i);
            mapControl = mapControl.ins(new LPair(i,new LVar()));
        }

        solver.add(map.dom(dom));
        solver.solve();
        //map.output();
        //mapControl.output();
        solver.add(map.eq(mapControl));
        solver.solve();

    }
    
    
    
    /**
     *  map = {?}
     *  dom = {4,3,2,1,0}
     * 
     *  dom(map,dom)  ------>  map = {(4,_?),(3,_?),(2,_?),(1,_?),(0,_?)}
     *  
     * */

    @Test
    public void testNotEmptyMapUndefinedClose() throws Failure{
        
    	Solver solver = new Solver();
        
    	LMap map  = new LMap();
        LMap mapControl = LMap.empty(); 
        LSet dom = LSet.empty();

        for(int i = 0 ; i<3; ++i){
            dom = dom.ins(i);
            mapControl = mapControl.ins(new LPair(i,new LVar()));
        }

        solver.add(map.dom(dom));
        solver.solve();
        map.output();
        //mapControl.output();
        solver.add(map.eq(mapControl));
        solver.solve();

    }
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						RANGE
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     *       r in V
     * --------------------
     * ran(r,r) -- > r = {}
     * 
     * */
    
    @Test
    public void testRangeSameVariableOpen() throws Failure{
        
    	Solver solver = new Solver();
        LMap map  = new LMap();

        solver.add(map.ran(map));
        solver.solve();

        assertTrue(map.isEmpty());
    }
    
    
    /**
     *       empty(r)
     * --------------------      
     * ran(r,a) ---> a = {}
     * 
     * */
    @Test
    public void testRangeEmptyMapOpen() throws Failure{
        
    	Solver solver = new Solver();
        LMap map  = new LMap();
        LSet ran = new LSet();

        map = LMap.empty();

        solver.add(map.ran(ran));
        
        solver.solve();
        
        assertTrue(map.equals(LMap.empty()) && ran.equals(LSet.empty()));
    }
    
    
    /**
     *       empty(a)
     * --------------------      
     * ran(r,a) ---> r = {}
     * 
     * */
    @Test
    public void testRangeEmptyRanOpen() throws Failure{
        
    	Solver solver = new Solver();
        LMap map  = new LMap();
        LSet ran = LSet.empty();
        
        solver.add(map.ran(ran));
        
        solver.solve();
        
        assertTrue(map.equals(LMap.empty()) && ran.equals(LSet.empty()));
    }

    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * ran = {0,1,2,3,4|_?}
     * 
     * ran(map,ranControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyRanOpen(){
        Solver solver = new Solver();

        LMap map  = new LMap();
        LSet ranControl = new LSet();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(i);
        }

        solver.add(map.dom(ranControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * ran = {0,1,2,3,4}
     * 
     * ran(map,ranControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyRanClosed(){
        Solver solver = new Solver();

        LMap map  = LMap.empty();
        LSet ranControl = LSet.empty();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(i);
        }

        solver.add(map.dom(ranControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }
    
    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * ran in V
     * 
     * ran(map,ran) ----> ran = {1,2,3,4,5|_?}
     * 
     * */
    
    @Test
    public void testNotEmptyRanUndefinedOpen() throws Failure{
        
    	LSet ran = new LSet();
        LMap map  = new LMap();
        LSet ranControl = new LSet();
        Solver solver = new Solver();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(new IntLVar(i+1));
        }


        solver.add(map.ran(ran));
        solver.solve();
        solver.add(ran.eq(ranControl));
        solver.solve();

        assertTrue(true);
    }
    
    
    /**
     * map = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * ran in V
     * 
     * ran(map,ran) ----> ran = {1,2,3,4,5}
     * 
     * */
    
    @Test
    public void testNotEmptyRanUndefinedClosed() throws Failure{
        
    	LSet ran = new LSet();
        LMap map  = new LMap();
        LSet ranControl = new LSet();
        Solver solver = new Solver();

        for(int i = 0 ; i<3; ++i){
            map = map.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(new IntLVar(i+1));
        }


        solver.add(map.ran(ran));
        solver.solve();
        solver.add(ran.eq(ranControl));
        solver.solve();

        assertTrue(true);
    }
    
    
    /**
     * map in V
     * 
     * a = {1}
     * 
     * ran(map,a) ----> map = {[_?,1]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_closed_only_one_elem(){
    	
    	Solver solver = new Solver();
		
		LMap map = new LMap();//.setName("map");
		LSet a = LSet.empty().ins(1).setName("a");
				
		LMap control = new LMap().ins(new LPair(new LVar(),1));
		
		
		try {
			solver.add(map.ran(a));
			solver.solve();
			
			assertTrue(control.toString().equals(map.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * map in V
     * 
     * a = {1,2,3}
     * 
     * ran(map,a) ----> map = {[_?,1],[_?,2],[_?,3]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_closed_more_elems(){
    	
    	Solver solver = new Solver();
		
		LMap map = new LMap();//.setName("map");
		LSet a = LSet.empty().ins(1).ins(2).ins(3).setName("a");
				
		LMap control = new LMap().ins(new LPair(new LVar(),1)).ins(new LPair(new LVar(),2)).ins(new LPair(new LVar(),3));
		//System.out.println(control.toString());
		
		try {
			solver.add(map.ran(a));
			solver.solve();
			//System.out.println(map.toString());
			assertTrue(control.toString().equals(map.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * map in V
     * 
     * a = {1| _?}
     * 
     * ran(map,a) ----> map = {[_?,1]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_open_only_one_elem(){
    	
    	Solver solver = new Solver();
		
		LMap map = new LMap();//.setName("map");
		LSet a = new LSet().ins(1).setName("a");
				
		LMap control = new LMap().ins(new LPair(new LVar(),1));
		
		
		try {
			solver.add(map.ran(a));
			solver.solve();
			
			assertTrue(control.toString().equals(map.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    
    /**
     * map in V
     * 
     * a = {1,2,3}
     * 
     * ran(map,a) ----> map = {[_?,1],[_?,2],[_?,3]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_open_more_elems(){
    	
    	Solver solver = new Solver();
		
		LMap map = new LMap();//.setName("map");
		LSet a = new LSet().ins(1).ins(2).ins(3).setName("a");
				
		LMap control = new LMap().ins(new LPair(new LVar(),1)).ins(new LPair(new LVar(),2)).ins(new LPair(new LVar(),3));
		//System.out.println(control.toString());
		
		try {
			solver.add(map.ran(a));
			solver.solve();
			//System.out.println(map.toString());
			assertTrue(control.toString().equals(map.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						IS GROUND
     * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     */
    
    
    /**
     * map = {(1,null),(2,null)}
     * 
     * isGround(map) => true
     * */
    @Test
    public void testDomainGroundMapAllLVarClose() throws InvocationTargetException, IllegalAccessException, NoSuchMethodException {
        LMap map = LMap.empty().ins(new LPair(new LVar(1), new LVar())).ins(new LPair(new LVar(2), new LVar()));
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    
    /**
     * map = {(1,null),(2,null)}
     * 
     * isGround(map) => true
     * */
    @Test
    public void testDomainGroundMapNotLVarClose()throws InvocationTargetException, IllegalAccessException, NoSuchMethodException{
        LMap map = LMap.empty().ins(new LPair(1, new LVar())).ins(new LPair(new LVar(2), new LVar()));
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    /**
     * map = {(1,null),(null,null)}
     * 
     * isGround(map) => false
     * */
    @Test
    public void testNotDomainGroundMapClose()throws InvocationTargetException, IllegalAccessException, NoSuchMethodException{
        LMap map = LMap.empty().ins(new LPair(1, new LVar())).ins(new LPair());
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione non ground", result);
    }
    
    
    /**
     * map = {(1,2),(2,3)|_?}
     * 
     * isGround(map) => true
     * */
    @Test
    public void testDomainGroundMapAllLVarOpen()throws InvocationTargetException, IllegalAccessException, NoSuchMethodException{
        LMap map = new LMap();
        map = LMap.empty().ins(new LPair(new LVar(1),2)).ins(new LPair(new LVar(2),3));
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    /**
     * map = {(1,null),(2,null)|_?}
     * 
     * isGround(map) => true
     * */
    @Test
    public void testDomainGroundMapNotLVarOpen()throws InvocationTargetException, IllegalAccessException, NoSuchMethodException{
        LMap map = new LMap();
        map = map.ins(new LPair(1, new LVar())).ins(new LPair(new LVar(2), new LVar()));
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        //map.output();

        assertEquals("Funzione non ground", result);
    }
    
    /**
     * map = {(1,2),(null,null)|_?}
     * 
     * isGround(map) => false
     * */
    @Test
    public void testNotDomainGroundMapOpen()throws InvocationTargetException, IllegalAccessException, NoSuchMethodException{
        LMap map = new LMap();
        map = map.ins(new LPair(1,2)).ins(new LPair());
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LMap.class, map)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione non ground", result);
    }
    
    

    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						COMP
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     * R = {}
     * S in V
     * Q in V
     * 
     * comp(R,S,q) ===>>> q = {}
     */
    
    public void testCompR_Empty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r = LMap.empty();
    	LMap s = new LMap();
    	LMap q = new LMap();
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	assertTrue(r.equals(q));
    }
    
    
    /**
     * R in V
     * S = {}
     * Q in V
     * 
     * comp(R,S,q) ===>>> q = {}
     */
    
    public void testCompS_Empty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r = new LMap();
    	LMap s = LMap.empty();
    	LMap q = new LMap();
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	assertTrue(s.equals(q));
    }
    
    /**
     * R = {(1,2),(3,4)}
     * S = {(5,6),(7,8)}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> SOLVED
     */
    
    public void testCompQ_EmptyClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = LMap.empty().ins(new LPair(5,6)).ins(new LPair(7,8));
    	LMap q = LMap.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertTrue(false);
    	}
    	
    	assertTrue(true);
    }
    
    
    /**
     * R = {(1,2),(3,4)|_?}
     * S = {(5,6),(7,8)|_?}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> SOLVED
     */
    
    public void testCompQ_EmptyOpen() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r = new LMap().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = new LMap().ins(new LPair(5,6)).ins(new LPair(7,8));
    	LMap q = LMap.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertTrue(false);
    	}
    	
    	assertTrue(true);
    }
    
    
    /**
     * R = {(1,2),(3,4)}
     * S = {(2,6),(4,8)}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> FAILURE
     */
    
    public void testCompQ_EmptyClosedFailure() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = LMap.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = LMap.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertFalse(false);
    	}
    	
    	
    }
    
    /**
     * R in V
     * S in V
     * Q = {(1,6),(3,8)}
     * 
     * comp(R,S,Q) ===>>>  R={[3,_?],[1,_?]|_?}   S={[_?,8],[_?,6]|_?}
     */
    
    public void testCompQ_NotEmptyClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r_Control= new LMap().ins(new LPair(1,new LVar())).ins(new LPair(3,new LVar()));
    	LMap s_Control= new LMap().ins(new LPair(new LVar(),6)).ins(new LPair(new LVar(),8));
    	
    	LMap r = new LMap();
    	LMap s = new LMap();
    	LMap q = LMap.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(r.eq(r_Control));
    		solver.add(s.eq(s_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    
    }
    
    /**
     * R in V
     * S in V
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>>  R={[3,_?],[1,_?]|_?}   S={[_?,8],[_?,6]|_?}
     */
 
    public void testCompQ_NotEmptyOpen() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r_Control= new LMap().ins(new LPair(1,new LVar())).ins(new LPair(3,new LVar()));
    	LMap s_Control= new LMap().ins(new LPair(new LVar(),6)).ins(new LPair(new LVar(),8));
    	
    	LMap r = new LMap();
    	LMap s = new LMap();
    	LMap q = new LMap().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(r.eq(r_Control));
    		solver.add(s.eq(s_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
 
    
    /**
     * R in {(1,2),(3,4)|_?}
     * S in V
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> S={[4,8],[1,6]|_?}
     */
    
    public void testCompQ_NotEmpty_R_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap s_Control= new LMap().ins(new LPair(2,6)).ins(new LPair(4,8));
    	
    	LMap r = new LMap().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = new LMap();
    	LMap q = new LMap().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		//assertTrue(s.equals(s_Control));
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in V
     * S in {(2,6),(4,8)|_?}
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> R={[1,2],[3,4]|_?}
     */
    
    public void testCompQ_NotEmpty_S_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap r_Control= new LMap().ins(new LPair(1,2)).ins(new LPair(3,4));
    	
    	LMap r = new LMap();
    	LMap s = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = new LMap().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	    		
    		//assertTrue(r.equals(r_Control));
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    /**
     * R in {(1,2),(3,4)|_?}
     * S in {(2,6),(4,8)|_?}
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> SOLVED
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LMap r = new LMap().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = new LMap().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	try {
			
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = {(1,6),(3,8)}
     * 
     * comp(R,S,Q) ===>>> SOLVED
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmptyandClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = LMap.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = LMap.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	try {
			
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = {(9,6),(7,8)}
     * 
     * comp(R,S,Q) ===>>> FAILURE
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmptyandClosed_Fail() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = LMap.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = LMap.empty().ins(new LPair(9,6)).ins(new LPair(7,8));
    	
    	
    	try {
			
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		assertTrue(false);
		} catch (Failure e) {
			assertTrue(true);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)|_?}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={(1,6),(3,8)|_?}
     */
    
    public void testCompQ_NotInit() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap q_Control= new LMap().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = new LMap();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={(1,6),(3,8)}
     */
    
    public void testCompQ_NotInitClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LMap q_Control= LMap.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	LMap r = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LMap s = LMap.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LMap q = new LMap();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 				UNION
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    /**
     * M1 = {(1,2),(3,4)}
     * M2 = {(2,6),(4,8)|_?}
     * M3 in V
     * 
     * un(M1,M2,M3) ==>> M3={(1,2),(3,4),(2,6),(4,8)|_?}
     */
    
    @Test
    public void testLMap_un() {
    	
    	Solver solver = new Solver();
    	
    	LMap m1 = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4)).setName("m1");
		LMap m2 = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8)).setName("m2");
		LMap m3 = new LMap();
		
		LMap control = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m3.union(m1, m2));
			solver.add(m3.eq(control));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * M1 = {(1,2),(3,4)}
     * M2 in V
     * M3 = {(1,2),(3,4),(2,6),(4,8)|_?}
     * 
     * un(M1,M2,M3) ==>> M2 = {(2,6),(4,8)|_?}
     */
    
    @Test
    public void testLMap_un_M2notInit() {
    	
    	Solver solver = new Solver();
    	
    	LMap m1 = LMap.empty().ins(new LPair(1,2)).ins(new LPair(3,4)).setName("m1");
		LMap control = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8)).setName("m2");
		LMap m2 = new LMap();
		
		LMap m3 = new LMap().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m3.union(m1, m2));
			solver.add(m2.eq(control));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }

    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						IS PFUN
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     * map = {(1,null),(2,null)}
     * 
     * pfun(map) => true
     * */
    @Test
    public void testPfunGroundDomainClosed() throws Failure{
        LRel r = LRel.empty().ins(new LPair(1, new LVar())).ins(new LPair(2, new LVar()));
        LMap map = new LMap();
        Solver solver = new Solver();
        solver.add(map.eq(r));
        solver.add(map.pfun());
        solver.solve();
        assertTrue(true);
    }
    
    /**
     * map = {(1,null),(2,null)|_?}
     * 
     * pfun(map) => true
     * */
    @Test
    public void testPfunGroundDomainOpen() throws Failure{
        LRel r = new LRel().ins(new LPair(1, new LVar())).ins(new LPair(2, new LVar()));
        LMap map = new LMap();
        Solver solver = new Solver();
        solver.add(map.eq(r));
        solver.add(map.pfun());
        solver.solve();
        assertTrue(true);
    }
    
    /**
     * map = {}
     * 
     * pfun(map) => true
     * 
     * */
    @Test
    public void testPfunEmpty() throws Failure{
        LMap map = LMap.empty();
        Solver solver = new Solver();
        
        solver.add(map.pfun());
        solver.solve();
        assertTrue(true);
    }
    
    
    /**
     * map = {(1,5),(A,B)}
     * 
     * pfun(map) => true
     * 
     * */
    
    @Test
    public void testPfunNotGroundDomainClose() throws Failure{
        LMap map = new LMap();
    	LRel r = LRel.empty().ins(new LPair(1,new LVar(5))).ins(new LPair(new LVar("A"),new LVar("B")));
        
        Solver solver = new Solver();
        
        solver.add(map.eq(r));
        solver.add(map.pfun());
        assertTrue(true);
    }
    
    
    /**
     * map = {(1,5),(A,B)}
     * 
     * pfun(map) => true
     * 
     * */
    

    @Test
    public void testPfunNotGroundDomainOpen() throws Failure{
        
        LMap map = new LMap();
        LRel r = new LRel().ins(new LPair(1,new IntLVar(5))).ins(new LPair(new LVar(1),new IntLVar()));
        Solver solver = new Solver();

        solver.add(map.eq(r));
        solver.add(map.pfun());
        solver.solve();
        assertTrue(true);
    }
    
    
    /**
     * map = {(1,5),(1,_?)}
     * 
     * pfun(map) => true
     * 
     * */
    

    @Test
    public void testPfunFail() throws Failure{
        
        LMap map = new LMap();
        LRel r = LRel.empty().ins(new LPair(1,5)).ins(new LPair(1,"a"));
        Solver solver = new Solver();

        solver.add(map.eq(r));
        solver.add(map.pfun());
        try {
        	solver.solve();
        	assertTrue(false);
        }catch(Failure f) {
        	assertTrue(true);
        }
    }
 
    
    /**
     * {log}=> pfun({[a,1],[a,2]}).
     *  no.
     * 
     * */
    @Test
    public void testPfunGroundFailSetlog()throws Failure{
        Solver solver = new Solver();
        LMap map;
        
        try {
        map = LMap.empty().ins(new LPair('a',1)).ins(new LPair('a',2));
        
            solver.add(map.pfun());
            solver.solve();
        }catch(Exception f){
            assertTrue(true);
        }
    }
    
    /**
     * {log}=> pfun({[a,1],[a,2]}|_?).
     *  no.
     * 
     * */
    @Test
    public void testPfunGroundFailSetlog2()throws Failure{
        Solver solver = new Solver();
        LMap map = new LMap();
        
        try {
        map.ins(new LPair('a',1)).ins(new LPair('a',2));
        
        
            solver.add(map.pfun());
            solver.solve();
        }catch(Exception f){
            assertTrue(true);
        }
    }
    
    /**********************************************************/
    
    @Test
    public void testInsPfunCheck() throws Failure{
    	
    	LVar x1 = new LVar("x1");
    	LVar x2 = new LVar("x2");
    	LVar y1 = new LVar("y1");
    	LVar y2 = new LVar("y2");
    	
    	LPair p1 = new LPair(x1, y1);
    	LPair p2 = new LPair(x2, y2);
    	
    	LMap m = LMap.empty().ins(p1).ins(p2);
    	
    	Solver solver = new Solver();
    	
    	solver.add(m.dom(LSet.empty().ins(x1)));
    	solver.add(y1.neq(y2));
    	
    	solver.showStore();
    	
    	assertFalse(solver.check());
    }
}
