package jsetl.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import jsetl.*;
import jsetl.exception.Failure;
import junit.framework.TestCase;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;


/**
 *  @author Michael Cobianchi
 */

public class LPairTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}
    
    /**&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
     					CONSTRAINTS
     &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     *			allDiff 
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    @Test
    public void testPair_allDiff_equals() {
    	
    	LPair p = new LPair(2,2);
    	Solver solver = new Solver();
    	
    	try {
			solver.add(Constraint.allDifferent(p.toArray()));
			solver.solve();
			
			assertTrue(false);
		} catch (Failure e) {
			assertTrue(true);
		}
    	
    	
    }
    
    @Test
    public void testPair_allDiff_different() {
    	
    	LPair p = new LPair(1,2);
    	Solver solver = new Solver();
    	
    	try {
			solver.add(Constraint.allDifferent(p.toArray()));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			  EQ
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    @Test
    public void testPair_eq_P1_notInit() {
    	
    	LPair p1 = new LPair();
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.eq(p2));
			solver.solve();
			
			assertTrue(p1.equals(p2));
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    @Test
    public void testPair_eq_P1_InitEquals() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.eq(p2));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    @Test
    public void testPair_eq_P1_InitNotEquals() {
    	
    	LPair p1 = new LPair(3,2);
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.eq(p2));
			solver.solve();
			
			assertFalse(true);
		} catch (Failure e) {
			assertFalse(false);
		}
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			  NEQ
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    @Test
    public void testPair_neq_P1_notInit() {
    	
    	LPair p1 = new LPair();
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.neq(p2));
			solver.solve();
			
			assertTrue(!p1.equals(p2));
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    @Test
    public void testPair_neq_P1_InitEquals() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.neq(p2));
			solver.solve();
			
			assertFalse(true);
		} catch (Failure e) {
			assertFalse(false);
		}
    }
    
    @Test
    public void testPair_neq_P1_InitNotEquals() {
    	
    	LPair p1 = new LPair(3,2);
    	LPair p2 = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p1.neq(p2));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**%%%%%%%%%%%%%%%%%%%%
     * 			IN
     %%%%%%%%%%%%%%%%%%%%*/
    
    public void testPair_in_true() {
    	
    	LRel m = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LPair p = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p.in(m));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    public void testPair_in_false() {
    	
    	LRel m = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LPair p = new LPair(5,8);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p.in(m));
			solver.solve();
			
			assertTrue(false);
		} catch (Failure e) {
			assertTrue(true);
		}
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%
     * 		  NIN
     %%%%%%%%%%%%%%%%%%%%*/
    
    public void testPair_nin_false() {
    	
    	LRel m = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LPair p = new LPair(1,2);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p.nin(m));
			solver.solve();
			
			assertTrue(false);
		} catch (Failure e) {
			assertTrue(true);
		}
    }
    
    public void testPair_nin_true() {
    	
    	LRel m = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LPair p = new LPair(5,8);
    	
    	Solver solver = new Solver();
    	
    	try {
			solver.add(p.nin(m));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
     *   		END CONSTRAINTS
     &&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
    
    
    /**%%%%%%%%%%%%%%%%%%%%%		
     		GET NAME	
	 %%%%%%%%%%%%%%%%%%%%*/
    
    
	@Test
	public void testPairGetNameWithName() {
		LPair p = new LPair();
		p.setName("x");
		assertTrue("x".equals(p.getName()));
	}

	@Test
	public void testPairGetNameWithNoName() {
		LPair p = new LPair();
		assertTrue("?".equals(p.getName()));
	}
	
	/**%%%%%%%%%%%%%%%%%%%%%		
     *         SETNAME	
	%%%%%%%%%%%%%%%%%%%%%%%*/

	@Test
	public void testSetNameWithNotName() {
		LPair p = new LPair();
        p.setName("x_new");
        assertTrue("x_new".equals(p.getName()));
	}
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 					TO STRING
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    
    //%%%%%% PAIR UNINITIALIZED
    
    @Test
    public void testToStringUninitialized() {
    	LPair p = new LPair();
    	
    	assertEquals("_?",p.toString());
    }
    
    
    //%%%%%% PAIR WITH FIRST EL. INITIALIZED E WITH A INTEGER
    
    @Test
    public void testToStringFirstInitializedWithAInteger() {
    	
    	LPair p = new LPair(1, new LVar());
    	
    	assertEquals("(1,_?)", p.toString());
    }
    
    
    //%%%%%% PAIR WITH FIRST EL. INITIALIZED WITH A STRING
    
    @Test
    public void testToStringFirstInitializedWithAString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s);
    	
    	assertEquals("_"+s, p.toString());
    }
    
    
    //%%%%%% PAIR WITH FIRST EL. INITIALIZED WITH A UNINITIALIZED LVAR
    
    @Test
    public void testToStringFirstInitializedWithAUninitializedLVar() {
    	
    	LPair p = new LPair(new LVar(), new LVar());
    	
    	assertEquals("(_?,_?)", p.toString());
    }
    
    
    //%%%%%% PAIR WITH FIRST EL. INITIALIZED WITH A INITIALIZED LVAR
    
    @Test
    public void testToStringFirstInitializedWithAInitilalizedLVar() {
    	
    	LPair p = new LPair(new LVar(1), new LVar());
    	
    	assertEquals("(1,_?)", p.toString());
    }
    
    //%%%%%% PAIR INITIALIZED WITH TWO INTEGERS
	
    @Test
    public void testToStringInitializedWith2Integers() {
    	
    	LPair p = new LPair(1,2);
    	
    	assertEquals("(1,2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INTEGER AND SECOND STRING
	
    @Test
    public void testToStringInitializedWithFirstIntegerAndSecondString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(1,s);
    	
    	assertEquals("(1,"+s+")",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INTEGER AND SECOND UNINITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstIntegerAndSecondUninitializedLvar() {
    	
    	LPair p = new LPair(1,new LVar());
    	
    	assertEquals("(1,_?)",p.toString());
    }
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INTEGER AND SECOND INITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstIntegerAndSecondInitializedLvar() {
    	
    	LPair p = new LPair(1,new LVar(2));
    	
    	assertEquals("(1,2)",p.toString());
    }
    
    //%%%%%% PAIR INITIALIZED WITH FIRST STRING AND SECOND INTEGER
	
    @Test
    public void testToStringInitializedWithFirstStringAndSecondInteger() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,2);
    	
    	assertEquals("("+s+",2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST STRING AND SECOND STRING
	
    @Test
    public void testToStringInitializedWithFirstStringAndSecondString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p = new LPair(s1,s2);
    	
    	assertEquals("("+s1+","+s2+")",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST STRING AND SECOND UNINITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstStringAndSecondUninitilaizedLvar() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,new LVar());
    	
    	assertEquals("("+s+",_?)",p.toString());
    }
    
 
    //%%%%%% PAIR INITIALIZED WITH FIRST STRING AND SECOND INITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstStringAndSecondInitilaizedLvar() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,new LVar(2));
    	
    	assertEquals("("+s+",2)",p.toString());
    }
   

    //%%%%%% PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND INTEGER
	
    @Test
    public void testToStringInitializedWithFirstInitializedLvarAndSecondInteger() {
    	
    	LPair p = new LPair(new LVar(1),2);
    	
    	assertEquals("(1,2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND STRING
	
    @Test
    public void testToStringInitializedWithFirstInitializedLvarAndSecondString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(new LVar(1),s);
    	
    	assertEquals("(1,"+s+")",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND UNINITILAIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstInitializedLvarAndSecondUninitilaizedLvar() {
    	
    	LPair p = new LPair(new LVar(1),new LVar());
    	
    	assertEquals("(1,_?)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND INITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstInitializedLvarAndSecondInitilaizedLvar() {
    	
    	LPair p = new LPair(new LVar(1),new LVar(2));
    	
    	assertEquals("(1,2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND INTEGER
	
    @Test
    public void testToStringInitializedWithFirstUninitializedLvarAndSecondInteger() {
    	
    	LPair p = new LPair(new LVar(),2);
    	
    	assertEquals("(_?,2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND STRING
	
    @Test
    public void testToStringInitializedWithFirstUninitializedLvarAndSecondString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(new LVar(),s);
    	
    	assertEquals("(_?,"+s+")",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND INITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstUninitializedLvarAndSecondInitialized() {
    	
    	LPair p = new LPair(new LVar(),new LVar(2));
    	
    	assertEquals("(_?,2)",p.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND UNINITIALIZED LVAR
	
    @Test
    public void testToStringInitializedWithFirstUninitializedLvarAndSecondUninitializedLvar() {
    	
    	LPair p = new LPair(new LVar(),new LVar());
    	
    	assertEquals("(_?,_?)",p.toString());
    }
    
    //%%%%%% PAIR INITIALIZED WHITH PAIR INITIALIZED
    
    @Test
    public void testToStringInitializedWithPairInitialized() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(p1);
    	
    	assertEquals(p1.toString(), p2.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WHITH PAIR FIRST INITIALIZED
    
    @Test
    public void testToStringInitializedWithPairFirstInitialized() {
    	
    	LPair p1 = new LPair(1, new LVar());
    	LPair p2 = new LPair(p1);
    	
    	assertEquals(p1.toString(), p2.toString());
    }
    
    
    //%%%%%% PAIR INITIALIZED WHITH PAIR FIRST UNINITIALIZED
    
    @Test
    public void testToStringInitializedWithPairFirstUninitialized() {
    	
    	LPair p1 = new LPair();
    	LPair p2 = new LPair(p1);
    	
    	assertEquals(p1.toString(), p2.toString());
    }
    
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 					GET
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    //%%%%%% PAIR GET FIRST ELEM.
    
    @Test
    public void testPairGetFirst() {
    	
    	LPair p = new LPair(1,2);
    	
    	assertEquals(1, p.getFirst());
    }
    
    
    //%%%%%% PAIR GET SECOND ELEM.
    
    @Test
    public void testPairGetSecond() {
    	
    	LPair p = new LPair(1,2);
    	
    	assertEquals(2, p.getSecond());
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 					INITIALIZED
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    //%%%%%% PAIR FIRST EL. IS INIT
    
    @Test
    public void testPairIsFirstInit() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
    	
    	LPair p = new LPair(1, new LVar());
    	Method isFirstInit = LPair.class.getDeclaredMethod("isFirstInit");
    	isFirstInit.setAccessible(true);
    	assertTrue((Boolean) isFirstInit.invoke(p));
    }
    
    
    //%%%%%% PAIR SECOND EL. IS INIT
    
    @Test
    public void testPairIsSecondInit() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException{
    	
    	LPair p = new LPair(1,2);

		Method isSecondInit = LPair.class.getDeclaredMethod("isSecondInit");
		isSecondInit.setAccessible(true);
		assertTrue((Boolean) isSecondInit.invoke(p));
    }
    
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 					IS BOUND
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    //%%%%%% PAIR UNINITIALIZED IS BUOND
    
    @Test
    public void testPairUninizializedIsBound() {
    	
    	LPair p = new LPair();
    	
    	assertFalse(p.isBound());
    }
    
    
    //%%%%%% PAIR INITIALIZED IS BOUND
    
    @Test
    public void testPairInitializedIsBuond() {
    	
    	LPair p = new LPair(1,2);
    	
    	assertTrue(p.isBound());
    }
    
    //%%%%%% PAIR CLONED
  	@Test
  	public void testPairIsBoundClone() {
  		 LPair p1 = new LPair(1,2);
//  		 LVar p = p1.visit();
  		 LPair p = p1.clone();
  	     assertTrue(p.isBound());
  	}
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 				GET VALUE
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    //%%%%% PAIR UNINITIALIZED GET VALUE
    
    @Test
    public void testPairUnitializedGetValue() {
    	
    	LPair p = new LPair();
    	
    	assertEquals(null, p.getValue());
    }
    
    
    //%%%%%% GET VALUE PAIR WITH TWO INTEGERS
    
    @Test
    public void testPairGetValueWithFirstIntegerAndSecondInteger() {
    	
    	LPair p = new LPair(1,2);
    	
    	assertEquals("[1, 2]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INTEGER AND SECOND STRING
    
    @Test
    public void testPairGetValueWithFirstIntegerAndSecondString() {
    	
    	String s =new String("hello");
    	LPair p = new LPair(1,s);
    	
    	assertEquals("[1, "+s+"]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INTEGER AND SECOND UNINITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstIntegerAndSecondUninitializedLvar() {
    	
    	LPair p = new LPair(1,new LVar());
    	
    	assertEquals("[1, _?]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INTEGER AND SECOND INITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstIntegerAndSecondInitializedVar() {
    	
    	LPair p = new LPair(1,new LVar(2));
    	
    	assertEquals("[1, 2]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST STRING AND SECOND INTEGER
    
    @Test
    public void testPairGetValueWithFirstStringAndSecondInteger() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,2);
    	
    	assertEquals("["+s+", 2]", p.getValue().toString());
    }
    
    

    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST STRING AND SECOND STRING
    
    @Test
    public void testPairGetValueWithFirstStringAndSecondString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p = new LPair(s1,s2);
    	
    	assertEquals("["+s1+", "+s2+"]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST STRING AND SECOND UNINITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstStringAndSecondUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,new LVar());
    	
    	assertEquals("["+s+", _?]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST STRING AND SECOND INITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstStringAndSecondInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(s,new LVar(2));
    	
    	assertEquals("["+s+", 2]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND INTEGER
    
    @Test
    public void testPairGetValueWithFirstInitializedLvarAndSecondInteger() {
    	
    	LPair p = new LPair(new LVar(1),2);
    	
    	assertEquals("[1, 2]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND STRING
    
    @Test
    public void testPairGetValueWithFirstInitializedLvarAndSecondString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(new LVar(1),s);
    	
    	assertEquals("[1, "+s+"]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND INITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstInitializedLvarAndSecondInitializedLvar() {
    	
    	LPair p = new LPair(new LVar(1),new LVar(2));
    	
    	assertEquals("[1, 2]", p.getValue().toString());
    }
    

    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST INITIALIZED LVAR AND SECOND UNINITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstInitializedLvarAndSecondUninitializedLvar() {
    	
    	LPair p = new LPair(new LVar(1),new LVar());
    	
    	assertEquals("[1, _?]", p.getValue().toString());
    }
    

    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND INTEGER
    
    @Test
    public void testPairGetValueWithFirstUninitializedLvarAndSecondInteger() {
    	
    	LPair p = new LPair(new LVar(),2);
    	
    	assertEquals("[_?, 2]", p.getValue().toString());
    }
    
    
    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND STRING
    
    @Test
    public void testPairGetValueWithFirstUninitializedLvarAndSecondString() {
    	
    	String s = new String("hello");
    	LPair p = new LPair(new LVar(),s);
    	
    	assertEquals("[_?, "+s+"]", p.getValue().toString());
    }
    

    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND INITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstUninitializedLvarAndSecondInitializedLvar() {
    	
    	LPair p = new LPair(new LVar(),new LVar(2));
    	
    	assertEquals("[_?, 2]", p.getValue().toString());
    }
    

    //%%%%%% GET VALUE PAIR INITIALIZED WITH FIRST UNINITIALIZED LVAR AND SECOND UNINITIALIZED LVAR
    
    @Test
    public void testPairGetValueWithFirstUninitializedLvarAndSecondUninitializedLvar() {
    	
    	LPair p = new LPair(new LVar(),new LVar());
    	
    	assertEquals("[_?, _?]", p.getValue().toString());
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						EQUALS
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    //%%%%%% EQUALS PAIR INITIALIZED AND LSET
    
    @Test
    public void testEqualsPairInitializedWithLset() {
    	
    	LPair p = new LPair(1, 2);
    	LSet s1 = LSet.empty().ins(1).ins(2);
    	LSet s2 = LSet.empty().ins(p);
    	
    	assertFalse(p.equals(s1) && p.equals(s2));
    }
    
    
    //%%%%%% EQUALS PAIR() AND LSET
    
    @Test
    public void testEqualsPairUninitializedWithLset() {
    	
    	LPair p = new LPair();
    	LSet s1 = new LSet();
    	LSet s2 = LSet.empty();
    	
    	assertFalse(p.equals(s1) && p.equals(s2));
    }
   
    //%%%%%% EQUALS PAIR() AND PAIR()
    
    @Test
    public void testEqualsPairUninitializedWithPairUninitialized() {
    	
    	LPair p1 = new LPair();
    	LPair p2 = new LPair();
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,2);
    	LPair p3 = new LPair(2,1);
    	
    	assertTrue(p1.equals(p2) && !p2.equals(p3));
    }
    
    
    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerIntegerWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,2);
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairIntegerStringWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairIntegerStringWithPairIntegerString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(1,s1);
    	LPair p2 = new LPair(1,s1);
    	LPair p3 = new LPair(1,s2);
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairIntegerInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(1,new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairIntegerStringWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(1,s2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairIntegerStringWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(s,new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerStringWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerStringWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(1),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerStringWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerStringWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerStringWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,s);
    	LPair p2 = new LPair(new LVar(),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerUninitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(1,2);
    	
    	assertTrue(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(s));
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(1,new LVar(2));
    	LPair p3 = new LPair(1,new LVar(3));
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(1,new LVar(s2));
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(1));
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(s));
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }

    
    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar(s));
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairIntegerInitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(1,new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    
     //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairStringIntegerWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairStringIntegerWithPairIntegerString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairIntegerInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairStringIntegerWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairStringIntegerWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringIntegerWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringIntegerWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringIntegerWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringIntegerWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INTEGER) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringIntegerWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,2);
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairStringStringWithPairIntegerInteger() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairStringStringWithPairIntegerString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p3 = new LPair(1,s2);
    	
    	assertFalse(p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairIntegerInitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairIntegerUninitializedLavr() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairStringStringWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	String s3 = new String("mondo");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(s1,s2);
    	LPair p3 = new LPair(s1,s3);
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairStringStringWithPairStringInteger() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(s1,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairStringUninitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(s1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairStringInitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(s1,new LVar(s2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringStringWithPairInitializedLvarInteger() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringStringWithPairInitializedLvarString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("hello");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(1),s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairInitializedLvarUninitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairInitializedLvarInitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("hello");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(1),new LVar(s2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringStringWithPairUninitializedLvarInteger() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringStringWithPairUninitializedLvarString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(),s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,STRING) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringStringWithPairUninitializedLvarInitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,s2);
    	LPair p2 = new LPair(new LVar(),new LVar(s2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairIntegerInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,new LVar());
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }

   
    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(1,new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringUninitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairIntegerString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s2,new LVar(s1));
    	LPair p2 = new LPair(1,s1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairIntegerInitializedLvar() {

    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(1,new LVar(2));

    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,new LVar(s2));
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(1));
    	LPair p2 = new LPair(s,1);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairStringInitializedLvar() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
     	LPair p1 = new LPair(s1,new LVar(2));
    	LPair p2 = new LPair(s1,new LVar(2));
    	LPair p3 = new LPair(s2,new LVar(2));
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairInitializedLvarString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,new LVar(s2));
    	LPair p2 = new LPair(new LVar(1),s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairUninitializedLvarString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(s1,new LVar(s2));
    	LPair p2 = new LPair(new LVar(),s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(STRING,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairStringInitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(s,new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
 

 
 	//%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(1,2);
    	
    	assertTrue(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairIntegerString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,INTEGER) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED,INTEGER) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedIntegerWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INNITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarIntegerWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),2);
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairIntegerString() {
    	
    	String s1 = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s1);
    	LPair p2 = new LPair(1,s1);
    	
    	assertTrue(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairIntegerInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(1,new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(1),s2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INTEGER,STRING) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(s,new LVar(s));
    	LPair p3 = new LPair(new LVar(s), s);
    	
    	assertFalse(p1.equals(p2) || p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(1),s);
    	LPair p3 = new LPair(new LVar(s), s);
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED lVAR,STRING) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedStringWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(1),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarStringWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),s);
    	LPair p2 = new LPair(new LVar(),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITILIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(s1),new LVar());
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairinitialziedLvarUninitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar());
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALZIED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairInitialziedLvarUninitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar());
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar());
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarUninitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(1,2);
    	
    	assertTrue(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar(s));
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	LPair p3 = new LPair(new LVar(1),new LVar(3));
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INITALZIED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALZIED LVAR,INITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(s1),new LVar(s2));
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar(1));
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar(2));
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar(2));
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED lVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),2);
    	LPair p3 = new LPair(new LVar(1), 3);
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar(s));
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	LPair p3 = new LPair(new LVar(1), new LVar(3));
    	
    	assertTrue(p1.equals(p2) && !p1.equals(p3));
    }
    
    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(1),new LVar(s));
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(INITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairInitializedLvarInitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(1),new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }


 	//%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairIntegerString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED,INTEGER) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedIntegerWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INTEGER) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarIntegerWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),2);
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairIntegerInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
   
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairIntegerString() {
    	
    	String s1 = new String("hello");
    	LPair p1 = new LPair(new LVar(),s1);
    	LPair p2 = new LPair(1,s1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairIntegerInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(1,new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairIntegerUninitializedLavr() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(),s2);
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINTEGER,STRING) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(s,new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairInitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairInitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED lVAR,STRING) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedStringWithPairInitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(1),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairUninitializedLvarInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairUninitializedLvarUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,STRING) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarStringWithPairUninitializedLvarInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),s);
    	LPair p2 = new LPair(new LVar(),new LVar(s));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(1,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITILIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairUninitialziedLvarUninitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(s),new LVar());
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALZIED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairUninitialziedLvarUninitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(1),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarUninitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar());
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairIntegerInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(1,2);
    	
    	assertFalse(p1.equals(p2));
    }
    
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairIntegerString() {
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(s));
    	LPair p2 = new LPair(1,s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairIntegerInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITALZIED LVAR,INITIALIZED LVAR) AND PAIR(INTEGER,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairIntegerUninitializedLavr() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(1,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALZIED LVAR,INITIALIZED LVAR) AND PAIR(STRING,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairStringString() {
    	
    	String s1 = new String("hello");
    	String s2 = new String("world");
    	LPair p1 = new LPair(new LVar(),new LVar(s2));
    	LPair p2 = new LPair(s1,s2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNIINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairStringInteger() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(1));
    	LPair p2 = new LPair(s,1);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,UNINITILAIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairStringUninitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(s,new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(STRING,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairStringInitializedLvar() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(s,new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED lVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairInitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),2);
    	    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairInitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(s));
    	LPair p2 = new LPair(new LVar(1),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairInitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(INITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairInitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(1),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }
    
    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INTEGER)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairUninitializedLvarInteger() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(),2);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,STRING)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairUninitializedLvarString() {
    	
    	String s = new String("hello");
    	LPair p1 = new LPair(new LVar(),new LVar(s));
    	LPair p2 = new LPair(new LVar(),s);
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,UNINITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairUninitializedLvarUninitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar());
    	
    	assertFalse(p1.equals(p2));
    }
    

    //%%%%%% EQUALS PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR) AND PAIR(UNINITIALIZED LVAR,INITIALIZED LVAR)
    
    @Test
    public void testEqualsPairUninitializedLvarInitializedLvarWithPairUninitializedLvarInitializedLvar() {
    	
    	LPair p1 = new LPair(new LVar(),new LVar(2));
    	LPair p2 = new LPair(new LVar(),new LVar(2));
    	
    	assertFalse(p1.equals(p2));
    }

 
 
}
