package jsetl.test;

import jsetl.*;
import jsetl.exception.Failure;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.Solver;
import junit.framework.TestCase;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class LRelTest extends TestCase{

    @Before
    public void setUp() {
        LObject.USEJOLLYTOKEN = true;
    }

    @After
    public void tearDown(){LObject.USEJOLLYTOKEN = false;}
	
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						DOMAIN
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     *       r in V
     * --------------------
     * dom(r,r) -- > r = {}
     * 
     * */
    
    @Test
    public void testDomainSameVariableOpen() throws Failure{

    	Solver solver = new Solver();
        LRel rel  = new LRel();

        solver.add(rel.dom(rel));
        solver.solve();

        assertTrue(rel.isEmpty());
    }
    
    
    /**
     *       empty(r)
     * --------------------      
     * dom(r,a) ---> a = {}
     * 
     * */
    @Test
    public void testDomainEmptyRelOpen() throws Failure{
        Solver solver = new Solver();
        LRel rel  = new LRel();
        LSet dom = new LSet();

        rel = LRel.empty();

        solver.add(rel.dom(dom));
        
        solver.solve();
        
        assertTrue(rel.equals(LRel.empty()) && dom.equals(LSet.empty()));
    }
    
    
    /**
     *       empty(a)
     * --------------------      
     * dom(r,a) ---> r = {}
     * 
     * */
    @Test
    public void testDomainEmptyDomOpen() throws Failure{
        Solver solver = new Solver();
        LRel rel  = new LRel();
        LSet dom = LSet.empty();

        solver.add(rel.dom(dom));
        
        solver.solve();
        
        assertTrue(rel.equals(LRel.empty()) && dom.equals(LSet.empty()));
    }

    

    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * dom = {0,1,2,3,4|_?}
     * 
     * dom(rel,domControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyDomOpen(){
        Solver solver = new Solver();

        LRel rel  = new LRel();
        LSet domControl = new LSet();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(rel.dom(domControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * dom = {0,1,2,3,4}
     * 
     * dom(rel,domControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyDomClosed(){
        Solver solver = new Solver();

        LRel rel  = LRel.empty();
        LSet domControl = LSet.empty();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(rel.dom(domControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * dom in V
     * 
     * dom(rel,dom)  ---->  dom = {0,1,2,3,4|_?}
     *  
     * */
    @Test
    public void testNotEmptyDomUndefinedOpen() throws Failure{

        Solver solver = new Solver();

        LSet dom = new LSet();
        LRel rel  = new LRel();
        LSet domControl = new LSet();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(rel.dom(dom));
        solver.solve();
        // dom.output();
        solver.add(dom.eq(domControl));
        solver.solve();
        assertTrue(true);
    }
    
    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * dom in V
     * 
     * dom(rel,dom)  ---->  dom = {0,1,2,3,4}
     *  
     * */
    @Test
    public void testNotEmptyDomUndefinedClosed() throws Failure{

        Solver solver = new Solver();

        LSet dom = new LSet();
        LRel rel  = new LRel();
        LSet domControl = new LSet();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            domControl = domControl.ins(i);
        }

        solver.add(rel.dom(dom));
        solver.solve();
        // dom.output();
        solver.add(dom.eq(domControl));
        solver.solve();
        assertTrue(true);
    }
    
      
    /**
     *  rel = {?}
     *  dom = {4,3,2,1,0|_?}
     * 
     *  dom(rel,dom)  ------>  rel = {(4,_?),(3,_?),(2,_?),(1,_?),(0,_?)|_?}
     *  
     * */

    @Test
    public void testNotEmptyRelUndefinedOpen() throws Failure{
        
    	Solver solver = new Solver();
        
    	LRel rel  = new LRel();
        LRel relControl = new LRel(); 
        LSet dom = new LSet();

        for(int i = 0 ; i<5; ++i){
            dom = dom.ins(i);
            relControl = relControl.ins(new LPair(i,new LVar()));
        }

        solver.add(rel.dom(dom));
        solver.solve();
        //rel.output();
        //relControl.output();
        solver.add(rel.eq(relControl));
        solver.solve();

    }
    
    
    
    /**
     *  rel = {?}
     *  dom = {4,3,2,1,0}
     * 
     *  dom(rel,dom)  ------>  rel = {(4,_?),(3,_?),(2,_?),(1,_?),(0,_?)}
     *  
     * */

    @Test
    public void testNotEmptyRelUndefinedClose() throws Failure{
        
    	Solver solver = new Solver();
        
    	LRel rel  = new LRel();
        LRel relControl = LRel.empty(); 
        LSet dom = LSet.empty();

        for(int i = 0 ; i<5; ++i){
            dom = dom.ins(i);
            relControl = relControl.ins(new LPair(i,new LVar()));
        }

        solver.add(rel.dom(dom));
        solver.solve();
        //rel.output();
        //relControl.output();
        solver.add(rel.eq(relControl));
        solver.solve();

    } 
    
    
    /**
     *  rel = {?}
     *  dom = {4}
     * 
     *  dom(rel,dom)  ------>  rel = {(4,_?)}
     *  
     * */

    @Test
    public void testNotEmptyRelUndefinedClose_Dom_one_elem() throws Failure{
        
    	Solver solver = new Solver();
        
    	LRel rel  = new LRel();
        LRel relControl = LRel.empty().ins(new LPair(4,new LVar())); 
        LSet dom = LSet.empty().ins(4);

        

        solver.add(rel.dom(dom));
        solver.solve();
        //rel.output();
        //relControl.output();
        solver.add(rel.eq(relControl));
        solver.solve();

    } 
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						RANGE
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     *       r in V
     * --------------------
     * ran(r,r) -- > r = {}
     * 
     * */
    
    @Test
    public void testRangeSameVariableOpen() throws Failure{
        
    	Solver solver = new Solver();
        LRel rel  = new LRel();

        solver.add(rel.ran(rel));
        solver.solve();

        assertTrue(rel.isEmpty());
    }
    
    
    /**
     *       empty(r)
     * --------------------      
     * ran(r,a) ---> a = {}
     * 
     * */
    @Test
    public void testRangeEmptyRelOpen() throws Failure{
        
    	Solver solver = new Solver();
        LRel rel  = new LRel();
        LSet ran = new LSet();

        rel = LRel.empty();

        solver.add(rel.ran(ran));
        
        solver.solve();
        
        assertTrue(rel.equals(LRel.empty()) && ran.equals(LSet.empty()));
    }
    
    
    /**
     *       empty(a)
     * --------------------      
     * ran(r,a) ---> r = {}
     * 
     * */
    @Test
    public void testRangeEmptyRanOpen() throws Failure{
        
    	Solver solver = new Solver();
        LRel rel  = new LRel();
        LSet ran = LSet.empty();
        
        solver.add(rel.ran(ran));
        
        solver.solve();
        
        assertTrue(rel.equals(LRel.empty()) && ran.equals(LSet.empty()));
    }

    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * ran = {0,1,2,3,4|_?}
     * 
     * ran(rel,ranControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyRanOpen(){
        Solver solver = new Solver();

        LRel rel  = new LRel();
        LSet ranControl = new LSet();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(i);
        }

        solver.add(rel.dom(ranControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }

    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * ran = {0,1,2,3,4}
     * 
     * ran(rel,ranControl)  ----> SOLVED
     * */
    
    @Test
    public void testNotEmptyRanClosed(){
        Solver solver = new Solver();

        LRel rel  = LRel.empty();
        LSet ranControl = LSet.empty();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(i);
        }

        solver.add(rel.dom(ranControl));

        try {
            solver.solve();
        } catch (Failure e) {
            assertTrue(false);
        }

        assertTrue(true);
    }
    
    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)|_?}
     * 
     * ran in V
     * 
     * ran(rel,ran) ----> ran = {1,2,3,4,5|_?}
     * 
     * */
    
    @Test
    public void testNotEmptyRanUndefinedOpen() throws Failure{
        
    	LSet ran = new LSet();
        LRel rel  = new LRel();
        LSet ranControl = new LSet();
        Solver solver = new Solver();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(new IntLVar(i+1));
        }


        solver.add(rel.ran(ran));
        solver.solve();
        solver.add(ran.eq(ranControl));
        solver.solve();

        assertTrue(true);
    }
    
    
    /**
     * rel = {(0,1),(1,2),(2,3),(3,4),(4,5)}
     * 
     * ran in V
     * 
     * ran(rel,ran) ----> ran = {1,2,3,4,5}
     * 
     * */
    
    @Test
    public void testNotEmptyRanUndefinedClosed() throws Failure{
        
    	LSet ran = new LSet();
        LRel rel  = new LRel();
        LSet ranControl = new LSet();
        Solver solver = new Solver();

        for(int i = 0 ; i<5; ++i){
            rel = rel.ins(new LPair(i,i+1));
            ranControl = ranControl.ins(new IntLVar(i+1));
        }


        solver.add(rel.ran(ran));
        solver.solve();
        solver.add(ran.eq(ranControl));
        solver.solve();

        assertTrue(true);
    }
    
    
    /**
     * rel in V
     * 
     * a = {1}
     * 
     * ran(rel,a) ----> rel = {[_?,1]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_closed_only_one_elem(){
    	
    	Solver solver = new Solver();
		
		LRel rel = new LRel();//.setName("rel");
		LSet a = LSet.empty().ins(1).setName("a");
				
		LRel control = new LRel().ins(new LPair(new LVar(),1));
		
		
		try {
			solver.add(rel.ran(a));
			solver.solve();
			solver.nextSolution();
			
			assertTrue(control.toString().equals(rel.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * rel in V
     * 
     * a = {1,2,3}
     * 
     * ran(rel,a) ----> rel = {[_?,1],[_?,2],[_?,3]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_closed_more_elems(){
    	
    	Solver solver = new Solver();
		
		LRel rel = new LRel();//.setName("rel");
		LSet a = LSet.empty().ins(1).ins(2).ins(3).setName("a");
				
		LRel control = new LRel().ins(new LPair(new LVar(),1)).ins(new LPair(new LVar(),2)).ins(new LPair(new LVar(),3));
		//System.out.println(control.toString());
		
		try {
			solver.add(rel.ran(a));
			solver.solve();
			//System.out.println(rel.toString());
			assertTrue(control.toString().equals(rel.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * rel in V
     * 
     * a = {1| _?}
     * 
     * ran(rel,a) ----> rel = {[_?,1]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_open_only_one_elem(){
    	
    	Solver solver = new Solver();
		
		LRel rel = new LRel();//.setName("rel");
		LSet a = new LSet().ins(1).setName("a");
				
		LRel control = new LRel().ins(new LPair(new LVar(),1));
		
		
		try {
			solver.add(rel.ran(a));
			solver.solve();
			
			assertTrue(control.toString().equals(rel.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    
    /**
     * rel in V
     * 
     * a = {1,2,3}
     * 
     * ran(rel,a) ----> rel = {[_?,1],[_?,2],[_?,3]| _?]}
     * 
     * */
    
    @Test
    public void testRan_A_open_more_elems(){
    	
    	Solver solver = new Solver();
		
		LRel rel = new LRel();//.setName("rel");
		LSet a = new LSet().ins(1).ins(2).ins(3).setName("a");
				
		LRel control = new LRel().ins(new LPair(new LVar(),1)).ins(new LPair(new LVar(),2)).ins(new LPair(new LVar(),3));
		//System.out.println(control.toString());
		
		try {
			solver.add(rel.ran(a));
			solver.solve();
			//System.out.println(rel.toString());
			assertTrue(control.toString().equals(rel.toString()));
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						IS GROUND
     * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     */
    
    
    /**
     * rel = {(1,null),(2,null)}
     * 
     * isGround(rel) => true
     * */
    @Test
    public void testGroundRelAllLVarClose() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        LRel rel = LRel.empty().ins(new LPair(new LVar(1), new LVar())).ins(new LPair(new LVar(2), new LVar()));
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    
    /**
     * rel = {(1,null),(2,null)}
     * 
     * isGround(rel) => true
     * */
    @Test
    public void testGroundRelNotLVarClose()throws NoSuchMethodException, InvocationTargetException, IllegalAccessException{
        LRel rel = LRel.empty().ins(new LPair(1, new LVar())).ins(new LPair(new LVar(2), new LVar()));
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    /**
     * rel = {(1,null),(null,null)}
     * 
     * isGround(rel) => false
     * */
    @Test
    public void testNotGroundRelClose()throws NoSuchMethodException, InvocationTargetException, IllegalAccessException{
        LRel rel = LRel.empty().ins(new LPair(1, new LVar())).ins(new LPair());
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione non ground", result);
    }
    
    
    /**
     * rel = {(1,2),(2,3)|_?}
     * 
     * isGround(rel) => true
     * */
    @Test
    public void testGroundRelAllLVarOpen()throws NoSuchMethodException, InvocationTargetException, IllegalAccessException{
        LRel rel = new LRel();
        rel = LRel.empty().ins(new LPair(new LVar(1),2)).ins(new LPair(new LVar(2),3));
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione ground", result);
    }
    
    /**
     * rel = {(1,null),(2,null)|_?}
     * 
     * isGround(rel) => true
     * */
    @Test
    public void testGroundRelNotLVarOpen()throws NoSuchMethodException, InvocationTargetException, IllegalAccessException{
        LRel rel = new LRel();
        rel = rel.ins(new LPair(1, new LVar())).ins(new LPair(new LVar(2), new LVar()));
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        //rel.output();

        assertEquals("Funzione non ground", result);
    }
    
    /**
     * rel = {(1,2),(null,null)|_?}
     * 
     * isGround(rel) => false
     * */
    @Test
    public void testNotGroundRelOpen() throws NoSuchMethodException, InvocationTargetException, IllegalAccessException {
        LRel rel = new LRel();
        rel = rel.ins(new LPair(1,2)).ins(new LPair());
        
        
        
        String result;

        Method isDomainGround = LRel.class.getDeclaredMethod("isDomainGround", LSet.class);
        isDomainGround.setAccessible(true);
        if((Boolean)isDomainGround.invoke(LRel.class, rel)){
            result = "Funzione ground";
        }else{
            result = "Funzione non ground";
        }
        
        assertEquals("Funzione non ground", result);
    }
    
    

    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 						COMP
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     * R = {}
     * S in V
     * Q in V
     * 
     * comp(R,S,q) ===>>> q = {}
     */
    
    public void testCompR_Empty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r = LRel.empty();
    	LRel s = new LRel();
    	LRel q = new LRel();
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	assertTrue(r.equals(q));
    }
    
    
    /**
     * R in V
     * S = {}
     * Q in V
     * 
     * comp(R,S,q) ===>>> q = {}
     */
    
    public void testCompS_Empty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r = new LRel();
    	LRel s = LRel.empty();
    	LRel q = new LRel();
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	assertTrue(s.equals(q));
    }
    
    /**
     * R = {(1,2),(3,4)}
     * S = {(5,6),(7,8)}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> SOLVED
     */
    
    public void testCompQ_EmptyClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = LRel.empty().ins(new LPair(5,6)).ins(new LPair(7,8));
    	LRel q = LRel.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertTrue(false);
    	}
    	
    	assertTrue(true);
    }
    
    
    /**
     * R = {(1,2),(3,4)|_?}
     * S = {(5,6),(7,8)|_?}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> SOLVED
     */
    
    public void testCompQ_EmptyOpen() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r = new LRel().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = new LRel().ins(new LPair(5,6)).ins(new LPair(7,8));
    	LRel q = LRel.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertTrue(false);
    	}
    	
    	assertTrue(true);
    }
    
    
    /**
     * R = {(1,2),(3,4)}
     * S = {(2,6),(4,8)}
     * Q = {}
     * 
     * comp(R,S,q) ===>>> FAILURE
     */
    
    public void testCompQ_EmptyClosedFailure() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = LRel.empty();
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	}catch(Failure e) {
    		assertFalse(false);
    	}
    	
    	
    }
    
    /**
     * R in V
     * S in V
     * Q = {(1,6),(3,8)}
     * 
     * comp(R,S,Q) ===>>>  R={[3,_?],[1,_?]|_?}   S={[_?,8],[_?,6]|_?}
     */
    
    public void testCompQ_NotEmptyClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r_Control= new LRel().ins(new LPair(1,new LVar())).ins(new LPair(3,new LVar()));
    	LRel s_Control= new LRel().ins(new LPair(new LVar(),6)).ins(new LPair(new LVar(),8));
    	
    	LRel r = new LRel();
    	LRel s = new LRel();
    	LRel q = LRel.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(r.eq(r_Control));
    		solver.add(s.eq(s_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    
    }
    
    /**
     * R in V
     * S in V
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>>  R={[3,_?],[1,_?]|_?}   S={[_?,8],[_?,6]|_?}
     */
 
    public void testCompQ_NotEmptyOpen() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r_Control= new LRel().ins(new LPair(1,new LVar())).ins(new LPair(3,new LVar()));
    	LRel s_Control= new LRel().ins(new LPair(new LVar(),6)).ins(new LPair(new LVar(),8));
    	
    	LRel r = new LRel();
    	LRel s = new LRel();
    	LRel q = new LRel().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(r.eq(r_Control));
    		solver.add(s.eq(s_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
 
    
    /**
     * R in {(1,2),(3,4)|_?}
     * S in V
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> S={[4,8],[1,6]|_?}
     */
    
    public void testCompQ_NotEmpty_R_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel s_Control= new LRel().ins(new LPair(2,6)).ins(new LPair(4,8));
    	
    	LRel r = new LRel().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = new LRel();
    	LRel q = new LRel().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		//solver.showStore();
    		//assertTrue(s.equals(s_Control));
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in V
     * S in {(2,6),(4,8)|_?}
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> R={[1,2],[3,4]|_?}
     */
    
    public void testCompQ_NotEmpty_S_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel r_Control= new LRel().ins(new LPair(1,2)).ins(new LPair(3,4));
    	
    	LRel r = new LRel();
    	LRel s = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = new LRel().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	try {
    		solver.add(r.comp(s, q));
    		solver.solve();
    	    		
    		//assertTrue(r.equals(r_Control));
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    /**
     * R in {(1,2),(3,4)|_?}
     * S in {(2,6),(4,8)|_?}
     * Q = {(1,6),(3,8)|_?}
     * 
     * comp(R,S,Q) ===>>> SOLVED
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmpty() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LRel r = new LRel().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = new LRel().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	try {
			
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = {(1,6),(3,8)}
     * 
     * comp(R,S,Q) ===>>> SOLVED
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmptyandClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = LRel.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	
    	try {
			
    		solver.add(r.comp(s, q));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = {(9,6),(7,8)}
     * 
     * comp(R,S,Q) ===>>> FAILURE
     */
    
    public void testCompQ_NotEmpty_RandS_NotEmptyandClosed_Fail() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = LRel.empty().ins(new LPair(9,6)).ins(new LPair(7,8));

    		solver.add(r.comp(s, q));
    		assertFalse(solver.check());

    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)|_?}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={(1,6),(3,8)|_?}
     */
    
    public void testCompQ_NotInit() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel q_Control= new LRel().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = new LRel();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2),(3,4)}
     * S in {(2,6),(4,8)}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={(1,6),(3,8)}
     */
    
    public void testCompQ_NotInitClosed() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel q_Control= LRel.empty().ins(new LPair(1,6)).ins(new LPair(3,8));
    	
    	LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4));
    	LRel s = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8));
    	LRel q = new LRel();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**
     * R in {(1,2)}
     * S in {(2,6)}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={(1,6)}
     */
    
    public void testCompQ_NotInitClosed_R_S_one_elem() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel q_Control= LRel.empty().ins(new LPair(1,6));
    	
    	LRel r = LRel.empty().ins(new LPair(1,2));
    	LRel s = LRel.empty().ins(new LPair(2,6));
    	LRel q = new LRel();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    /**
     * R in {(1,2)}
     * S in {(3,6)}
     * Q = V
     * 
     * comp(R,S,Q) ===>>> Q={}
     */
    
    public void testCompQ_NotInitClosed_R_S_one_elem2() throws Failure{
    	
    	Solver solver = new Solver();
    	
    	LRel q_Control= LRel.empty();
    	
    	LRel r = LRel.empty().ins(new LPair(1,2));
    	LRel s = LRel.empty().ins(new LPair(3,6));
    	LRel q = new LRel();
    	
    	
    	solver.add(r.comp(s, q));
    	solver.solve();
    	
    	//q.output();
    	
    	try {
			
    		solver.add(q.eq(q_Control));
    		solver.solve();
    		
    		//q.output();
    		
    		assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    	
    	
    }
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 				UNION
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    /**
     * M1 = {(1,2),(3,4)}
     * M2 = {(2,6),(4,8)|_?}
     * M3 in V
     * 
     * un(M1,M2,M3) ==>> M3={(1,2),(3,4),(2,6),(4,8)|_?}
     */
    
    @Test
    public void testLRel_un() {
    	
    	Solver solver = new Solver();
    	
    	LRel m1 = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4)).setName("m1");
		LRel m2 = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8)).setName("m2");
		LRel m3 = new LRel();
		
		LRel control = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m3.union(m1, m2));
			solver.add(m3.eq(control));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * M1 = {(1,2),(3,4)}
     * M2 in V
     * M3 = {(1,2),(3,4),(2,6),(4,8)|_?}
     * 
     * un(M1,M2,M3) ==>> M2 = {(2,6),(4,8)|_?}
     */
    
    @Test
    public void testLRel_un_M2notInit() {
    	
    	Solver solver = new Solver();
    	
    	LRel m1 = LRel.empty().ins(new LPair(1,2)).ins(new LPair(3,4)).setName("m1");
		LRel control = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8)).setName("m2");
		LRel m2 = new LRel();
		
		LRel m3 = new LRel().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m3.union(m1, m2));
			solver.add(m2.eq(control));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			INTERS
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    /**
     * M1 = {(4,8)}
     * M2 = {(1,6),(4,8)}
     * M3 = {(1,2),(3,4),(2,6),(4,8)}
     * 
     * inters(M1,M2,M3) ===>> SOLVED
     */
    
    @Test
    public void testLRel_inters() {
    	
    	Solver solver = new Solver();
    	
    	LRel m1 = LRel.empty().ins(new LPair(4,8)).setName("m1");
		LRel m2 = LRel.empty().ins(new LPair(1,6)).ins(new LPair(4,8)).setName("m2");
		LRel m3 = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m2.inters(m3,m1));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * M1 = {(4,8)}
     * M2 in V
     * M3 = {(1,2),(3,4),(2,6),(4,8)}
     * 
     * inters(M1,M2,M3) ===>> M2 = {(4,8)|_?}
     */
    
    @Test
    public void testLRel_inters_M2notInit() {
    	
    	Solver solver = new Solver();
    	
    	LRel m1 = LRel.empty().ins(new LPair(4,8)).setName("m1");
		LRel control = new LRel().ins(new LPair(4,8));
		LRel m3 = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		LRel m2 = new LRel();
		
		try {
			solver.add(m1.inters(m3, m2));
			solver.add(control.eq(m2));
			solver.solve();
			
			assertTrue(true);
		} catch (Failure e) {
			assertTrue(false);
		}
    }
    
    /**
     * M1 = {}
     * M2 = {(1,6),(4,8)}
     * M3 = {(1,2),(3,4),(2,6),(4,8)}
     * 
     * inters(M1,M2,M3) ===>> UNSOLVED
     */
    
    @Test
    public void testLRel_inters_false() {
    	
    	Solver solver = new Solver();
    	
    	LRel m1 = LRel.empty().setName("m1");
		LRel m2 = LRel.empty().ins(new LPair(1,6)).ins(new LPair(4,8)).setName("m2");
		LRel m3 = LRel.empty().ins(new LPair(2,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
		
		try {
			solver.add(m1.inters(m3, m2));
			solver.solve();
			
			assertFalse(true);
		} catch (Failure e) {
			assertFalse(false);
		}
    }
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			SUBSET
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    /**
     *
     * M2 = {(1,6),(4,8)}
     * M3 = {(1,6),(3,4),(2,6),(4,8)}
     * 
     * subset(M2,M3) ===>> SOLVED
     */
    
    @Test
    public void testLRel_subset_true() {
    	
    	Solver solver = new Solver();
    	
    	LRel m2 = LRel.empty().ins(new LPair(1,6)).ins(new LPair(4,8)).setName("m2");
		LRel m3 = LRel.empty().ins(new LPair(1,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
    
		try {
			solver.add(m2.subset(m3));
			solver.solve();
			
			assertTrue(true);
		} catch(Failure e) {
			assertTrue(false);
		}
    }
    
    
    /**
     * 
     * M2 = {(3,6),(4,8)}
     * M3 = {(1,2),(3,4),(2,6),(4,8)}
     * 
     * subset(M2,M3) ===>> UNSOLVED
     */
    
    @Test
    public void testLRel_subset_false() {
    	
    	Solver solver = new Solver();
    	
    	LRel m2 = LRel.empty().ins(new LPair(3,6)).ins(new LPair(4,8)).setName("m2");
		LRel m3 = LRel.empty().ins(new LPair(1,6)).ins(new LPair(4,8)).ins(new LPair(1,2)).ins(new LPair(3,4));
    
		try {
			solver.add(m2.subset(m3));
			solver.solve();
			
			assertFalse(true);
		} catch(Failure e) {
			assertFalse(false);
		}
    }
    
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			  ID
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    
    /**
     * 
     * R in V
     * 
     * id(R,R) ===>> R = {}
     */
    
    @Test
    public void testLRel_id_R_inVar() {
    	
    	Solver solver = new Solver();
		
		LRel r = new LRel();
		LRel control = LRel.empty();
		
		try {
			solver.add(r.id(r));
			solver.solve();
			
			assertTrue(r.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R in V
     * S = {}
     * 
     * id(R,S) ===>> R = {}
     */
    
    @Test
    public void testLRel_id_S_Empty() {
    	
    	Solver solver = new Solver();
		
    	LSet s = LSet.empty();
		LRel r = new LRel();
		LRel control = LRel.empty();
		
		try {
			solver.add(r.id(s));
			solver.solve();
			
			assertTrue(r.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R = {}
     * S in V
     * 
     * id(R,S) ===>> S = {}
     */
    
    @Test
    public void testLRel_id_R_Empty() {
    	
    	Solver solver = new Solver();
		
    	LSet s = new LSet();
		LRel r = LRel.empty();
		LSet control = LSet.empty();
		
		try {
			solver.add(r.id(s));
			solver.solve();
			
			assertTrue(s.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R in V
     * S = {1,2,3}
     * 
     * id(R,S) ===>> R = {[1,1],[2,2],[3,3]}
     */
    
    @Test
    public void testLRel_id_S_closed_R_in_V() {
    	
    	Solver solver = new Solver();
		
    	LSet s = LSet.empty().ins(1).ins(2).ins(3);
		LRel r = new LRel();
		LRel control = LRel.empty().ins(new LPair(1,1)).ins(new LPair(2,2)).ins(new LPair(3,3));
		
		try {
			solver.add(r.id(s));
			solver.solve();
			
			assertTrue(r.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    
    /**
     * 
     * R in V
     * S = {1,2,3| _?}
     * 
     * id(R,S) ===>> R = {[1,1],[2,2],[3,3]| _?}
     */
    
    @Test
    public void testLRel_id_S_open_R_in_V() {
    	
    	Solver solver = new Solver();
		
    	LSet s = LSet.empty().ins(1).ins(2).ins(3);
		LRel r = new LRel();
		LRel control = new LRel().ins(new LPair(1,1)).ins(new LPair(2,2)).ins(new LPair(3,3));
		
		try {
			solver.add(r.id(s));
			solver.add(r.eq(control));
			solver.solve();
			
			assertTrue(true);
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    
    /**
     * 
     * R = {[1,1],[2,2],[3,3]}
     * S in V
     * 
     * id(R,S) ===>> S = {1,2,3}
     */
    
    @Test
    public void testLRel_id_R_closed_S_in_V() {
    	
    	Solver solver = new Solver();
		
    	LSet control = LSet.empty().ins(1).ins(2).ins(3);
		LSet s = new LSet();
		LRel r = LRel.empty().ins(new LPair(1,1)).ins(new LPair(2,2)).ins(new LPair(3,3));
		
		try {
			solver.add(r.id(s));
			solver.solve();
			
			assertTrue(s.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R = {[1,1],[2,2],[3,3]| _?}
     * S in V
     * 
     * id(R,S) ===>> S = {1,2,3| _?}
     */
    
    @Test
    public void testLRel_id_R_open_S_in_V() {
    	
    	Solver solver = new Solver();
		
    	LSet control = new LSet().ins(1).ins(2).ins(3);
		LSet s = new LSet();
		LRel r = new LRel().ins(new LPair(1,1)).ins(new LPair(2,2)).ins(new LPair(3,3));
		
		try {
			solver.add(r.id(s));
			solver.add(s.eq(control));
			solver.solve();
			
			assertTrue(true);
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R = {[1,1],[2,2],[3,3]}
     * S = {1,3}
     * 
     * id(R,S) ===>> UNSOLVED
     */
    
    @Test
    public void testLRel_id_closed_false() {
    	
    	Solver solver = new Solver();
		
    	LSet s= LSet.empty().ins(1).ins(3);
		LRel r = LRel.empty().ins(new LPair(1,1)).ins(new LPair(2,2)).ins(new LPair(3,3));
		
		try {
			solver.add(r.id(s));
			
			solver.solve();
			
			assertTrue(false);
			
		} catch(Failure e) {
			
			assertTrue(true);
		}
		
    } 
    
    
    /**%%%%%%%%%%%%%%%%%%%%%%%%%%%
     * 			  INV
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
    
    
    /**
     * 
     * R in V
     * S = {}
     * 
     * inv(R,S) ===>> R = {}
     */
    
    @Test
    public void testLRel_inv_S_Empty() {
    	
    	Solver solver = new Solver();
		
    	LRel s = LRel.empty();
		LRel r = new LRel();
		LRel control = LRel.empty();
		
		try {
			
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(r.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    }
    
    /**
     * 
     * R = {}
     * S in V
     * 
     * inv(R,S) ===>> S = {}
     */
    
    @Test
    public void testLRel_inv_R_Empty() {
    	
    	Solver solver = new Solver();
		
    	LRel s = new LRel();
		LRel r = LRel.empty();
		LRel control = LRel.empty();
		
		try {
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(s.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    
    /**
     * 
     * R = {[1,2],[2,3],[3,4]}
     * S in V
     * 
     * id(R,S) ===>> S = {[2,1],[3,2],[4,3]}
     */
    
    @Test
    public void testLRel_inv_R_closed_S_in_V() {
    	
    	Solver solver = new Solver();
		
    	LRel control = LRel.empty().ins(new LPair(4,3)).ins(new LPair(3,2)).ins(new LPair(2,1));
		LRel s = new LRel();
		LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		
		try {
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(s.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    /**
     * 
     * R in V
     * S = {[1,2],[2,3],[3,4]}
     * 
     * id(R,S) ===>> S = {[2,1],[3,2],[4,3]}
     */
    
    @Test
    public void testLRel_inv_S_closed_R_in_V() {
    	
    	Solver solver = new Solver();
		
    	LRel control = LRel.empty().ins(new LPair(4,3)).ins(new LPair(3,2)).ins(new LPair(2,1));
		LRel r = new LRel();
		LRel s = LRel.empty().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		
		try {
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(r.equals(control));
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    }
    
    /**
     * 
     * R = {[1,2],[2,3],[3,4]}
     * S = {[2,1],[3,2],[4,3]}
     * 
     * id(R,S) ===>> SOLVED
     */
    
    @Test
    public void testLRel_inv_R_S_closed_solved() {
    	
    	Solver solver = new Solver();
		
    	LRel s = LRel.empty().ins(new LPair(4,3)).ins(new LPair(3,2)).ins(new LPair(2,1));
		//LRel s = new LRel();
		LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		
		try {
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(true);
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    } 
    
    /**
     * 
     * R = {[1,2],[2,3],[3,4]}
     * S = {[2,1],[3,2],[3,4]}
     * 
     * id(R,S) ===>> UNSOLVED
     */
    
    @Test
    public void testLRel_inv_R_S_closed_unsolved() {
    	
    	Solver solver = new Solver();
		
    	LRel s = LRel.empty().ins(new LPair(3,4)).ins(new LPair(3,2)).ins(new LPair(2,1));
		//LRel s = new LRel();
		LRel r = LRel.empty().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		
		try {
			solver.add(r.inv(s));
			solver.solve();
			
			assertTrue(false);
			
		} catch(Failure e) {
			
			assertTrue(true);
		}
		
    }
    
    /**
     * 
     * R = {[1,2],[2,3],[3,4]| _?}
     * S in V
     * 
     * id(R,S) ===>> S = {[2,1],[3,2],[4,3]| _?}
     */
    
    @Test
    public void testLRel_inv_R_open_S_in_V() {
    	
    	Solver solver = new Solver();
		
    	LRel control = new LRel().ins(new LPair(4,3)).ins(new LPair(3,2)).ins(new LPair(2,1));
		LRel s = new LRel();
		LRel r = new LRel().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		//System.out.println(control.toString());
		try {
			solver.add(r.inv(s));
			//solver.add(r.eq(control));
			solver.solve();
			//System.out.println(s.toString());
			//assertTrue(s.equals(control));
			assertEquals("{(2,1),(3,2),(4,3)/_?}", control.toString());
			assertTrue(true);
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    }
    
    /**
     * 
     * R in V
     * S = {[1,2],[2,3],[3,4]| _?}
     * 
     * id(R,S) ===>> S = {[2,1],[3,2],[4,3]| _?}
     */
    
   @Test
    public void testLRel_inv_S_open_R_in_V() {
    	
    	Solver solver = new Solver();
		
    	LRel control = new LRel().ins(new LPair(4,3)).ins(new LPair(3,2)).ins(new LPair(2,1));
		LRel r = new LRel();
		LRel s = new LRel().ins(new LPair(1,2)).ins(new LPair(2,3)).ins(new LPair(3,4));
		
		try {
			solver.add(r.inv(s));
			solver.add(r.eq(control));
			solver.solve();
			
			assertTrue(true);
			
		} catch(Failure e) {
			
			assertTrue(false);
		}
		
    }

}

