package jsetl.test;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import jsetl.LObject;
import junit.framework.TestCase;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.LSet;
import jsetl.LVar;
import jsetl.exception.NotInitLObjectException;

public class LSetTest extends TestCase {

	@Before
	public void setUp() {
		LObject.USEJOLLYTOKEN = true;
	}

	@After
	public void tearDown(){LObject.USEJOLLYTOKEN = false;}

	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	//LSET CON NOME
	@Test
	public void testGetNameWithName() {
		LSet x = LSet.empty().setName("x"); 
		assertTrue("x".equals(x.getName()));
    }
	
	//LSET SENZA NOME
	@Test
	public void testGetNameNotName() {
		LSet x = new LSet(); 
		assertTrue("?".equals(x.getName()));
	}
	
	@Test
	//LSET VUOTO
	public void testGetNameEmpty() {
		LSet x = LSet.empty(); 
		assertTrue("emptyLSet".equals(x.getName()));
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         SETNAME	
	//%%%%%%%%%%%%%%%%%%%%
	// LSET CON NOME
	@Test
	public void testSetNameWithName() {
		LSet x = LSet.empty().ins(1).setName("x"); 
		assertTrue("x".equals(x.getName()));
    }
	
	//LSET SENZA NOME
	@Test
	public void testSetNameNotName() {
		LSet x = new LSet(); 
		assertTrue("?".equals(x.getName()));
	}
	
	//LSET VUOTO
	@Test
	public void testSetNameEmpty() {
		LSet x = LSet.empty(); 
		assertTrue("emptyLSet".equals(x.getName()));
	}
	
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETVALUE	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testGetValueInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}
		
	//NON INIZIALIZZATO 
	@Test
	public void testGetValueUninitialized() {
		LSet x = new LSet(); 
		assertEquals(null,(x.getValue()));
	}

	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO , INTERO 
	public void testGetValueInitializedWithInsEmpty() {
		LSet x = LSet.empty().ins(1);   
        Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INS,A PARTIRE DA LSET VUOTO , LVar INIZIALIZZATA 
	public void testGetValueInitializedWithInsLVarInitialized() {
		LVar x1 = new LVar("x1",1);   
		LSet x = LSet.empty().ins(x1);
        Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO E LVar NON INIZIALIZZATO 
	public void testGetValueInitializedWithInsLVarUninitialized() {
		LVar x1 = new LVar("x1");   
		LSet x = LSet.empty().ins(x1);
        Set y = new HashSet();
		y.add(x1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test //INIZ COSTRUITO CON INS, A PARTIRE DA VUOTO E LVar INIZ, LVar NON INIZ E INTERO 
	// x1=2, {x2,x1,1}.getValue() --> [x2,2,1]
	public void testGetValueInitializedWithInsInteger_LVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LSet x = LSet.empty().ins(1).ins(x1).ins(x2);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(x2);
		assertEquals(y,(x.getValue()));
	}
	
	@Test //INIZ COSTRUITO CON INS, A PARTIRE DA VUOTO E LVar INIZ, LVar NON INIZ E INTERO (duplicato)
	// x1=2, {x2,x1,2}.getValue() --> [x2,2,1]
	public void testGetValueInitWithInsInt_LVarInit_LVarUninit_Dup() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LSet x = LSet.empty().ins(2).ins(x1).ins(x2);
		Set y = new HashSet();
		y.add(2);y.add(x2);
		assertEquals(y,(x.getValue()));
	}

	@Test //INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E INTERO
	public void testGetValueUninitialized_insInteger() {
		LSet r = new LSet("r");
		LSet x = r.ins(1);
		LSet y = LSet.empty().ins(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INS, LSET NON INIZIALIZZATO E LVar NON INIZIALIZZATO
	public void testGetValueUninitialized_insLVarUninitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1");
		LSet x = r.ins(x1);
		LSet y = LSet.empty().ins(x1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E LVar INIZIALIZZATO 
	public void testGetValueUninitialized_insLVarInitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1);
		LSet y = LSet.empty().ins(x1).ins(1);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZ COSTRUITO CON INS, LSET NON INIZ INTERO, LVar NON INIZ, LVar INIZ 
	public void testGetValueUninitialized_insInteger_LVarUninitialized_LVarInitialized() {
		LSet r = new LSet("r");
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1).ins(x0).ins(2);
		Set y = new HashSet();
		y.add(1);y.add(x0);y.add(2);
		assertEquals(y,(x.getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA VUOTO e ARRAY 
	public void testGetValueInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		LSet x = LSet.empty().insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
	@Test //INIZIALIZZATO COSTRUITO CON INSALL,A PARTIRE DA VUOTO VECTOR 
	public void testGetValueInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = LSet.empty().insAll(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e ARRAY 
	public void testGetValueUnitializedWithInsAllRest_Array() {
		LSet r= new LSet ();
		Integer [] x1={1,2,3};
		LSet x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e VECTOR 
	public void testGetValueUnitializedWithInsAllRest_Vector() {
		LSet r= new LSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = r.insAll(x1);   
        Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);
		assertEquals(y,(x.getValue()));
	}
	
		
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET INIZIALIZZATO VECTOR 
	public void testGetValueInitializedWithInsAll_Vector() {
		LSet r= LSet.empty().ins(1).ins(2);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		LSet x = r.insAll(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);y.add(3);y.add(4);y.add(5);
        assertEquals(y,(x.getValue()));
    }
	
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET  inizializzato e ARRAY 
	public void testGetValueInitializedWithInsAll_Array() {
		LSet r= LSet.empty().ins(1).ins(2);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		LSet x = r.insAll(x1);   
        Set y = new HashSet();
        y.add(1);y.add(2);y.add(3);y.add(4);y.add(5);
		assertEquals(y,(x.getValue()));
	}
		
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET VUOTO 
	public void testGetValueInitializedWithNewLSetEmpty() {
		LSet x1=LSet.empty();
		LSet x= new LSet(x1);
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}

	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET CON ELEMENTI INSERITI TRAMITE INS
	public void testGetValueInitializedWithNewLSetIns() {
		LSet x1 = LSet.empty().ins(1); 
		LSet x= new LSet(x1);
		Set y = new HashSet();
		y.add(1);
		assertEquals(y,(x.getValue()));
	}
	
	//LSET INIZ COSTRUITO CON COSTRUTTORI, LSET NON INIZIALIZZATO
	public void testGetValueInitializedWithNewLSetCostruttoriUninitialized() {
		LSet x1= new LSet();
		LSet x= new LSet(x1);
		LSet y = LSet.empty();
		assertEquals(null,(x.getValue()));
	}
		
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET VUOTO 
	public void testGetValueInitializedWithSetEmpty() {
		Set x1 = new HashSet();
		LSet x= new LSet(x1);
		Set y = new HashSet();
		assertEquals(y,(x.getValue()));
	}
	
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET CON ELEMENTI INSERITI TRAMITE ADD
	public void testGetValueInitializedWithSetAdd() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2); 
		LSet x= new LSet(x1);
		Set y = new HashSet();
		y.add(1);y.add(2);
		assertEquals(y,(x.getValue()));
	}
    /////TOSTRING
	@Test //VUOTO
	public void testToStringInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertEquals("{}",(x.toString()));
	}
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testToStringUninitialized() {
		LSet x = new LSet(); 
		assertEquals("_?",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO , INTERO  
	public void testToStringInitializedWithInsEmpty() {
		LSet x = LSet.empty().ins(1);   
        assertEquals("{1}",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INS,A PARTIRE DA LSET VUOTO , LVar INIZIALIZZATA  
	public void testToStringInitializedWithInsLVarInitialized() {
		LVar x1 = new LVar("x1",1);   
		LSet x = LSet.empty().ins(x1);
        assertEquals("{1}",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO E LVar NON INIZIALIZZATO 
	public void testToStringInitializedWithInsLVarUninitialized() {
		LVar x1 = new LVar("x1");   
		LSet x = LSet.empty().ins(x1);
        assertEquals("{_x1}",(x.toString()));
	}
			
	@Test //INIZ COSTRUITO CON INS, A PARTIRE DA VUOTO E LVar INIZ, LVar NON INIZ E INTERO 
	public void testToStringInitializedWithInsInteger_LVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LSet x = LSet.empty().ins(1).ins(x1).ins(x2);
        assertEquals("{_x2,2,1}",(x.toString()));
	}
			
	@Test //INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E INTERO
	public void testToStringUninitialized_insInteger() {
		LSet r = new LSet("r");
		LSet x = r.ins(1);
		assertEquals("{1/_r}",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INS, LSET NON INIZIALIZZATO E LVar NON INIZIALIZZATO
	public void testToStringUninitialized_insLVarUninitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1");
		LSet x = r.ins(x1);
		assertEquals("{_x1/_r}",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E LVar INIZIALIZZATO  
	public void testToStringUninitialized_insLVarInitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1);
		assertEquals("{1/_r}",(x.toString()));
	}
			
	@Test // INIZ COSTRUITO CON INS, LSET NON INIZ INTERO, LVar NON INIZ, LVar INIZ 
	public void testToStringUninitialized_insInteger_LVarUninitialized_LVarInitialized() {
		LSet r = new LSet("r");
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1).ins(x0).ins(2);
		assertEquals("{2,_x0,1/_r}",(x.toString()));
	}
			
	@Test // INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA VUOTO e ARRAY  
	public void testToStringInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		LSet x = LSet.empty().insAll(x1);   
        assertEquals("{1,2,3}",(x.toString()));
	}
			
	@Test //INIZIALIZZATO COSTRUITO CON INSALL,A PARTIRE DA VUOTO VECTOR  
	public void testToStringInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = LSet.empty().insAll(x1);
        assertEquals("{1,2,3}",(x.toString()));
	}
			
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e ARRAY 
	public void testToStringUnitializedWithInsAllRest_Array() {
		LSet r= new LSet ();
		Integer [] x1={1,2,3};
		LSet x = r.insAll(x1);   
       	assertEquals("{1,2,3/_?}",(x.toString()));
	}

	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e VECTOR 
	public void testToStringUnitializedWithInsAllRest_Vector() {
		LSet r= new LSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = r.insAll(x1);   
        assertEquals("{1,2,3/_?}",(x.toString()));
	}
			
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET INIZIALIZZATO VECTOR  
	public void testToStringInitializedWithInsAllLSetInitialized_Vector() {
		LSet r= LSet.empty().ins(5);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = r.insAll(x1);
        assertEquals("{1,2,3,5}",(x.toString()));
	}

	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET  inizializzato e ARRAY 
	public void testToStringInitializedWithInsAllLSetInitialized_Array() {
		LSet r= LSet.empty().ins(5);
		Integer [] x1={1,2,3};
		LSet x= r.insAll(x1);   
        assertEquals("{1,2,3,5}",(x.toString()));
	}

	
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET VUOTO  
	public void testToStringInitializedWithNewLSetEmpty() {
			LSet x1=LSet.empty();
			LSet x= new LSet(x1);
			assertEquals("{}",(x.toString()));
	}
	
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET CON ELEMENTI INSERITI TRAMITE INS
	public void testToStringInitializedWithNewLSetIns() {
			LSet x1 = LSet.empty().ins(1); 
			LSet x= new LSet(x1);
			assertEquals("{1}",(x.toString()));
	}
	
	//LSET INIZ COSTRUITO CON COSTRUTTORI, LSET NON INIZIALIZZATO
	public void testToStringInitializedWithNewLSetCostruttoriUninitialized() {
			LSet x1= new LSet();
			LSet x= new LSet(x1);
			assertEquals("_?",(x.toString()));
	}
				
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET VUOTO  
	public void testToStringInitializedWithSetEmpty() {
			Set x1 = new HashSet();
			LSet x= new LSet(x1);
			assertEquals("{}",(x.toString()));
	}
	
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET CON ELEMENTI INSERITI TRAMITE ADD
	public void testToStringInitializedWithSetAdd() {
			Set x1 = new HashSet();
			x1.add(1);x1.add(2); 
			LSet x= new LSet(x1);
			assertEquals("{1,2}",(x.toString()));
	}
		
	//%%%%%%%%%%%%%%%%%%%%%		
    //         NORMALIZE	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testNormalizedInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertEquals(x,(x.normalizeSet()));
	}
	
	//NON INIZIALIZZATO MODO DIRETTO
	@Test
	public void testNormalizedUninitialized() {
		LSet x = new LSet();
		assertEquals(x,(x.normalizeSet()));
	}
		
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO , INTERO  
	public void testNormalizedInitializedWithInsEmpty() {
		LSet x = LSet.empty().ins(1).ins(2).ins(1);
		LSet y = LSet.empty().ins(2).ins(1).setName("y");
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
	
	@Test // INIZIALIZZATO COSTRUITO CON INS,A PARTIRE DA LSET VUOTO , LVar INIZIALIZZATA  
	public void testNormalizedInitializedWithInsLVarInitialized() {
		LVar x1 = new LVar("x1",1);   
		LSet x = LSet.empty().ins(x1).ins(2).ins(1);
		LSet y = LSet.empty().ins(2).ins(1).setName("y");
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET VUOTO E LVar NON INIZIALIZZATO  
	public void testNormalizedInitializedWithInsLVarUninitialized() {
		LVar x1 = new LVar("x1");   
		LSet x = LSet.empty().ins(x1).ins(x1);
		LSet y = LSet.empty().ins(x1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test //INIZ COSTRUITO CON INS, A PARTIRE DA VUOTO E LVar INIZ, LVar NON INIZ E INTERO  
	public void testNormalizedInitializedWithInsInteger_LVarInitialized_LVarUninitialized() {
		LVar x1 = new LVar("x1",2);
		LVar x2 = new LVar("x2");   
		LSet x = LSet.empty().ins(2).ins(x1).ins(x2);
		LSet y = LSet.empty().ins(x1).ins(x2);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test //INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E INTERO
	public void testNormalizedUninitialized_insInteger() {
		LSet r = new LSet("r");
		LSet x = r.ins(1).ins(1).ins(2);
		LSet y = r.ins(1).ins(2);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test // INIZIALIZZATO COSTRUITO CON INS, LSET NON INIZIALIZZATO E LVar NON INIZIALIZZATO
	public void testNormalizedUninitialized_insLVarUninitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1");
		LSet x = r.ins(x1).ins(x1);
		LSet y= r.ins(x1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
    }
		
	@Test // INIZIALIZZATO COSTRUITO CON INS, A PARTIRE DA LSET NON INIZIALIZZATO E LVar INIZIALIZZATO 
	public void testNormalizedUninitialized_insLVarInitialized() {
		LSet r = new LSet("r");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1).ins(1);
		LSet y = r.ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test // INIZ COSTRUITO CON INS, LSET NON INIZ INTERO, LVar NON INIZ, LVar INIZ 
	public void testNormalizedUninitialized_insInteger_LVarUninitialized_LVarInitialized() {
		LSet r = new LSet("r");
		LVar x0 = new LVar("x0");
		LVar x1 = new LVar("x1",1);
		LSet x = r.ins(x1).ins(x0).ins(1);
		LSet y = r.ins(x0).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
				
	@Test // INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA VUOTO e ARRAY  
	public void testNormalizeInitializedWithInsAllEmpty_Array() {
		Integer [] x1={1,2,3};
		LSet x = LSet.empty().insAll(x1).ins(1);
		LSet y = LSet.empty().ins(3).ins(2).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test //INIZIALIZZATO COSTRUITO CON INSALL,A PARTIRE DA VUOTO VECTOR  
	public void testNormalizeInitializedWithInsAllEmpty_Vector() {
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = LSet.empty().insAll(x1).ins(1);
		LSet y = LSet.empty().ins(3).ins(2).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e ARRAY  
	public void testNormalizeUnitializedWithInsAllRest_Array() {
		LSet r= new LSet ();
		Integer [] x1={1,2,3};
		LSet x = r.insAll(x1).ins(2);
		LSet y = r.ins(3).ins(1).ins(2);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}

	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET non inizializzato e VECTOR  
	public void testNormalizeUnitializedWithInsAllRest_Vector() {
		LSet r= new LSet ();
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(1); x1.add(2);x1.add(3);
		LSet x = r.insAll(x1).ins(1);   
		LSet y = r.ins(3).ins(2).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
		
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET INIZIALIZZATO VECTOR  
	public void testNormalizedInitializedWithInsAll_Vector() {
		LSet r= LSet.empty().ins(1).ins(2).ins(1);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		LSet x = r.insAll(x1);
		LSet y = LSet.empty().ins(2).ins(1).ins(5).ins(4).ins(3);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
    }
	
	@Test //INIZIALIZZATO COSTRUITO CON INSALL, A PARTIRE DA LSET  inizializzato e ARRAY 
	public void testNormalizedInitializedWithInsAll_Array() {
		LSet r= LSet.empty().ins(1).ins(2).ins(1);
		ArrayList<Integer> x1 = new ArrayList<>();
		x1.add(3); x1.add(4);x1.add(5);
		LSet x = r.insAll(x1);   
		LSet y = LSet.empty().ins(2).ins(1).ins(5).ins(4).ins(3);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}		
			
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET VUOTO  
	public void testNormalizeInitializedWithNewLSetEmpty() {
		LSet x1=LSet.empty();
		LSet x= new LSet(x1);
		assertEquals(x,x.normalizeSet());
	}
	
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, LSET CON ELEMENTI INSERITI TRAMITE INS
	public void testNormalizeInitializedWithNewLSetIns() {
		LSet x1 = LSet.empty().ins(1).ins(2).ins(1); 
		LSet x= new LSet(x1);
		LSet y = LSet.empty().ins(2).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
	
	//LSET INIZ COSTRUITO CON COSTRUTTORI, LSET NON INIZIALIZZATO
	public void testNormalizeInitializedWithNewLSetUninitialized() {
		LSet x1= new LSet();
		LSet x= new LSet(x1);
		assertEquals(x,x.normalizeSet());
	}
			
	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET VUOTO 
	public void testNormalizeInitializedWithSetEmpty() {
		Set x1 = new HashSet();
		LSet x= new LSet(x1);
		assertEquals(x,x.normalizeSet());
	}

	@Test //LSET INIZ COSTRUITO CON COSTRUTTORI, SET CON ELEMENTI INSERITI TRAMITE ADD
	public void testNormalizeInitializedWithSetAdd() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2); x1.add(2);
		LSet x= new LSet(x1);
		LSet y = LSet.empty().ins(2).ins(1);
		assertEquals(y.getValue(),(x.normalizeSet().getValue()));
	}
			

	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         isBound	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testisBoundInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertTrue(x.isBound());
	}
				
	@Test //LSET NON INIZIALIZZATO
	public void testisBoundUnitialized() {
		LSet x = new LSet(); 
		assertFalse(x.isBound());
	}
				
	@Test //LSET INIZIALIZZATO con INS A PARTIRE DA LSET CLOSED
	public void testisBoundIninializedWithInsClosed() {
		LSet x = LSet.empty().ins(1).ins(2); 
		assertTrue(x.isBound());
	}
	
	@Test //LSET INIZIALIZZATO con INS A PARTIRE DA LSET OPEN
	public void testisBoundIninializedWithInsOpen() {
		LSet r= new LSet();
		LSet x = r.ins(1).ins(2); 
		assertTrue(x.isBound());
	}
				

	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET CLOSED
	public void testisBoundIninializedWithConstructLSetClosed() {
		LSet x1 = LSet.empty().ins(1).ins(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isBound());
	}
				
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET OPEN
		public void testisBoundIninializedWithConstructLSetOpen() {
		LSet r = new LSet("r");
		LSet x1 = r.ins(1).ins(2);
		LSet x = new LSet(x1);
		assertTrue(x.isBound());
	}
		

	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN SET CLOSED
	public void testisBoundIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isBound());
	}
	
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         GETREST	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testgetRestInitializedEmpty() {
			LSet x = LSet.empty().setName("x"); 
			assertEquals(x,(x.getTail()));
	}
			
	@Test //LSET NON INIZIALIZZATO
	public void testgetRestUnitialized() {
			LSet x = new LSet(); 
			assertEquals(x,(x.getTail()));
	}
			
	@Test //LSET INIZIALIZZATO con INS LSET CLOSED
	public void testgetRestIninializedWithInsClosed() {
			LSet x = LSet.empty().ins(1).ins(2); 
			LSet y = LSet.empty();
			assertEquals(y,(x.getTail()));
	}

	@Test //LSET INIZIALIZZATO con INS LSET OPEN
	public void testgetRestIninializedWithInsOpen() {
			LSet r= new LSet();
			LSet x = r.ins(1).ins(2); 
			assertEquals(r,(x.getTail()));
	}
			
	
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN  LSET CLOSED
	public void testgetRestIninializedWithConstructLSetClosed() {
			LSet x1 = LSet.empty().ins(1).ins(2);
			LSet x = new LSet(x1); 
			LSet y = LSet.empty();
			assertEquals(y,(x.getTail()));
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN  LSET OPEN
	public void testgetRestIninializedWithConstructLSetOpen() {
			LSet r = new LSet("r");
			LSet x1 = r.ins(1).ins(2);
			LSet x = new LSet(x1);
			assertEquals(r,(x.getTail()));
	}
	
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN SET CLOSED
	public void testgetRestIninializedWithConstructSetClosed() {
			Set x1 = new HashSet();
			x1.add(1);x1.add(2);
			LSet x = new LSet(x1);
			LSet y = LSet.empty();
			assertEquals(y,(x.getTail()));
	}
	
			
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISCLOSED	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsClosedInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertTrue(x.isClosed());
	}
			
	@Test //LSET NON INIZIALIZZATO
	public void testIsClosedUnitialized() {
		LSet x = new LSet(); 
		assertFalse(x.isClosed());
	}
			
	@Test //LSET INIZIALIZZATO con INS LSET CLOSED
	public void testIsClosedIninializedWithInsClosed() {
		LSet x = LSet.empty().ins(1).ins(2); 
		assertTrue(x.isClosed());
	}

	@Test //LSET INIZIALIZZATO con INS LSET OPEN
	public void testIsClosedIninializedWithInsOpen() {
		LSet r= new LSet();
		LSet x = r.ins(1).ins(2); 
		assertFalse(x.isClosed());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET CLOSED
	public void testIsClosedIninializedWithConstructLSetClosed() {
		LSet x1 = LSet.empty().ins(1).ins(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isClosed());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET OPEN
	public void testIsClosedIninializedWithConstructLSetOpen() {
		LSet r = new LSet("r");
		LSet x1 = r.ins(1).ins(2);
		LSet x = new LSet(x1);
		assertFalse(x.isClosed());
	}
				
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN SET CLOSED
	public void testIsClosedIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isClosed());
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISEMPTY	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsEmptyInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertTrue(x.isEmpty());
	}
			
	@Test //LSET NON INIZIALIZZATO
	public void testIsEmptyUnitialized() {
		LSet x = new LSet(); 
		try{
			 x.isEmpty();
			 fail("Eccezione isEmpty con lset non inizializzato");
		 }
		 catch(NotInitLObjectException e){}
		
	}
		
	
	@Test //LSET INIZIALIZZATO con INS LSET CLOSED
	public void testIsEmptyIninializedWithInsClosed() {
		LSet x = LSet.empty().ins(1).ins(2); 
		assertFalse(x.isEmpty());
	}
	
	@Test //LSET INIZIALIZZATO con INS LSET OPEN
	public void testIsEmptyIninializedWithInsOpen() {
		LSet r= new LSet();
		LSet x = r.ins(1).ins(2); 
		assertFalse(x.isEmpty());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET CLOSED
	public void testIsEmptyIninializedWithConstructLSetClosed() {
		LSet x1 = LSet.empty().ins(1).ins(2);
		LSet x = new LSet(x1); 
		assertFalse(x.isEmpty());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET OPEN
	public void testIsEmptyIninializedWithConstructLSetOpen() {
		LSet r = new LSet("r");
		LSet x1 = r.ins(1).ins(2);
		LSet x = new LSet(x1);
		assertFalse(x.isEmpty());
	}
				
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UNSET CLOSED
	public void testIsEmptyIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		LSet x = new LSet(x1); 
		assertFalse(x.isEmpty());
	}
	
	//%%%%%%%%%%%%%%%%%%%%%		
    //         ISGROUND	
	//%%%%%%%%%%%%%%%%%%%%
	@Test //VUOTO
	public void testIsGroundInitializedEmpty() {
		LSet x = LSet.empty().setName("x"); 
		assertTrue(x.isGround());
	}
			
	@Test //LSET NON INIZIALIZZATO
	public void testIsGroundUnitialized() {
		LSet x = new LSet();
		assertFalse(x.isGround());
	}
			
	@Test //LSET INIZIALIZZATO con INS LSET CLOSED
	public void testIsGroundIninializedWithInsClosed() {
		LSet x = LSet.empty().ins(1).ins(2); 
		assertTrue(x.isGround());
	}
	
	@Test //LSET INIZIALIZZATO con INS LSET OPEN
	public void testIsGroundIninializedWithInsOpen() {
		LSet r= new LSet();
		LSet x = r.ins(1).ins(2); 
		assertFalse(x.isGround());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN LSET CLOSED
	public void testIsGroundIninializedWithConstructLSetClosed() {
		LSet x1 = LSet.empty().ins(1).ins(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isGround());
	}
			
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN  LSET OPEN
	public void testIsGroundIninializedWithConstructLSetOpen() {
		LSet r = new LSet("r");
		LSet x1 = r.ins(1).ins(2);
		LSet x = new LSet(x1);
		assertFalse(x.isGround());
	}
	
	@Test //LSET INIZIALIZZATO con COSTRUTTORI A CUI E' PASSATO UN SET CLOSED
	public void testIsGroundIninializedWithConstructSetClosed() {
		Set x1 = new HashSet();
		x1.add(1);x1.add(2);
		LSet x = new LSet(x1); 
		assertTrue(x.isGround());
	}			

	//%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(lset)	
	//%%%%%%%%%%%%%%%%%%%%
	@Test // VUOTO --> VUOTO
	public void testEqualsEmpty_LSetEmpty() {
		LSet x = LSet.empty();
		LSet y = LSet.empty();
		assertTrue(x.equals(y));
	}
	
	@Test // VUOTO --> LSET NON INIZIALIZZATO
	public void testEqualsEmpty_LSetUninitilized() {
		LSet x = LSet.empty();
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
	
	@Test // VUOTO --> LSET VUOTO, INS INTERO 
	public void testEqualsEmpty_LSetEmptyInsInteger() {
		LSet x = LSet.empty();
		LSet y = LSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LSET VUOTO, INS LVar UNINITIALIZZATO
	public void testEqualsEmpty_LSetEmptyInsLVarUninitialized() {
		LSet x = LSet.empty();
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LSET VUOTO, INS LVar INITIALIZZATO
	public void testEqualsEmpty_LSetEmptyInsLVarInitialized() {
		LSet x = LSet.empty();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test // VUOTO --> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsEmpty_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LSet x = LSet.empty();
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test // VUOTO --> RESTO, INS INTERO
	public void testEqualsEmpty_LSetEmptyRestInsInteger() {
		LSet x = LSet.empty();
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
			
	@Test // LSET NON INIZIALIZZATO --> VUOTO
	public void testEqualsLSetUninitilized_LSetEmpty() {
		LSet x = new LSet();
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}

	@Test // LSET NON INIZIALIZZATO --> STESSO LSET NON INIZIALIZZATO
	public void testEqualsLSetUninitilized_SameLSetUninitilized() {
		LSet x = new LSet();
		assertTrue(x.equals(x));
	}
	
	@Test // LSET NON INIZIALIZZATO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetUninitilized_LSetUninitilized() {
		LSet x = new LSet();
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET NON INIZIALIZZATO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetUninitilized_LSetEmptyInsInteger() {
		LSet x = new LSet();
		LSet y = LSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET NON INIZIALIZZATO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetUninitilized_LSetEmptyInsLVarUninitialized() {
		LSet x = new LSet();
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET NON INIZIALIZZATO--> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetUninitilized_LSetEmptyInsLVarInitialized() {
		LSet x = new LSet();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET NON INIZIALIZZATO --> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetUninitilized_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LSet x = new LSet();
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLSetUninitilized_LSetEmptyRestInsInteger() {
		LSet x = new LSet();
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				

	@Test //LSET VUOTO INS INTERO --> VUOTO
	public void testEqualsLSetEmptyInsInteger_LSetEmpty() {
		LSet x = LSet.empty().ins(1);
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET  VUOTO INS INTERO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetEmptyInsInteger_LSetUninitilized() {
		LSet x = LSet.empty().ins(1);
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS INTERO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetEmptyInsInteger_LSetEmptyInsInteger() {
		LSet x = LSet.empty().ins(1);
		LSet y = LSet.empty().ins(1);
		assertTrue(x.equals(y));
	}
				
	@Test //LSET VUOTO INS INTERO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyInsInteger_LSetEmptyInsLVarUninitialized() {
		LSet x = LSet.empty().ins(1);
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
			
	@Test //LSET VUOTO  INS INTERO--> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetEmptyInsInteger_LSetEmptyInsLVarInitialized() {
		LSet x = LSet.empty().ins(1);
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
				
	@Test //LSET VUOTO INS INTERO --> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetEmptyInsInteger_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LSet x = LSet.empty().ins(1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				

	@Test //LSET VUOTO INS INTERO  --> RESTO, INS INTERO
	public void testEqualsLSetEmptyInsInteger_LSetEmptyRestInsInteger() {
		LSet x = LSet.empty().ins(1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO--> VUOTO
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmpty() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetUninitilized() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LSet y = new LSet();
		assertFalse(x.equals(y));
   }
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmptyInsInteger() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LSet y = LSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmptyInsLVarUninitialized() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO--> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmptyInsLVarInitialized() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO--> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar NON INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLSetEmptyInsLVarUninitialized_LSetEmptyRestInsInteger() {
		LVar x1 = new LVar();
		LSet x = LSet.empty().ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}				

	@Test //LSET VUOTO INS LVar INIZIALIZZATO--> VUOTO
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmpty() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar INIZIALIZZATO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetEmptyInsLVarInitialized_LSetUninitilized() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar INIZIALIZZATO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmptyInsInteger() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LSet y = LSet.empty().ins(1);
		assertTrue(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar INIZIALIZZATO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmptyInsLVarUninitialized() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar  INIZIALIZZATO--> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmptyInsLVarInitialized() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertTrue(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar  INIZIALIZZATO--> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LVar y0= new LVar();
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(1).ins(y0).ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO INS LVar  INIZIALIZZATO  --> RESTO, INS INTERO
	public void testEqualsLSetEmptyInsLVarInitialized_LSetEmptyRestInsInteger() {
		LVar x1 = new LVar("X1",1);
		LSet x = LSet.empty().ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}	
				
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO--> VUOTO
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmpty() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet y = LSet.empty();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetUninitilized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet y = new LSet();
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmptyInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet y = LSet.empty().ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmptyInsLVarUninitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LVar y1= new LVar(); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO --> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmptyInsLVarInitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LVar y1= new LVar("y1",1); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO--> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x0= new LVar("x0");
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LVar y0= new LVar("y0");
		LVar y1= new LVar("y1",2); 
		LSet y = LSet.empty().ins(y1);
		assertFalse(x.equals(y));
	}
				
	@Test //LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO  --> RESTO, INS INTERO
	public void testEqualsLSetEmptyInsLVarInitializedLVarUninitializedInteger_LSetEmptyRestInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",1); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}	

	@Test //RESTO, INS INTERO--> VUOTO
	public void testEqualsLSetEmptyRestInsInteger_LSetEmpty() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LSET NON INIZIALIZZATO
	public void testEqualsLSetEmptyRestInsInteger_LSetUninitilized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LSET VUOTO, INS INTERO 
	public void testEqualsLSetEmptyRestInsInteger_LSetEmptyInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LSET VUOTO, INS LVar UNINITIALIZEC
	public void testEqualsLSetEmptyRestInsInteger_LSetEmptyInsLVarUninitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
	@Test //RESTO, INS INTERO --> LSET VUOTO, INS LVar INITIALIZEC
	public void testEqualsLSetEmptyRestInsInteger_LSetEmptyInsLVarInitialized() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				
				
	@Test //RESTO, INS INTERO--> LSET VUOTO, INS LVar INITIALIZATO, LVar NON INIZIALIZZATO, INTERO
	public void testEqualsLSetEmptyRestInsInteger_LSetEmptyInsLVarInitializedLVarUninitializedInteger() {
		LVar x0= new LVar("x0");
		LVar x1= new LVar("x1",2); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}
				

	@Test //RESTO, INS INTERO  --> RESTO, INS INTERO
	public void testEqualsLSetEmptyRestInsInteger_LSetEmptyRestInsInteger() {
		LVar x0= new LVar();
		LVar x1= new LVar("x1",1); 
		LSet x = LSet.empty().ins(1).ins(x0).ins(x1);
		LSet r= new LSet("r");
		LSet y = r.ins(1);
		assertFalse(x.equals(y));
	}

	//%%%%%%%%%%%%%%%%%%%%%%%%%		
    //         EQUALS(object)	
	//%%%%%%%%%%%%%%%%%%%%%%%%
	
	// LSET NON INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetUninitialized_HashSetUninitialized() {
		LSet x = new LSet();
		Set s = new HashSet();
        assertFalse(x.equals(s));
	}
	
	// LSET NON INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetUninitialized_HashSetInitialized() {
		LSet x = new LSet();
		Set s = new HashSet();
		s.add(2);s.add(3);
        assertFalse(x.equals(s));
	}
	
	// LSET NON INIZIALIZZATO ---> INTERO
	@Test
	public void testEqualsObjectLSetUninitialized_Integer() {
		LSet x = new LSet();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LSET NON INIZIALIZZATO ---> ARRAY
	@Test
	public void testEqualsObjectLSetUninitialized_Array() {
		LSet x = new LSet();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	

	// LSET VUOTO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetEmpty_HashSetUninitialized() {
		LSet x = LSet.empty();
		Set s = new HashSet();
        assertTrue(x.equals(s));
	}
	
	// LSET VUOTO ---> SET INIZIALIZZATO
	public void testEqualsObjectLSetEmpty_HashSetInitialized() {
		LSet x = LSet.empty();
		Set s = new HashSet();
		s.add(2);s.add(3);
		assertFalse(x.equals(s));
	}
	
	// LSET VUOTO ---> INTERO
	public void testEqualsObjectLSetEmpty_Integer() {
		LSet x = LSet.empty();
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LSET VUOTO ---> ARRAY
	public void testEqualsObjectLSetEmpty_Array() {
		LSet x = LSet.empty();
		Integer[] y= {4,3};
		assertFalse(x.equals(y));
	}
	
	// LSET VUOTO INS INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsInteger_HashSetUninitialized() {
		LSet x = LSet.empty().ins(1).ins(2);
		Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// LSET VUOTO INS INTERO ---> SET INIZIALIZZATO UGUALE
	public void testEqualsObjectLSetInitializedWithInsInteger_HashSetInitializedSame() {
		LSet x = LSet.empty().ins(1).ins(2);
		Set s = new HashSet();
		s.add(1);s.add(2);
        assertTrue(x.equals(s));
	}
	
	// LSET VUOTO INS INTERO---> INTERO
	public void testEqualsObjectLSetInitializedWithInsInteger_Integer() {
		LSet x = LSet.empty().ins(1).ins(2);
		Integer y=4;
		assertFalse(x.equals(y));
	}
	

	// LSET VUOTO INS INTERO ---> ARRAY 
	public void testEqualsObjectLSetInitializedWithInsInteger_ArraySame() {
		LSet x = LSet.empty().ins(1).ins(2);
		Integer[] y= {1,2};
		assertFalse(x.equals(y));
	}

	// LSET INIZIALIZZATO INS con LVar INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithLVarInsInitialized_HashSetUninitialized() {
		LVar x1 = new LVar("x1",1);
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	// LSET INIZIALIZZATO INS con LVar INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithLVarInsInitialized_HashSetInitialized() {
		LVar x1 = new LVar("x1",1);
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar INIZIALIZZATO---> INTERO
	public void testEqualsObjectLSetInitializedWithInsLVarInitialized_Integer() {
		LVar x1 = new LVar("x1",1);
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// LSET INIZIALIZZATO INS con LVar INIZIALIZZATO---> ARRAY UGUALE
	public void testEqualsObjectLSetInitializedWithInsLVarInitialized_ArraySame() {
		LVar x1 = new LVar("x1",1);
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsLVarUnitialized_HashSetUninitialized() {
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsLVarUnitialized_HashSetInitialized() {
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO---> INTERO
	public void testEqualsObjectLSetInitializedWithInsLVarUnitialized_Integer() {
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO---> ARRAY UGUALE
	public void testEqualsObjectLSetInitializedWithInsLVarUnitialized_ArraySame() {
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsLVarUnitializedLVarInitializedInteger_HashSetUninitialized() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetInitializedWithInsLVarUnitializedLVarInitializedInteger_HashSetInitialized() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	     Set s = new HashSet();
	    s.add(1);
		assertFalse(x.equals(s));
	}
	
	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO---> INTERO
	public void testEqualsObjectLSetInitializedWithInsLVarUnitializedLVarInitializedInteger_Integer() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// LSET INIZIALIZZATO INS con LVar NON INIZIALIZZATO, LVar INIZIALIZZATO, INTERO---> ARRAY UGUALE
	public void testEqualsObjectLSetInitializedWithInsLVarUnitializedLVarInitializedInteger_ArraySame() {
		LVar x0 = new LVar("x0",2);
		LVar x1 = new LVar("x1");
	    LSet x = LSet.empty().ins(x0).ins(x1).ins(1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}
	
	// LSET NON INIZIALIZZATO INS INTERO ---> SET NON INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetUninitializedInsInteger_HashSetUninitialized() {
		LSet r = new LSet ("r");
		LSet x = r.ins(1).setName("x");
	    Set s = new HashSet();
		assertFalse(x.equals(s));
	}
	
	// LSET NON INIZIALIZZATO INS INTERO ---> SET INIZIALIZZATO
	@Test
	public void testEqualsObjectLSetUninitializedInsInteger_HashSetInitialized() {
		LSet r = new LSet ("r");
		LSet x = r.ins(1).setName("x");
	    Set s = new HashSet();
	    s.add(1);
		assertTrue(x.equals(s));
	}
	
	// LSET NON INIZIALIZZATO INS INTERO---> INTERO
	public void testEqualsObjectLSetUninitializedInsInteger_Integer() {
		LSet r = new LSet ("r");
		LSet x = r.ins(1).setName("x");
	    Integer y=4;
	    assertFalse(x.equals(y));
	}
	

	// LSET NON INIZIALIZZATO INS INTERO ---> ARRAY 
	public void testEqualsObjectLSetUninitializedInsInteger_Array() {
		LSet r = new LSet ("r");
		LSet x = r.ins(1).setName("x");
	    Integer[] y= {1,2};
	    assertFalse(x.equals(y));
	}

	public void testToString01(){
		LSet lSet = new LSet("R").ins(1,2,3,new LVar("x"),4);
		assertEquals("{1,2,3,_x,4/_R}", lSet.toString());
	}
		
}