package jsetl.test;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;

import jsetl.exception.Failure;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import jsetl.*;

import junit.framework.TestCase;

// Author: Michele Giacobini

public class ORTest extends TestCase{
	
	@Before
	public void setUp() {}
	
	@After
	public void tearDown() {}
	
	@Test
	public void testORConstraint1() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");
		solver.add(x.eq(1).or(x.eq(1)).and(x.neq(1))); // (x=1 OR x=1) AND x neq 1
/*		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore(); 
		assertTrue("Store: _x = 1 OR _x = 1 AND _x neq 1".equals(newStream.toString().trim()));
*/
        String CS = solver.getConstraint().toString();   //NEW
        assertTrue("_x = 1 OR _x = 1 AND _x neq 1".equals(CS));		
//      assertEquals(false, solver.check());
        assertFalse(solver.check());
//		System.out.flush();
//		System.setOut(old);
		//System.out.println("Check: " + solver.check());
	}

	@Test
	public void testORConstraint2() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar x = new IntLVar("x");       
        solver.add(x.eq(1).or(x.eq(2)));  // x=1 OR x=2
        solver.solve();
        assertTrue(x.getValue().equals(1));
        solver.nextSolution();
        assertTrue(x.getValue().equals(2));        
	}	

// Prova test constraint store
	
	@Test
	public void testANDConstraintPROVA() throws Failure, IOException{
		Solver solver = new Solver();
		LVar x = new LVar("x");       
        solver.add(x.neq(1).and(x.neq(2)));  // x neq 1 AND x neq 2
        solver.solve();
		Constraint c = new Constraint();
		c.add(x.neq(1));
		c.add(x.neq(2));		
        assertEquals(solver.getConstraint(),c);	
//OPPURE
		Constraint c1 = x.neq(1).and(x.neq(2));		
        assertEquals(solver.getConstraint(),c1);		
	}	
	
	
	public void testEqOrEqAndNeqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar xx = new IntLVar("xx");
		solver.add(xx.eq(1).or(xx.eq(2)).and(xx.neq(3)));
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _xx = 1 OR _xx = 2 AND _xx neq 3".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println("Check: " + solver.check());
	}
	
	@Test
	public void testEqOrEqAndEqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar y = new IntLVar("y");
		solver.add(y.eq(1).or(y.eq(2)).and(y.eq(2)));
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _y = 1 OR _y = 2 AND _y = 2".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println(solver.check());
		//solver.solve();
		//solver.showStoreAll();
	}
	
	@Test
	public void testEqOrEqAndEqCheckFalse() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar yy = new IntLVar("yy");
		solver.add(yy.eq(2).or(yy.eq(2)).and(yy.eq(1)));
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _yy = 2 OR _yy = 2 AND _yy = 1".equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println(solver.check());
		//solver.solve();
		//solver.showStoreAll();
	}
	
	@Test
	public void testEqOrNeqAndEqCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar v = new IntLVar("v");
		solver.add(v.eq(1).or(v.neq(2)).and(v.eq(3)));
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream (newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _v = 1 OR _v neq 2 AND _v = 3".equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		//solver.showStore();
		//System.out.println("Check: " + solver.check());
	}
	
	@Test
	public void testEqOrNeqAndEqCheckFalse() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar vv = new IntLVar("vv");
		solver.add(vv.eq(1).or(vv.neq(2)).and(vv.eq(2)));
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream (newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _vv = 1 OR _vv neq 2 AND _vv = 2".equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
		//solver.showStore();
		//System.out.println("Check: " + solver.check());
	}
	
	@Test
	public void testOrTestCheckTrue() throws Failure, IOException{
		Solver solver = new Solver();
		IntLVar w = new IntLVar("w");
		solver.add(w.eq(1).orTest(w.eq(2)).and(w.neq(1)).and(w.neq(3)));
		//System.out.println("Chech : " + solver.check());
		//solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _w = 1 OR _w = 2 AND _w neq 1 AND _w neq 3".trim().equals(newStream.toString().trim()));
		assertEquals(true, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println(solver.check());
		//solver.showStoreAll();
	}

	/*@Test
	public void testOrTestCheckFalse() throws Failure, IOException{
		solver solver = new solver();
		IntLVar ww = new IntLVar("ww");
		solver.add(ww.eq(2).orTest((ww.neq(2)).and(ww.eq(2))));
		//System.out.println("Chech : " + solver.check());
		solver.showStore();
		ByteArrayOutputStream newStream = new ByteArrayOutputStream();
		PrintStream constraintStore = new PrintStream(newStream);
		PrintStream old = System.out;
		System.setOut(constraintStore);
		solver.showStore();
		assertTrue("Store: _ww = 2 OR _ww neq 2 AND _ww = 2".trim().equals(newStream.toString().trim()));
		assertEquals(false, solver.check());
		System.out.flush();
		System.setOut(old);
		//System.out.println(solver.check());
		//solver.showStoreAll();
	}*/
}
