package jsetl.test;

import java.util.ArrayList;

import jsetl.*;

import org.junit.Test;
import junit.framework.TestCase;

public class PartiallySpecifiedFSTest extends TestCase {

	@Test
	public void
	testPermutationsOfTwoElements() {
		/* {X, Y} = {1, 2} */
		SetLVar S = new SetLVar("S");
    	IntLVar X = new IntLVar("X");
    	IntLVar Y = new IntLVar("Y");
    	IntLVar[] s = {X, Y};
    	Solver solver = new Solver();
    	
    	solver.add(S.eq(new MultiInterval(1,2)));
    	solver.add(S.eq(s));
    	ArrayList<IntLVar> v = new ArrayList<IntLVar>();
    	v.add(X);
    	v.add(Y);
    	solver.add(S.eq(v));
    	solver.add(X.label());
    	assertTrue(solver.check());
    	
    	int i = 0;
        do {
        	++i;
        	assertEquals(X.getValue() + Y.getValue(), 3);
        } while(solver.nextSolution());
        assertEquals(i, 2);
	}
	
	@Test
	public void
	testPermutationsOfThreeElements() {
		/* {X1, X2, X3} = {1, 2, 3} */
		IntLVar X1 = new IntLVar("X1");
		IntLVar X2 = new IntLVar("X2");
		IntLVar X3 = new IntLVar("X3");
		IntLVar[] x = {X1, X2, X3};
		SetLVar A = new SetLVar();
		Solver solver = new Solver();
		
		solver.add(A.eq(x));
		solver.add(A.eq(new MultiInterval(1, 3)));
		solver.add(X1.label());
		solver.add(X2.label());
		assertTrue(solver.check());
		
		int i = 0;
		do {
			++i;
			assertEquals(X1.getValue() + X2.getValue() + X3.getValue(), 6);
		} while(solver.nextSolution());
		assertEquals(i, 6);
	}
	
	@Test
	public void
	testUnboundedSetsUnification1() {
        /* {1 | R1} = {2 | R2} */
        SetLVar T1 = new SetLVar("T1");
    	SetLVar T2 = new SetLVar("T2");
    	SetLVar R1 = new SetLVar("R1");
    	SetLVar R2 = new SetLVar("R2");
    	
    	// T1 eq T2 = {1} U R1 eq {2} U R2.
    	Solver solver = new Solver();
    	solver.add(T1.eq(new IntLVar(1), R1));
    	solver.add(T2.eq(new IntLVar(2), R2));
    	solver.add(T1.eq(T2));
    	solver.add(R1.dom(new MultiInterval(), new MultiInterval(1, 3)));
    	solver.add(R1.label());
    	solver.add(R2.label());
    	assertTrue(solver.check());
    	
    	int i = 0;
        do {
        	++i;
        	assertEquals(T1, T2);
        } while(solver.nextSolution());
        assertEquals(i, 8);
	}
        
	@Test
	public void
	testUnboundedSetsUnification2() {
		/* {1 | P1} = {Z | Q1} */
        SetLVar P = new SetLVar("P");
    	SetLVar Q = new SetLVar("Q");
        SetLVar P1 = new SetLVar("P1");
    	SetLVar Q1 = new SetLVar("Q1");
    	IntLVar Z = new IntLVar("Z");
    	Solver solver = new Solver();
    	
    	solver.add(P.eq(new IntLVar(1), P1));
    	solver.add(Q.eq(Z, Q1));
    	solver.add(P.eq(Q));	
    	// FIXME: If P1 domain is too large, the computation doesn't
    	// 		  terminate in reasonable time.
    	solver.add(P1.dom(new MultiInterval(), new MultiInterval(0, 1)));
    	solver.add(P1.label());
    	solver.add(Q1.label());
    	solver.add(Z.label());
    	assertTrue(solver.check());
    	
    	int i = 0;
        do {
        	++i;
        	MultiInterval tmp = new MultiInterval(0, 1);
        	assertTrue(tmp.contains(P1.getValue()));
        	assertTrue(tmp.contains(Z.getValue()));
        	assertTrue(tmp.contains(Q1.getValue()));
        } while(solver.nextSolution());
        assertEquals(i, 12);
	}
	
	@Test
	public void
	testBoundedSetsUnification2() {
        /* {1, N} = {M, 2} */
        SetLVar V = new SetLVar();
    	SetLVar W = new SetLVar();
    	IntLVar N = new IntLVar("N");
    	IntLVar M = new IntLVar("M");
    	ArrayList<IntLVar> l = new ArrayList<IntLVar>();
    	l.add(new IntLVar(1));
    	l.add(N);
    	IntLVar[] l1 = {M, new IntLVar(2)};
    	Solver solver = new Solver();
    	
    	solver.add(V.eq(l));
    	solver.add(W.eq(l1));
    	solver.add(V.eq(W));
    	assertTrue(solver.check());
    	
    	int i = 0;
        do {
        	++i;
        	assertEquals((int) N.getValue(), 2);
        	assertEquals((int) M.getValue(), 1);
        } while(solver.nextSolution());
        assertEquals(i, 1);
	}
         
	@Test
	public void
	testUnboundedSetInconsistency() {
        /* B = {Y1, Y2 | _}, #B < 2, Y1 neq Y2 */
        IntLVar Y1 = new IntLVar("Y1", 0, 10);
        IntLVar Y2 = new IntLVar("Y2");
        IntLVar[] y = {Y1, Y2};
        SetLVar B = new SetLVar();
        ArrayList<IntLVar> w = new ArrayList<IntLVar>();
    	w.add(Y1);
    	w.add(Y2);
        Solver solver = new Solver();
        
        solver.add(B.eq(y, new SetLVar()));
    	solver.add(B.eq(w, new SetLVar()));
        solver.add(B.card().lt(2));
        solver.add(Y1.neq(Y2));
        solver.add(Y1.label());
        assertFalse(solver.check());
	}
}
